
//
// Modifications History
//
//$ LB (13/06/2002)  : changed ObjBitmap management, according to the new ObjBitmap structure
//




#include "layer.h"
#include "utils.h"
#include "bitmap.h"


#define NO_TRANSPARENCY -1

#if DEBUG_LAYER
int Layer::ctr=0;
#endif

#if DEBUG_ALLOCATED	
int Layer::size_allocated=0;
int Layer::nb_obj=0;
#endif

extern mmachine mm;

Layer::Layer ( PtrObjBitmap rgb, PtrObjBitmap alpha, int transparency )
{

	this->RGBbitmap = rgb;
	this->AlphaBitmap = alpha;

	this->Transparency = transparency;
	if ( rgb != NULL )
		this->sourceRect = Rect2D( 0, 0, rgb->TailleW, rgb->TailleH );
	else
		this->sourceRect = Rect2D( 0, 0, 0, 0 );

	this->Xdecal = 0;
	this->Ydecal = 0;

	this->allocated = 0;

	this->Next = NULL;


#if DEBUG_LAYER
	MMechostr(MSKTRACE,"Layer:constructeur non alloue:nb Layer:%d\n",++ctr);
#endif
}










Layer::Layer ( int width, int height, int transparency, int withalpha )
{
	this->AlphaBitmap = NULL;
	this->Transparency=transparency;
	this->RGBbitmap=(PtrObjBitmap)malloc(sizeof(ObjBitmap));
	this->RGBbitmap->TailleW=width;
	this->RGBbitmap->TailleH=height;
	this->RGBbitmap->BPP=16;
	ObjBitmap_NewDIBSection(this->RGBbitmap,NULL);
#if DEBUG_ALLOCATED
	size_allocated+=this->RGBbitmap->TailleW*this->RGBbitmap->TailleH*this->RGBbitmap->BPP/8;
	nb_obj++;
#endif
	if (withalpha)
	{
		this->AlphaBitmap=(PtrObjBitmap)malloc(sizeof(ObjBitmap));
		this->AlphaBitmap->TailleW=width;
		this->AlphaBitmap->TailleH=height;
		this->AlphaBitmap->BPP=8;
		ObjBitmap_NewDIBSection(this->AlphaBitmap,NULL);
		this->fillAlphaLayer( 255 );
#if DEBUG_ALLOCATED
		size_allocated+=this->AlphaBitmap->TailleW*this->AlphaBitmap->TailleH*this->AlphaBitmap->BPP/8;
		nb_obj++;
#endif
	}

	this->sourceRect = Rect2D( 0, 0, width, height );
	this->allocated = 1;

	this->Xdecal = 0;
	this->Ydecal = 0;

	this->Next = NULL;

	if ( transparency != NO_TRANSPARENCY )
		this->fillLayer( Color24To16(transparency) ); 

#if DEBUG_ALLOCATED
	MMechostr(MSKTRACE,"Layer allocated nb:%d  size:%d\n",nb_obj,size_allocated);
#endif
#if DEBUG_LAYER
	MMechostr(MSKTRACE,"Layer:constructeur alloue:nb Layer:%d\n",++ctr);
#endif

}

// FIN BOURRIN: TEMPORAIRE


Layer::~Layer ()
{
	if ( this->allocated == 1 )
	{
		if ( this->RGBbitmap != NULL )
		{
#if DEBUG_ALLOCATED
	size_allocated-=this->RGBbitmap->TailleW*this->RGBbitmap->TailleH*this->RGBbitmap->BPP/8;
	nb_obj--;
#endif
			DeleteObject( this->RGBbitmap->DIBhandler);
			free( this->RGBbitmap );
		}
		if ( this->AlphaBitmap != NULL ) 
		{
#if DEBUG_ALLOCATED
		size_allocated-=this->AlphaBitmap->TailleW*this->AlphaBitmap->TailleH*this->AlphaBitmap->BPP/8;
		nb_obj--;
#endif
			DeleteObject( this->AlphaBitmap->DIBhandler);
			free( this->AlphaBitmap );
		}
#if DEBUG_ALLOCATED
	MMechostr(MSKTRACE,"Layer allocated nb:%d  size:%d\n",nb_obj,size_allocated);
#endif
#if DEBUG_LAYER
		MMechostr(MSKTRACE,"Layer:destructeur alloue:nb Layer:%d\n",--ctr);
#endif
	}
#if DEBUG_LAYER
	else
		MMechostr(MSKTRACE,"Layer:destructeur non alloue:nb Layer:%d\n",--ctr);
#endif

	this->allocated = 0;
}

void Layer::fillLayer ( int color )
{
	register int i, j;
	int dcx, dcy;
	int height, width;

	if ( this->RGBbitmap != NULL )
	{
		height = this->RGBbitmap->TailleH;
		width  = this->RGBbitmap->TailleW;

		dcy = 0;
		for ( j = 0 ; j < height ; j ++ )
		{
			dcx = dcy ;
			for ( i = 0 ; i < width ; i ++ , dcx ++ )
				this->RGBbitmap->bits[ dcx ] = color;
			dcy += this->RGBbitmap->BPL >> 1 ;
		}
	}
}

void Layer::fillPartLayer ( int color, Rect2D rect )
{
	register int i, j;
	int dcx, dcy;

	if ( this->RGBbitmap != NULL )
	{
		dcy = rect.RctHG.iptY * ( this->RGBbitmap->BPL >> 1 ) + rect.RctHG.iptX ;
		for ( j = rect.RctHG.iptY; j < rect.RctBD.iptY ; j ++ )
		{
			dcx = dcy ;
			for ( i = rect.RctHG.iptX; i < rect.RctBD.iptX ; i ++ , dcx ++ )
				this->RGBbitmap->bits[ dcx ] = color;
			dcy += this->RGBbitmap->BPL >> 1;
		}
	}
}

void GetBitmapPalette ( PtrObjBitmap B , PtrPalette P )
{
    RGBQUAD Pal [ 256 ] ;
    HDC dc ;
    int i ;
    HBITMAP OldBitmap ;

    dc = CreateCompatibleDC ( NULL ) ;
    OldBitmap = (HBITMAP)SelectObject ( dc , B->DIBhandler) ;
    i = GetDIBColorTable ( dc , 0 , 256 , Pal ) ;
    SelectObject ( dc , OldBitmap ) ;
    DeleteDC ( dc ) ;

    for ( i = 0 ; i < 256 ; i ++ )
    {
        GET_RED(P,i) = Pal[i].rgbRed ;
        GET_GREEN(P,i) = Pal[i].rgbGreen ;
        GET_BLUE(P,i) = Pal[i].rgbBlue ;
    
    }
}

void CopyObjBitmapToObjBitmap(PtrObjBitmap BD,int dposx,int dposy,PtrObjBitmap BS,int sposx,int sposy,int taillew,int tailleh,int couleur)
{
	register unsigned char * BufS , * BufD ;       
    register int j ;
    struct Palette Pal ;       
	int i , dcx , dcy , scx , scy ;

	if ( ClipBlit ( BD->TailleW , BD->TailleH , BS->TailleW , BS->TailleH , &dposx , &dposy , &sposx , &sposy , &taillew , &tailleh ) )
    {      
			if (( BD->BPP == 8 ) && ( BS->BPP == 8 ))
            {
                BufS = ( unsigned char * ) BS->bits;
                BufD = ( unsigned char * ) BD->bits;

                dcy = dposy * BD->BPL + dposx ;
                scy = sposy * BS->BPL + sposx ;
                if ( couleur == NIL )  for ( j = 0 ; j < tailleh ; j ++ )
                {                     
                   
                    CopyMemory(BufD + dcy , BufS + scy , taillew ) ;
                    dcy += BD->BPL ; scy += BS->BPL ;
                } else
                for ( j = 0 ; j < tailleh ; j ++ )
                {                     
                    dcx = dcy  ;
                    scx = scy ;
                    for ( i = 0 ; i < taillew ; i ++ )
                    {             
                        if ( BufS [scx ] != couleur ) BufD [ dcx ] = BufS [ scx ] ;
                        dcx ++ ;
                        scx ++ ;
                    }
                    dcy += BD->BPL ;
                    scy += BS->BPL ;
                }
            } else if (( BD->BPP == 16 ) && ( BS->BPP == 8 ))
            {
                BufS = ( unsigned char * ) BS->bits;
                dcy = dposy * ( BD->BPL >> 1 ) + dposx ;
                scy = sposy * BS->BPL + sposx ;
                GetBitmapPalette ( BS , & Pal ) ;
                
                for ( j = 0 ; j < tailleh ; j ++ )
                {
                    dcx = dcy ;
                    scx = scy ;
                    for ( i = 0 ; i < taillew ; i ++)
                    {
                        if ( BufS [ scx ] != couleur )
                        BD->bits[ dcx ] =  ( GET_BLUE(&Pal,BufS [ scx ])>>3)  + 
                            ( (GET_GREEN ( &Pal,BufS[ scx ]) >>3)<< 5 )    + 
                            ( (GET_RED ( &Pal , BufS [ scx ]) >>3) << 10 ) ;
                        dcx ++ ;
                        scx ++ ;
                    }
                    dcy += BD->BPL>>1 ;
                    scy += BS->BPL ;
                }
            } else if (( BD->BPP == 16 ) && ( BS->BPP == 16 ))
            {
                dcy = dposy * ( BD->BPL >> 1 ) + dposx ;
                scy = sposy * ( BS->BPL >> 1 ) + sposx ;                           
                if ( couleur == NIL ) 
                {            
                    BufS = (( unsigned char * ) BS->bits) + ( scy << 1 ) ;
                    BufD = (( unsigned char * ) BD->bits) + ( dcy << 1 ) ;
                    taillew <<= 1 ;
                    for ( j = 0 ; j < tailleh ; j ++ )
                    {                 
                        memcpy( BufD,BufS,taillew) ;                   
                        BufD += BD->BPL ;
                        BufS += BS->BPL ;
                    }
                }
                else 
                {
                    couleur = Color24To16(couleur); 
                    for ( j = 0 ; j < tailleh ; j ++ )
                    {
                        dcx = dcy ;
                        scx = scy ;
                        for ( i = 0 ; i < taillew ; i ++ , dcx ++ , scx ++ )
                        if ( BS->bits[ scx ] != couleur ) BD->bits[ dcx ] = BS->bits[ scx ] ;                               
                        dcy += BD->BPL >> 1 ;
                        scy += BS->BPL >> 1 ;
                    }
                }
            }
    }
}

void Layer::copyLayer( Layer *src )
{
	if (src->RGBbitmap!=NULL)
			CopyObjBitmapToObjBitmap(this->RGBbitmap,0,0,src->RGBbitmap,0,0,src->RGBbitmap->TailleW,src->RGBbitmap->TailleH,-1);
	if (src->AlphaBitmap!=NULL)
			CopyObjBitmapToObjBitmap(this->AlphaBitmap,0,0,src->AlphaBitmap,0,0,src->AlphaBitmap->TailleW,src->AlphaBitmap->TailleH,-1);
}

void Layer::copyPartLayer( Layer *src, Rect2D rect)
{
	if (src->RGBbitmap!=NULL)
			CopyObjBitmapToObjBitmap(this->RGBbitmap,rect.RctHG.iptX,rect.RctHG.iptY,src->RGBbitmap,rect.RctHG.iptX,rect.RctHG.iptY,rect.RctBD.iptX-rect.RctHG.iptX-1,rect.RctBD.iptY-rect.RctHG.iptY-1,-1);
	if (src->AlphaBitmap!=NULL)
			CopyObjBitmapToObjBitmap(this->AlphaBitmap,rect.RctHG.iptX,rect.RctHG.iptY,src->AlphaBitmap,rect.RctHG.iptX,rect.RctHG.iptY,rect.RctBD.iptX-rect.RctHG.iptX-1,rect.RctBD.iptY-rect.RctHG.iptY-1,-1);			
}


void Layer::fillAlphaLayer ( unsigned char coefficiency )
{
	if ( this->AlphaBitmap != NULL )
		this->fillPartAlphaLayer( coefficiency, Rect2D( 0, 0, this->AlphaBitmap->TailleW, this->AlphaBitmap->TailleH ) );
}

void Layer::fillPartAlphaLayer(unsigned char coefficiency,Rect2D rect)
{
	register int i, j;
	int dcx, dcy;
	

	if ( this->AlphaBitmap != NULL )
	{
		dcy = rect.RctHG.iptY * this->AlphaBitmap->BPL + rect.RctHG.iptX;
		for ( j = 0; j < rect.RctBD.iptY - rect.RctHG.iptY ; j ++ )
		{
			dcx = dcy ;
			for ( i = 0; i < rect.RctBD.iptX - rect.RctHG.iptX ; i ++ , dcx ++ )
				((unsigned char *)this->AlphaBitmap->bits)[ dcx ] = coefficiency;
			dcy += this->AlphaBitmap->BPL;
		}
	}
}

void Layer::clearAlphaLayer(Rect2D rect)
{
	this->fillPartAlphaLayer( 0, rect );
}

/*
Layer::clearAlphaLayer(Rect2D rect)
{
	if (this->AlphaBitmap!=NULL)
	{
		register j;
		int w,h,dcy;
		SizeRectangle(rect,&w,&h);
		dcy=rect.RctHG.iptY * ( this->AlphaBitmap->BPL >> 1 ) + rect.RctHG.iptX ;
		for ( j = 0 ; j < h ; j ++ )
		{                     
			FillMemory (this->AlphaBitmap->Bitmap+dcy,w,0);
			dcy += this->AlphaBitmap->BPL >> 1;
		}
	}
}
*/

void Layer::drawLine(int x1,int y1,int x2,int y2,int mode,int taille,int color)
{
	HDC Dc;
	HBITMAP OldBitmap;
	HPEN Pinceau , OldPen ;
	POINT Ligne [ 2 ] ;

	// on ne dessine que sur le RGB
	Dc = CreateCompatibleDC ( NULL ) ;
    OldBitmap = (HBITMAP) SelectObject ( Dc ,RGBbitmap->DIBhandler) ;
    
    if ( mode == 0 || taille <= 0 )
        Pinceau = CreatePen ( PS_NULL  , taille , color ) ;
    else 
		Pinceau = CreatePen ( PS_SOLID , taille , color ) ;
    
	OldPen = (HPEN)SelectObject ( Dc , Pinceau ) ;
    

    Ligne [ 0 ] .x = x1 ;
    Ligne [ 0 ] .y = y1 ;
    Ligne [ 1 ] .x = x2 ;
    Ligne [ 1 ] .y = y2 ;

    if ( x1 == x2 && y1 == y2 ) SetPixel ( Dc , x1 , y1 , color ) ;
    else Polygon ( Dc , Ligne , 2 ) ;   
   
    OldPen = (HPEN)SelectObject ( Dc , OldPen ) ;
    SelectObject ( Dc , OldBitmap ) ;
    DeleteObject ( OldPen ) ;
	DeleteDC ( Dc ) ;
}


// manipulations de listes

Layer *Layer::addLayer( Layer *next )
{
	if ( next != NULL )
	{
		if ( this->Next != NULL )
			next->addLayer( this->Next );
		this->Next = next;
	}
	return this;
}

Layer *Layer::nextLayer()
{
	return this->Next;
}

Layer *Layer::removeNext()
{
	if (this->Next != NULL )
		this->Next = this->Next->Next;
	return this;
}

Layer *Layer::nulifyAllNext()
{
	if ( this != NULL )
	{
		this->Next->nulifyAllNext();
		this->Next = NULL;
	}
	return this;
}

Layer *Layer::nulifyNext()
{
	if ( this != NULL )
		this->Next = NULL;
	return this;
}

Layer *Layer::removeAllNext()
{
	if (this->Next != NULL )
	{
		this->Next->nulifyAllNext();
		this->Next=NULL;
	}
	return this;
}

void deleteLayers( Layer *first )
{
	if ( first != NULL )
	{
		deleteLayers( first->nextLayer() );
		delete first;
	}
}

int dsAlphaBitmap(mmachine m,int abmp)
{
	int tmp_res;
	CHECK(MMpush(m,abmp));
	CHECK(MMpush(mm,Msearchinsyspak(mm,"_DSalphaBitmap")));
	Minterpreter(mm);
	MMpull(m);
	return 0;
}
