/******************************************************************************************/
/*                                                                                        */
/*  bitmap.c                                                                              */
/*                                                                                        */
/*  Implementation des fonctions du packages Bitmap.pkg                                   */
/*                                                                                        */
/******************************************************************************************/


//
// Modifications History
//
//$ LB (13/06/2002) : -- added a objbitmap handler counter : "ObjBitmapHandler"
//
//					  -- added ObjBitmap_GetNewHandler function : return a valid handler for a new ObjBitmap
//
//                    -- NewObjBitmap function ==> ObjBitmap_New : don't create a DIBSection anymore, when creating 
//                       an ObjBitmap. Alloc memory for buffer and palette. The DIBSection will be create later if needed.
//
//                    -- added ObjBitmap_CreateDIBsection function : create DIBSection from the bitmap buffer
//
//                    -- added calls to ObjBitmap_CreateDIBSection, in order to ensure the compatibility with the older system.
//                       next stage : suppr those calls where the DIBSection management can be replaced by a manual management of the bitmap buffer
//
//
//$LB (22/07/2003) : add _DIBbitmap to easy the process of Multimedia objects
//




#include "x/Version.h"
#include "x/scolplugin.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "objstr.h"

#include "loadbmp.h"
#include "scolsys.h"

//#define TRACE2D    







/**************************************************************************************************/
/*                                                                                                */
/* D A T A                                                                                        */
/*                                                                                                */
/**************************************************************************************************/


typedef struct BitmapInfo2
{
    BITMAPINFO Bit ;
    RGBQUAD Quad [ 256 ] ;
} BitmapInfo2 ;

typedef struct LPalette2
{
    WORD    Version ;
    WORD    ColorNum ;
    PALETTEENTRY Colors [256 ] ;
} LPalette ;






/**************************************************************************************************/
/*                                                                                                */
/* B O D Y                                                                                        */
/*                                                                                                */
/**************************************************************************************************/




//$ LB (13/06/2002) : don't create a DIBSection anymore, when creating an ObjBitmap.
//                    alloc memory for buffer and palette. The DIBSection will be create later if needed.

/********************************************************/
/*                                                      */
/* int ObjBitmap_New (old NewObjBitmap)                 */
/*                                                      */
/* create a new bitmap buffer and return a handler      */
/*                                                      */
/********************************************************/
int ObjBitmap_New ( PtrObjBitmap B , PtrPalette P )
{
    
     int tf;
	 int sizeImage;
	 int newHandler;


 	 //
	 // Init Data
	 //
	 B->ShiftPC = B->BPP >> 4;                              // shift per color (<=>BPP / 8 / 2)
	 B->BPL = B->TailleW * (B->BPP >>3);                    // nb bytes per line
	 if (B->BPL % 4 != 0 ) B->BPL += 4 - ( B->BPL % 4 ) ;   // align 32 bits (Windows constraint)

	 //$ LB (13/06/2002) : use BPL instead of (BPP * 8 * Width), because Windows Bitmaps are 32 bits aligned...
	 sizeImage = B->BPL * abs(B->TailleH);



	 //
     // alloc Palette
	 //
     if ( B->BPP == 8 )
	 {

			B->table = (PALETTEENTRY*) malloc (256 * sizeof (PALETTEENTRY));

			if (!B->table) return 0;

			if (P)
				for ( tf = 0 ; tf < 256 ; tf ++ )
				{
					B->table[tf].peRed = GET_RED(P,tf) ; 
					B->table[tf].peGreen = GET_GREEN(P,tf) ;
					B->table[tf].peBlue = GET_BLUE(P,tf) ;
					B->table[tf].peFlags = ( unsigned char ) NULL ;
				}
				
			else 
				for ( tf = 0 ; tf < 256 ; tf ++ )
				{
					B->table[tf].peRed = tf; 
					B->table[tf].peGreen = tf;
					B->table[tf].peBlue = tf;
					B->table[tf].peFlags = ( unsigned char ) NULL ;
				}

     } else B->table = NULL;


 	 //
     // alloc Bitmap Buffer
	 //
	 B->bits = (OBJBITMAP_BUFFER) malloc (sizeImage); 
	 if (!B->bits) return 0;



	 //
	 // Init Data
	 //
 	 //memset ((void*) B->bits, 0, sizeImage);  // bitmap buffer
	 B->bits[0] = 0;

     B->DIBhandler = NULL;                    // DIBSection handler
	 #ifdef USE_DRAWDIB
         B->Flags = BUFFER_FLAG_DOWN;         // Flags
     #else
		 B->Flags = 0 ;     
	 #endif


     newHandler = OBJgetNewHandler ();
	 //MMechostr(MSKDEBUG, "creating bitmap %d %d %d\n", newHandler, B->TailleW, B->TailleH);
      
     return newHandler; // return the new handler
}    








//$ LB (13/06/2002) : added ObjBitmap_CreateDIBsection
/********************************************************/
/*                                                      */
/* int ObjBitmap_CreateDIBSection                       */
/*                                                      */
/* create DIBSection from the bitmap buffer             */
/*                                                      */
/********************************************************/

int ObjBitmap_CreateDIBSection (PtrObjBitmap B)
{
DIBSECTION Info ;
BitmapInfo2 Bi ;
int tf,i ;
HDC dc ;
LPalette Pal ;
HPALETTE HPal , HPOld ;
OBJBITMAP_BUFFER bits;
UINT fuColorUse;


	if (B->DIBhandler) return 0; // job has already been done....

     
	//
	// Palette : create the Windows Palette from the table buffer
	//
    if ( B->BPP == 8 )
    {
            Pal.Version = ( unsigned short ) GetVersion () ;
            Pal.ColorNum = 256 ;
            if ( B->table != NULL)
			for (i=0; i < 256; i++)
			{
	            Pal.Colors[i].peRed = B->table[i].peRed; 
				Pal.Colors[i].peGreen = B->table[i].peGreen; 
				Pal.Colors[i].peBlue = B->table[i].peBlue; 
				Pal.Colors[i].peFlags = B->table[i].peFlags; 
			}

			
            HPal = CreatePalette ( (LOGPALETTE*) (&Pal) ) ;            
            dc = CreateCompatibleDC ( NULL ) ;
            HPOld = SelectPalette ( dc , HPal , FALSE ) ;

    } else dc = NULL ;

  
	//
	// fill BITMAP INFOS
	//
    Bi.Bit.bmiHeader.biSize = sizeof ( BITMAPINFOHEADER );
    Bi.Bit.bmiHeader.biWidth = B->TailleW;

    #ifdef USE_DRAWDIB
         Bi.Bit.bmiHeader.biHeight = B->TailleH ;
    #else
         Bi.Bit.bmiHeader.biHeight = - B->TailleH ;
    #endif

	Bi.Bit.bmiHeader.biPlanes = 1;
    Bi.Bit.bmiHeader.biBitCount = B->BPP;

    if ( B->BPP == 8 || B->BPP == 16 || B->BPP == 24 ) 
         Bi.Bit.bmiHeader.biCompression = BI_RGB ;
    else  
	{
		MMechostr ( 1 , "BITMAP %d DIBSection : wrong BPP %d" , B->handler, B->BPP);
		return 0;
	}

    Bi.Bit.bmiHeader.biSizeImage = 0 ;
    Bi.Bit.bmiHeader.biXPelsPerMeter = 0 ;
    Bi.Bit.bmiHeader.biYPelsPerMeter = 0 ;

    if ( B->BPP == 8 )
    {
        Bi.Bit.bmiHeader.biClrUsed = 256;
        fuColorUse = DIB_PAL_COLORS; 
    }
    else 
    {
        Bi.Bit.bmiHeader.biClrUsed = 0 ;
        fuColorUse = DIB_RGB_COLORS ;
    }
	tf = fuColorUse;
    Bi.Bit.bmiHeader.biClrImportant = Bi.Bit.bmiHeader.biClrUsed ;




	//
	// DIB SECTION : create
	//
    if ((B->DIBhandler = CreateDIBSection ( dc, (BITMAPINFO * ) &Bi, tf, (void**)&bits, NULL, 0 )) == NULL) 
	{
		MMechostr ( 1 , "BITMAP %d DIBSection ERROR WIN32 Num %d" , B->handler, GetLastError ());
		return 0;
	}



	//
	// Palette
	//
    if ( dc ) 
    {
        SelectPalette ( dc , HPOld , FALSE ) ;
        DeleteObject ( HPal ) ;
        DeleteDC ( dc ) ;
		if (B->table) free (B->table);
    }


	//
	// DIB SECTION Buffer : set the DIBsection buffer with the bitmap buffer
	//
	memcpy (bits, B->bits, B->BPL * B->TailleH);
	free (B->bits);
	i = GetObject ( B->DIBhandler, sizeof( DIBSECTION ) , &Info ) ;
	if (!i) MMechostr ( 1 , "GETOBJECT ERROR Num %d" , GetLastError ());
	B->bits = (OBJBITMAP_BUFFER)Info.dsBm.bmBits;


     
    return 1;          
}





//$LB (22/07/2003) : add _DIBbitmap to easy the process of Multimedia objects
//

/*****************************************************************************/
/*                                                                           */
/* int DIBbitmap   ( mmachine m )                                            */
/*                                                                           */
/* fun [ObjBitmap] ObjBitmap                                                 */
/*                                                                           */
/* force the switch from a local memory bitmap to a DIB section              */
/* (usefull to video API)                                                    */
/*****************************************************************************/

int DIBbitmap ( mmachine m )
{

        int s;
        PtrObjVoid O ;
        PtrObjBitmap B ;

//***********************************
#if DEBUG_LIB2DOS
MMechostr (0, "\nDIBbitmap");
#endif
//***********************************
        
        
        s = MMpull(m);
        if ( s == NIL )
        {
            MMechostr ( 1 , "_DIBbitmap : Buffer object is NIL\n" ) ;
            return MMpush(m,NIL) ;
        }
        O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        B = ( PtrObjBitmap ) MMstart(m, (O->Buffer>>1) ) ;

		if (B->bits == NULL)
		{
			MMechostr(MSKTRACE,"_DIBbitmap : Bitmap already destroyed %x\n",B->handler);
			return MMpush(m,NIL);
		}

		ObjBitmap_CreateDIBSection (B);

        return MMpush(m,s);
}




/******************************************************************************/
/*                                                                            */
/*  void SetBitmapPalette ( PtrObjBitmap B , PtrPalette P ) ;                 */
/*                                                                            */
/*  Fonction qui affecte une nouvelle palette a un bitmap                     */
/*                                                                            */
/******************************************************************************/

void SetBitmapPalette ( PtrObjBitmap B , PtrPalette P ) 
{
RGBQUAD Pal [ 257 ] ;
HDC dc ;
int i ;
HBITMAP OldBitmap ;   


	//$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);



    for ( i = 0 ; i < 256 ; i ++ ) 
    {
        Pal[i].rgbRed      = GET_RED   (P,i);
        Pal[i].rgbGreen    = GET_GREEN (P,i);
        Pal[i].rgbBlue     = GET_BLUE  (P,i);
        Pal[i].rgbReserved = 0 ;
    }


      
    dc = CreateCompatibleDC ( NULL ) ;   
    OldBitmap = SelectObject ( dc , B->DIBhandler) ;
    i=SetDIBColorTable ( dc , 0 , 256 , &(Pal[0]) ) ;
    SelectObject ( dc , OldBitmap ) ;
    DeleteDC ( dc ) ;
}





/*********************************************************************************/
/*                                                                               */
/*  void GetBitmapPalette ( PtrObjBitmap B , PtrPalette P ) ;                    */
/*                                                                               */
/*  Fonction qui renvoi la palette d'un bitmap                                   */
/*                                                                               */
/*********************************************************************************/

void GetBitmapPalette ( PtrObjBitmap B , PtrPalette P )
{
RGBQUAD Pal [ 256 ] ;
HDC dc ;
int i ;
HBITMAP OldBitmap ;


	//$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

    dc = CreateCompatibleDC ( NULL ) ;
    OldBitmap = SelectObject ( dc , B->DIBhandler) ;
    i = GetDIBColorTable ( dc , 0 , 256 , Pal ) ;
    SelectObject ( dc , OldBitmap ) ;
    DeleteDC ( dc ) ;

    for ( i = 0 ; i < 256 ; i ++ )
    {
        GET_RED(P,i) = Pal[i].rgbRed ;
        GET_GREEN(P,i) = Pal[i].rgbGreen ;
        GET_BLUE(P,i) = Pal[i].rgbBlue ;
    
    }
}




int fillPalette(mmachine m,char *palette, int s2)
{
    RGBQUAD Pal [ 256 ] ;
    HDC dc ;
    int i ;
    HBITMAP OldBitmap ;
    PtrObjVoid OB ;
    PtrObjBitmap B ;


    if ( s2 == NIL ) return -1;
    OB = ( PtrObjVoid ) MMstart(m,s2>>1);
    if ( OB->Type != OBJ_TYPE_BITMAP << 1 ) return -1;
    B = ( PtrObjBitmap ) MMstart(m,OB->Buffer>>1);


	//$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);


    dc = CreateCompatibleDC ( NULL ) ;
    OldBitmap = SelectObject ( dc , B->DIBhandler) ;
    i = GetDIBColorTable ( dc , 0 , 256 , Pal ) ;
    SelectObject ( dc , OldBitmap ) ;
    DeleteDC ( dc ) ;

    for ( i = 0 ; i < 256 ; i ++ )
    {
        palette[i*4+2] = Pal[i].rgbRed ;
        palette[i*4+1] = Pal[i].rgbGreen ;
        palette[i*4] = Pal[i].rgbBlue ;
    }

	return 0;
}


extern sysexe (*SCexeSearch)(char *categ, char *name);


int GRdrawIcon(mmachine m)
{
	PtrObjVoid OB ;
	PtrObjBitmap B ;
	HDC Dcb ;
	sysexe e;
	char *path,*categ;
	HICON hic;
	HBITMAP OldBitmap ;
	int c,s,s2,x,y,num;
	SHFILEINFO sfi;

	y=MMpull(m)>>1;
	x=MMpull(m)>>1;
	num=MMpull(m)>>1;
	s2 = MMpull(m) ;
	s = MMpull(m)>>1 ;
	c = MMpull(m)>>1 ;

	if ((s2==NIL)||(s==NIL)) return MMpush(m,NIL);
	categ=NULL;
	if (c!=NIL) categ=MMstartstr(m,c);
	e=SCexeSearch(categ,MMstartstr(m,s));
	if (e==NULL) return MMpush(m,NIL);
	path=e->path;
	if (path==NULL) return MMpush(m,NIL);

	hic=ExtractIcon((HINSTANCE)SCgetExtra("this_inst"),path,num);
	if ((num==0)&&( (((int)hic)==1) || (hic==NULL)))
	{
		MMechostr(MSKDEBUG,"try again\n");
		CoInitialize(NULL);
		if (SHGetFileInfo(path,0,&sfi,sizeof(SHFILEINFO),SHGFI_ICON)==0)
		{
			MMechostr(MSKDEBUG,"cannot retrieve icon %d from %s\n",num,path);
			return MMpush(m,NIL);
		}
		hic=sfi.hIcon;
		CoUninitialize();

//		strcpy(buf,path);
//		hic=ExtractAssociatedIcon(thisinst,buf,&num);
	}

	if ( (((int)hic)==1) || (hic==NULL))
	{
		MMechostr(MSKDEBUG,"cannot retrieve icon %d from %s (%d)\n",num,path,hic);
		return MMpush(m,NIL);
	}
	OB = ( PtrObjVoid ) MMstart(m, (s2>>1) ) ;        
	B = ( PtrObjBitmap ) MMstart(m, (OB->Buffer>>1) ) ;


	//$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);


	Dcb = CreateCompatibleDC ( NULL ) ;
	OldBitmap = SelectObject ( Dcb , B->DIBhandler) ;
	DrawIcon(Dcb,x,y,hic);
	SelectObject ( Dcb , OldBitmap ) ;
	DeleteDC ( Dcb ) ;
	DestroyIcon(hic);
	return MMpush(m,0);
}






int GRinfoIcon(mmachine m)
{
	sysexe e;
	char *path,*categ;
	HICON hic;
	int c,s,num;
	ICONINFO icinf;
	BITMAP bmpinf;
	SHFILEINFO sfi;
	
	num=MMpull(m)>>1;
	s = MMpull(m)>>1 ;
	c = MMpull(m)>>1 ;

	if (s==NIL) return MMpush(m,NIL);
	categ=NULL;
	if (c!=NIL) categ=MMstartstr(m,c);
	e=SCexeSearch(categ,MMstartstr(m,s));
	if (e==NULL) return MMpush(m,NIL);
	path=e->path;
	if (path==NULL) return MMpush(m,NIL);

	hic=ExtractIcon((HINSTANCE)SCgetExtra("this_inst"),path,num);
	if ((num==0)&&( (((int)hic)==1) || (hic==NULL)))
	{
		MMechostr(MSKDEBUG,"try again\n");
		CoInitialize(NULL);
		SHGetFileInfo(path,0,&sfi,sizeof(SHFILEINFO),SHGFI_ICON);
		hic=sfi.hIcon;
		CoUninitialize();
	}
	if ( (((int)hic)==1) || (hic==NULL) || (!GetIconInfo(hic,&icinf)) )
	{
		MMechostr(MSKDEBUG,"cannot retrieve icon %d from %s\n",num,path);
		return MMpush(m,NIL);
	}
	GetObject(icinf.hbmMask,sizeof(BITMAP),&bmpinf);
	if (MMpush(m,bmpinf.bmWidth*2)) return MERRMEM;
	if (MMpush(m,bmpinf.bmHeight*2)) return MERRMEM;
	if (MMpush(m,2*2)) return MERRMEM;
	DestroyIcon(hic);
	return MBdeftab(m);
}







/*****************************************************************************/
/*                                                                           */
/* int GRLoadBitmap ( mmachine m )                                           */
/*                                                                           */
/* correspond a la fonction magma I _load_bitmap ( S )                       */
/* qui charge un bitmap BMP en memoire windows                               */
/*                                                                           */
/*****************************************************************************/

int GenericLoadBitmap ( mmachine m , char BPP )
{
    int R ;
    int res,l,s,s2;
    PtrObjVoid O ;
    PtrObjBitmap B ;
    char Name [ 8000 ] ;   


    /******************************* DEBUG 2D *******************************/
    #ifdef TRACE2D                    
        MMechostr( 1 , "DBG _load_bitmap\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /***************************************************************************/


    /* test channel nil */
    if(MMget(m,1)==NIL)
    {
        MMechostr(MSKDEBUG,"_LDbitmap generic : Channel NIL\n");
        m->pp+=1;
        return 0;
    }
    /* test nom nil */

    s = MMget(m,0) ;
    if ( s == NIL ) 
    {
		MMpull(m);
        MMset(m,0,NIL);
        return 0 ;
    }
    strcpy ( Name , ( char * ) MMstart(m, (s>>1)+1) ) ; 
        
    
    l = ( sizeof ( struct ObjVoid ) + 3 ) >> 2 ;
    s = MMmallocCLR (m,l,TYPETAB) ;
    if ( s==NIL) return MERRMEM ;
    if ( MMpush(m,(s<<1)+1)) return MERRMEM ;
    l = ( sizeof ( struct ObjBitmap ) + 3 ) >> 2 ;
    s2 = MMmalloc ( m,l,TYPEBUF ) ;
    if ( s2 == NIL ) return MERRMEM ;
    s = MMpull(m) ;

    O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
    B = ( PtrObjBitmap ) MMstart(m, s2 ) ;
    O->Type = OBJ_TYPE_BITMAP << 1 ;
    O->Buffer = ( s2 << 1 ) + 1 ;
    O->Tab = MMpull(m);   
    B->PosY = 0 ;
    B->PosX = 0 ;
    O->Father = NIL ;
   
    B->bits = NULL ;

    LoadBMP ( Name , B , BPP ) ;

    if ( B->bits == NULL )
    {
        MMechostr ( 1 , "_load_bitmap failed\n" ) ;
        MMpull(m);
        return MMpush(m,NIL);
    }
     
    R = B->handler;
//    MMechostr ( 1 , "LDbitmap%d %x %d %d %s\n",B->BPP,B->HBitmap,B->TailleW,B->TailleH,Name) ;

    MMpush(m,s) ;
    res = OBJcreate(m,OBJTYPBITMAP,(int)R,0,(int)NULL) ;   

    /************************** DEBUG 2D **********************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG _load_bitmap done\n" ) ;
    #endif
    /*************************************************************/

    return res ;
}





int GRLoadBitmap ( mmachine m ) 
{
    return GenericLoadBitmap ( m ,  16 ) ;
}





/*****************************************************************************/
/*                                                                           */
/* int GREmptyBuffer   ( mmachine m )                                        */
/*                                                                           */
/* correspond a la fonction magma Obj _empty_buffer ( Obj , I )              */
/* laquelle remplit un buffer graphique d'une couleur unie                   */
/*                                                                           */
/*****************************************************************************/

#define FILL_VERSION

int GREmptyBuffer ( mmachine m )
{

        int s , c ;
        PtrObjVoid O ;
        PtrObjBitmap B ;
#ifdef FILL_VERSION
         int i ;
         short int * Image ;
         unsigned char * Image8 ;
#else        

        HPEN Pinceau , OldPen ;
        HWND OldItem ;
        LOGBRUSH Brush ;
        HBRUSH HBrush , OldBrush ;
        HDC Dc ;
#endif

        /*************** DEBUG 2D *********************************************/
        #ifdef TRACE2D
            MMechostr  ( 1 , "DBG _FILLbitmap\n" ) ;
            FDebug2D ( m ) ;
        #endif
        /**********************************************************************/

        
        c = MMpull(m) >> 1 ;
     
        
        s = MMpull(m) ;
        if ( s == NIL )
        {
            MMechostr ( 1 , "_FILLbitmap : Buffer object is NIL\n" ) ;
            return MMpush(m,NIL) ;
        }
        O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        
        B = ( PtrObjBitmap ) MMstart(m, (O->Buffer>>1) ) ;

		if (B->bits == NULL)
		{
			MMechostr(MSKTRACE,"FILLbitmap : Bitmap already destroyed %x\n",B->handler);
			return MMpush(m,NIL);
		}

#ifdef FILL_VERSION
                
        if ( B->BPP == 16 )
        {


            c = Color24To16(c);
            Image = ( short int * ) B->bits;            
           
            i = ( B->BPL * B->TailleH ) >> B->ShiftPC;
            while ( i ) Image [ -- i ] = c ;            
        }
        else if ( B->BPP == 8 )
        {




            
            Image8 = ( unsigned char * ) B->bits;
            i = ( B->BPL * B->TailleH ) ;
            while ( i ) Image8 [-- i ] = ( unsigned char ) c ;
        }
#else


		//$ LB (13/06/2002) : create DIBSection from bitmap buffer
		if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);


        Dc = CreateCompatibleDC ( NULL ) ;
        OldItem = SelectObject ( Dc , B->DIBhandler) ;
        Pinceau = CreatePen ( PS_SOLID , 0 , c ) ;
        Brush.lbStyle = BS_SOLID ;
        Brush.lbColor = c ;

        HBrush = CreateBrushIndirect ( & Brush ) ;        
        OldPen = SelectObject ( Dc , Pinceau ) ;
        OldBrush = SelectObject ( Dc , HBrush ) ;
        Rectangle ( Dc , 0 , 0 , B->TailleW , B->TailleH ) ;        
        SelectObject ( Dc , OldPen ) ;
        SelectObject ( Dc , OldBrush ) ;
        DeleteObject ( HBrush ) ;
        DeleteObject ( Pinceau ) ;
        SelectObject ( Dc , OldItem ) ;
        DeleteDC ( Dc ) ;
#endif

        s = MMpush(m,s) ;
    
        /************************ DEBUG 2D ***************************************/
        #ifdef TRACE2D
            FDebug2D ( m ) ;
            MMechostr ( 1 , "DBG _FILLbitmap done\n" ) ;
        #endif
        /*************************************************************************/

        return s ;
}


    



/*****************************************************************************/
/*                                                                           */
/* int GRGetBitmapSize  ( mmachine m )                                       */
/*                                                                           */
/*  correspond a la fonction magma I _get_bitmap_wsize  ( Obj )              */
/*  qui renvoi la taille d'un bitmap                                         */
/*                                                                           */
/*****************************************************************************/

int GRGetBitmapSize ( mmachine m )
{
    PtrObjVoid O ;
    PtrObjBitmap B ;
    int s , h , w ; 

    /******************************* dEBUG 2D *********************************/
    #ifdef TRACE
        MMechostr ( 1 , "DBG _get_bitmap_size\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /***************************************************************************/
    s = MMpull ( m ) ;     
    if ( s == NIL )
    {
        MMechostr ( 1 , "GetBitmapWSize : Bitmap object is NIL\n" ) ;
        w = -1 ; 
        h = -1 ;
    } 
    else
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;    
        B = ( PtrObjBitmap ) MMstart(m, (O->Buffer>>1) ) ;
        w = B->TailleW << 1 ;
        h = B->TailleH << 1 ;
    }
    if ( MMpush(m,w )) return MERRMEM ;
    if ( MMpush(m,h )) return MERRMEM ;
    if ( MMpush(m,4 )) return MERRMEM ;
    s = MBdeftab ( m ) ;

    /******************************** DEBUG 2D ***********************************/
    #ifdef TRACE
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG _get_bitmap_size done\n" ) ;
    #endif
    /*****************************************************************************/

    return s ;

}

/*****************************************************************************/
/*                                                                           */
/* int GRCreateBitmap ( mmachine m ) ;                                       */
/*                                                                           */
/* correspond a la fonction magma Obj _create_buffer ( I TailleW , I TailleH,*/
/* I Handler ) laquelle cree une buffer graphique                            */
/*                                                                           */
/*****************************************************************************/

int GenericCreateBitmap ( mmachine m , int BPP )
{
        int s , s2 , l , res ;
        PtrObjVoid O ;
        PtrObjBitmap B ;        
        int R ;        

        /******************* DEBUG 2D **************************************/
        #ifdef TRACE2D
            MMechostr ( 1 , "DBG _create_bitmap\n" ) ;
            FDebug2D ( m ) ;
        #endif
        /********************************************************************/


		if ((MMget(m,0)<=0)||(MMget(m,1)<=0))
		{
            MMechostr ( 1 , "Creation of graphical Buffer impossible.\n" ) ;
            MMechostr ( 1 , "W=%d H=%d\n",MMget(m,1)>>1,MMget(m,0)>>1 ) ;
			MMpull(m);
			MMpull(m);
			MMpull(m);
			return MMpush(m,NIL);
		}
        
        l = ( sizeof ( struct ObjVoid ) + 3 ) >> 2 ;
        s = MMmallocCLR (m,l,TYPETAB) ;
        if ( s == NIL ) return MERRMEM ;
        if ( MMpush(m,(s<<1)+1)) return MERRMEM  ;
        l = ( sizeof ( struct ObjBitmap ) + 3 ) >> 2 ;
        s2 = MMmalloc ( m,l,TYPEBUF ) ;
        if ( s2 == NIL ) return MERRMEM ;
        s = MMpull(m) ;

        O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        B = ( PtrObjBitmap ) MMstart(m, s2 ) ;
     
        O->Buffer = (s2<<1)+1;
        O->Tab = NIL ;
        O->Father = NIL ;
        O->Type = OBJ_TYPE_BITMAP << 1 ;

        B->TailleH = MMpull(m)>> 1 ;
        B->TailleW = MMpull(m) >> 1 ;
        B->BPP = BPP ;
        B->handler = ObjBitmap_New ( B , NULL ) ;

        if ( B->bits == NULL )
        {
            MMechostr ( 1 , "Creation of graphical Buffer failed.\n" ) ;
            MMechostr ( 1 , "W=%d H=%d\n",B->TailleW,B->TailleH ) ;
            MMpull(m) ; /* depile le channel */ 
            return MMpush(m,NIL);
        }       

        R = B->handler;
        MMpush(m,s) ;
//        MMechostr ( 1 , "CRbitmap%d %x %d %d\n",B->BPP,B->HBitmap,B->TailleW,B->TailleH) ;
        res = OBJcreate(m,OBJTYPBITMAP,(int)R,0,(int)NULL ) ;

        /*************************** DEBUG 2D ***************************/
        #ifdef TRACE2D
            FDebug2D ( m ) ;
            MMechostr ( 1 , "DBG _create_buffer done\n" ) ;
        #endif
        /******************************************************************/

        return res ;
}



int GRCreateBitmap ( mmachine m )
{
    return GenericCreateBitmap( m , 16 ) ;
}




/*****************************************************************************/
/*                                                                           */
/*  int GRBlitBitmap ( mmachine m ) ;                                        */
/*                                                                           */
/* correspond a la fonction magma Obj _paint_buffer ( ObjWin , ObjBuf ,      */
/* I PosX , I PosY ) ;                                                       */
/*                                                                           */
/*****************************************************************************/

int GRBlitBitmap ( mmachine m )
{
        int PosX , PosY , s , s2 , res ;
        PtrObjVoid OW , OB ;
        PtrObjWindow W ;
        PtrObjBitmap B ;
        HDC Dcw , Dcb ;       
        HBITMAP OldBitmap ;

        /************************** DEBUG 2D ***************************************/
        #ifdef TRACE2D
            MMechostr ( 1 , "DBG _paint_buffer\n" ) ;
        #endif
        /****************************************************************************/

        PosY = MMpull(m)>>1 ;
        PosX = MMpull(m)>>1 ;
        s2 = MMpull(m) ;
        s = MMpull(m) ;

        if ( s2 != NIL && s != NIL )
        {
            OB = ( PtrObjVoid ) MMstart(m, (s2>>1) ) ;        
            OW = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        
            B = ( PtrObjBitmap ) MMstart(m, (OB->Buffer>>1) ) ;
            W = ( PtrObjWindow ) MMstart(m, (OW->Buffer>>1) ) ;

			
			//$ LB (13/06/2002) : create DIBSection from bitmap buffer
			if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

        
            Dcw = GetDC ( W->WHandler ) ;       
            Dcb = CreateCompatibleDC ( Dcw ) ;
            OldBitmap = SelectObject ( Dcb , B->DIBhandler) ;
            BitBlt ( Dcw , PosX , PosY , B->TailleW ,
                                B->TailleH , Dcb , 0 , 0 , SRCCOPY ) ;
            SelectObject ( Dcb , OldBitmap ) ;
            DeleteDC ( Dcb ) ;      
        }
        res = MMpush(m,s) ;

        /*********************** DEBUG 2D *****************************/
        #ifdef TRACE2D
            FDebug2D ( m ) ;
            MMechostr ( 1 , "DBG _paint_buffer Done\n" ) ;
        #endif
        /****************************************************************/

        return res ;
}





/*****************************************************************************/ 
/*                                                                           */ 
/*  int GRStretchBlit ( mmachine m ) ;                                       */ 
/*                                                                           */ 
/* Correspond a la fonction scol ObjWin _STBLITbitmap (                      */ 
/* ObjWin , I WX , I WY , I WW , I WY , ObjBitmap , I BX , I BY , I BW ,I BH)*/ 
/*                                                                           */ 
/*  Laquelle effectue une copie par stretching dans une fenetre d'un bitmap  */ 
/*                                                                           */ 
/*****************************************************************************/ 

int GRStretchBlit ( mmachine m )
{
        int WX , WY , BX , BY , s , s2 , res , BW , BH , WW , WH ;
        PtrObjVoid OW , OB ;
        PtrObjWindow W ;
        PtrObjBitmap B ;
        HDC Dcw , Dcb ;       
        HBITMAP OldBitmap ;

        /************************** DEBUG 2D ***************************************/
        #ifdef TRACE2D
            MMechostr ( 1 , "(DBG) _STBLTbitmap(8)\n" ) ;
        #endif
        /****************************************************************************/

        BH = MMpull(m) >> 1 ;
        BW = MMpull(m) >> 1 ;
        BY = MMpull(m) >> 1 ;
        BX = MMpull(m) >> 1 ;
        s2 = MMpull(m) ;
        WH = MMpull(m) >> 1 ;
        WW = MMpull(m) >> 1 ;
        WY = MMpull(m) >> 1 ;
        WX = MMpull(m) >> 1 ;
        s = MMpull(m) ;

        if ( s != NIL && s2 != NIL )
        {
            OB = ( PtrObjVoid ) MMstart(m, (s2>>1) ) ;    
            OW = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        
            B = ( PtrObjBitmap ) MMstart(m, (OB->Buffer>>1) ) ;
            W = ( PtrObjWindow ) MMstart(m, (OW->Buffer>>1) ) ;

			
			//$ LB (13/06/2002) : create DIBSection from bitmap buffer
			if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

        
            Dcw = GetDC ( W->WHandler ) ;       
            Dcb = CreateCompatibleDC ( Dcw ) ;
            OldBitmap = SelectObject ( Dcb , B->DIBhandler) ;
            StretchBlt( Dcw , WX , WY , WW , WH , Dcb , BX , BY , BW , BH , SRCCOPY ) ;

            SelectObject ( Dcb , OldBitmap ) ;
            DeleteDC ( Dcb ) ;      
        }
        
        res = MMpush(m,s) ;

        /*********************** DEBUG 2D *****************************/
        #ifdef TRACE2D
            FDebug2D ( m ) ;
            MMechostr ( 1 , "(DBG) _STBLITbitmap(8) Done\n" ) ;
        #endif
        /****************************************************************/

        return res ;
}






/*****************************************************************************/
/*                                                                           */
/* int GRDrawRectangle ( mmachine m )                                        */
/*                                                                           */
/* correspond a la fonction magma Obj _draw_rectangle ( Obj Buffer , I PosX ,*/
/* I PosY , I TailleW , I TailleH , I Mode contour , I Taille contour ,      */
/* I Couleur contour , I mode remplissage , I couleur remplissage )          */
/* laquelle dessine u nrectangle dans une buffer graphique                   */
/*                                                                           */
/*****************************************************************************/

int GRDrawRectangle ( mmachine m )
{
        int Couleur_Remplissage ;
        int Mode_Remplissage ;
        int Couleur_Contour ;
        int Taille_Contour ;
        int Mode_Contour ;
        int TailleW , TailleH , PosX , PosY , s ;
        PtrObjVoid O ;        
		PtrObjBitmap B;
        HPEN Pinceau , OldPen ;
        HWND Item , OldItem ;
        LOGBRUSH Brush ;
        HBRUSH HBrush , OldBrush ;
        HDC Dc ;

        /******************** DEBUG 2D ************************/
        #ifdef TRACE2D
            MMechostr( 1 , "DBG _draw_rectangle\n" ) ;
            FDebug2D ( m ) ;
        #endif
        /*******************************************************/
        

        Couleur_Remplissage = MMpull(m)>> 1 ;
        Mode_Remplissage = MMpull(m)>>1 ;
        Couleur_Contour = MMpull(m)>>1 ;
        Taille_Contour = MMpull(m)>>1 ;
        Mode_Contour = MMpull(m)>>1 ;
        TailleH = MMpull(m)>>1 ;
        TailleW = MMpull(m)>>1 ;
        PosY = MMpull(m)>>1 ;
        PosX = MMpull(m)>>1 ;
        s = MMpull(m) ;
        if ( s == NIL )
        {
            MMechostr ( 1 , "DrawRectangle : Buffer is NIL\n" ) ;
            return MMpush(m,NIL) ;
        }
        O = ( PtrObjVoid ) MMstart(m, (s>>1 ) ) ;        
        switch ( O->Type >> 1 )
        {
            case OBJ_TYPE_BITMAP :

				B = ( PtrObjBitmap ) MMstart(m, (O->Buffer>>1));
				
				//$ LB (13/06/2002) : create DIBSection from bitmap buffer
				if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

                Item = (HWND)B->DIBhandler;
                Dc = CreateCompatibleDC ( NULL ) ;
                OldItem = SelectObject ( Dc , Item ) ;
                break ;

            case OBJ_TYPE_WINDOW :
                Item = (( PtrObjWindow ) MMstart(m,(O->Buffer>>1) )) ->WHandler ;
                Dc = GetDC ( Item ) ;
                break ;
        }

        if ( Taille_Contour <= 0 ) {Taille_Contour = 0 ; Mode_Contour = DRAW_INVISIBLE;}

        if ( Mode_Contour == DRAW_INVISIBLE )
                Pinceau = CreatePen ( PS_NULL , Taille_Contour , Couleur_Contour ) ;
        else Pinceau = CreatePen ( PS_SOLID , Taille_Contour , Couleur_Contour ) ;
        if ( Mode_Remplissage == DRAW_INVISIBLE )
                Brush.lbStyle = BS_NULL ;
        else Brush.lbStyle = BS_SOLID ;
        Brush.lbColor = Couleur_Remplissage ;
              
        HBrush = CreateBrushIndirect ( & Brush ) ;        
        OldPen = SelectObject ( Dc , Pinceau ) ;
        OldBrush = SelectObject ( Dc , HBrush ) ;
        Rectangle ( Dc , PosX , PosY , PosX + TailleW , PosY + TailleH ) ;        
        SelectObject ( Dc , OldPen ) ;
        SelectObject ( Dc , OldBrush ) ;
        DeleteObject ( HBrush ) ;
        DeleteObject ( Pinceau ) ;
        if ( O->Type == OBJ_TYPE_BITMAP << 1 )
        {
            SelectObject ( Dc , OldItem ) ;
            DeleteDC ( Dc ) ;
        }
        s = MMpush(m,s) ;

        /******************************** DEBUG 2D *****************************/
        #ifdef TRACE2D
            FDebug2D ( m ) ;
            MMechostr ( 1 , "DBG _draw_rectangle done\n" ) ;
        #endif
        /************************************************************************/

        return s ;
}    







/*****************************************************************************/
/*                                                                           */
/* int GRDrawLine ( mmachine m )                                             */
/*                                                                           */
/* cette fonction correspond a la fonction magma Obj _draw_line (            */
/* ( Obj Buffer , I X1 , I Y1 , I X2 , I Y2 , I Mode_Contour,I Taille_Contour*/
/* I Couleur_Contour )                                                       */
/*                                                                           */
/*****************************************************************************/

int GRDrawLine ( mmachine m )
{
    int Taille_Contour , Mode_Contour , Couleur_Contour , X1 , Y1 , X2 , Y2 ;
    int s ;
    PtrObjVoid O ;    
	PtrObjBitmap B;
    HPEN Pinceau , OldPen ;
    HWND Item , OldItem ;
    POINT Ligne [ 2 ] ;
    HDC Dc ;

    /******************************** DEBUG 2D ********************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _draw_line\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**************************************************************************/

    Couleur_Contour = MMpull(m)>>1 ;
    Taille_Contour = MMpull ( m ) >> 1 ;
    Mode_Contour = MMpull(m)>>1 ;
    Y2 = MMpull(m) >> 1 ;
    X2 = MMpull(m) >> 1 ;
    Y1 = MMpull(m) >> 1 ;
    X1 = MMpull(m) >> 1 ;
    s = MMpull(m) ;
    if ( s== NIL )
    {
        MMechostr ( 1 , "DrawLine : Buffer is NIL\n" ) ;
        return MMpush(m,NIL) ;
    }
    O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;         
    switch ( O->Type >> 1 )
    {
        case OBJ_TYPE_BITMAP :          

			B = ( PtrObjBitmap ) MMstart(m, (O->Buffer>>1));
				
			//$ LB (13/06/2002) : create DIBSection from bitmap buffer
			if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

            Dc = CreateCompatibleDC ( NULL ) ;
            Item = (HWND)B->DIBhandler;
            OldItem = SelectObject ( Dc , Item ) ;
            break ;

        case OBJ_TYPE_WINDOW :
            Item = (( PtrObjWindow ) MMstart(m,(O->Buffer>>1) ) ) ->WHandler ;
            Dc = GetDC ( Item ) ;
            break ;
    }

    if ( Mode_Contour == DRAW_INVISIBLE ||Taille_Contour <= 0 )
        Pinceau = CreatePen ( PS_NULL  , Taille_Contour , Couleur_Contour ) ;
    else Pinceau = CreatePen ( PS_SOLID , Taille_Contour , Couleur_Contour ) ;
    OldPen = SelectObject ( Dc , Pinceau ) ;
    

    Ligne [ 0 ] .x = X1 ;
    Ligne [ 0 ] .y = Y1 ;
    Ligne [ 1 ] .x = X2 ;
    Ligne [ 1 ] .y = Y2 ;

    if ( X1 == X2 && Y1 == Y2 ) SetPixel ( Dc , X1 , Y1 , Couleur_Contour ) ;
    else Polygon ( Dc , Ligne , 2 ) ;   
   
    OldPen = SelectObject ( Dc , OldPen ) ;
    s = MMpush(m,s) ;
    if ( O->Type == OBJ_TYPE_BITMAP << 1 )
    {
        SelectObject ( Dc , OldItem ) ;
        DeleteDC ( Dc ) ;
    }
    DeleteObject ( OldPen ) ;

    /******************** DEBUG 2D ****************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG _draw_line done\n" ) ;
    #endif
    /***********************************************************/

    return s ;
            
}







/*****************************************************************************/
/*                                                                           */
/* int GRDrawCircle ( mmachine m )                                           */
/*                                                                           */
/* correspond a la fonction magma Obj _draw_circle ( Obj Buffer , I PosX ,   */
/* I PosY , I Rayon , I Mode_Contour , I Taille_Contour , I Couleur_Contour, */
/* I Mode_Remplissage , I Couleur_Remplissage ) ;                            */
/* laquelle dessine un cercle dans un buffer graphique                       */
/*                                                                           */
/*****************************************************************************/

int GRDrawCircle ( mmachine m )
{
    int s , PosX , PosY , Mode_Contour , Taille_Contour , Couleur_Contour , Rayon ;
    int Mode_Fill , Couleur_Fill ;
    PtrObjVoid O ;
    PtrObjBitmap B ;
    HPEN Pinceau , OldPen ;
    HWND OldItem , Item ;
    LOGBRUSH Brush ;
    HBRUSH HBrush , OldBrush ;
    HDC Dc ;

    /******************************** DEBUG 2D ******************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _draw_circle\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /***********************************************************************/


    Couleur_Fill = MMpull(m) >> 1 ;
    Mode_Fill = MMpull(m) >> 1 ;
    Couleur_Contour = MMpull(m) >> 1 ;
    Taille_Contour = MMpull(m) >> 1 ;
    Mode_Contour = MMpull(m) >> 1 ;
    Rayon = MMpull(m) >> 1 ;
    PosY = MMpull(m) >> 1 ;
    PosX = MMpull(m) >> 1 ;
    s = MMpull(m) ;
    if ( s == NIL )
    {
        MMechostr ( 1 , "DrawCircle : Buffer is NIL\n" ) ;
        return MMpush(m,NIL) ;
    }
    O = ( PtrObjVoid ) MMstart(m, ( s>>1 ) ) ;

    switch ( O->Type >> 1 )
    {
        case OBJ_TYPE_BITMAP :
            B = ( PtrObjBitmap ) MMstart(m, (O->Buffer>>1) ) ;
				
			//$ LB (13/06/2002) : create DIBSection from bitmap buffer
			if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

            Dc = CreateCompatibleDC ( NULL ) ;
            Item = (HWND)B->DIBhandler; 
            OldItem = SelectObject ( Dc , Item ) ;
            break ;
        case OBJ_TYPE_WINDOW :
            Item = (( PtrObjWindow ) MMstart(m,( O->Buffer>>1) ) ) ->WHandler ;
            Dc = GetDC ( Item ) ;
            break ;
    }

    if ( Taille_Contour <= 0 ) { Taille_Contour = 0 ; Mode_Contour = DRAW_INVISIBLE;}

    if ( Mode_Contour == DRAW_INVISIBLE )
        Pinceau = CreatePen ( PS_NULL , Taille_Contour , Couleur_Contour ) ;
    else Pinceau = CreatePen ( PS_SOLID , Taille_Contour , Couleur_Contour ) ;
    if ( Mode_Fill == DRAW_INVISIBLE ) Brush.lbStyle = BS_NULL ;
    else Brush.lbStyle = BS_SOLID ;
    Brush.lbColor = Couleur_Fill ;
  
    HBrush = CreateBrushIndirect ( & Brush ) ;
    OldBrush = SelectObject ( Dc , HBrush ) ;
    OldPen = SelectObject ( Dc , Pinceau ) ;
    
    Ellipse ( Dc , PosX - Rayon , PosY - Rayon , PosX + Rayon , PosY + Rayon ) ;

    SelectObject ( Dc , OldBrush ) ;    
    SelectObject ( Dc , OldPen ) ;
    DeleteObject ( Pinceau ) ;
    DeleteObject ( HBrush ) ;
    if ( O->Type == OBJ_TYPE_BITMAP << 1 )
    {
        SelectObject ( Dc , OldItem ) ;
        DeleteDC ( Dc ) ;
    }

    s = MMpush(m,s) ;

    /************************* DEBUG 2D *******************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG _draw_circle done\n" ) ;
    #endif
    /******************************************************************/

    return s ;
}






/********************************************************************************/
/*                                                                              */
/* int GRLoadBitmap8 ( mmachine m ) ;                                           */
/*                                                                              */
/* correspond a la fonction magma ObjBitmap8 _LDbitmap8 ( S )                   */
/* laquelle charge un bitmap 256 couleurs                                       */
/*                                                                              */
/********************************************************************************/

int GRLoadBitmap8 ( mmachine m )
{

    return GenericLoadBitmap ( m , 8 ) ;
}





/***********************************************************************************/
/*                                                                                 */
/*  int GRCreatebitmap8 ( mmachine m ) ;                                           */
/*                                                                                 */
/*  correspond ala fonction ObjBitmap8 _create_bitmap8 ( I tialleW , I tailleH )   */
/* laquelle cree un bitmap palette de la taille desiree                            */
/*                                                                                 */
/***********************************************************************************/

int GRCreateBitmap8 ( mmachine m ) 
{
   return GenericCreateBitmap ( m , 8 ) ;
}





/***********************************************************************************/
/*                                                                                 */
/*  int GRDestroyBitmap ( mmachine m ) ;                                           */
/*                                                                                 */
/*  correspond a la fonction magma I _destroy_bitmap ( ObjBitmap )                 */
/* et a la fonction I _destroy_bitmap8 ( ObjBitmap8 )                              */
/* lesquelles detruisent un objet bitmap et libere la memoire de celui-ci          */
/*                                                                                 */
/***********************************************************************************/

int GRDestroyBitmap ( mmachine m )
{
    int s ; 
    PtrObjVoid O ;
    PtrObjBitmap M ;

    /*********************** DEBUG 2D ****************************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _destroy_bitmap\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**************************************************************************/

    s = MMget(m,0) ;
    if ( s == NIL ) 
    {
        MMset(m,0,0) ;
        return 0 ;
    }

    O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
    M = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;  

    OBJdelTH(m,OBJTYPBITMAP,(int)M->handler);
     
    /************************ DEBUG 2D *******************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _destroy_bitmap done\n" ) ;
    #endif
    /******************************************************************************/

    MMset(m,0,0) ;
    return 0 ;
}




/**********************************************************************************/
/*                                                                                */
/*  int GRGetbitmapPalette ( mmachine m ) ;                                       */
/*                                                                                */
/*  correspond a la fonction tab [i ] = _get_bitmap_paletet ( ObjBItmap8 ) ;      */
/* laquelle renvoi la palette d'un bitmap palete                                  */
/*                                                                                */
/**********************************************************************************/

int GRGetBitmapPalette ( mmachine m ) 
{
    int s ;
    PtrObjVoid O ;
    PtrObjBitmap M ;
    struct Palette Pal ;

    /*********************** DEBUG 2D ****************************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _get_bitmap_palette\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**************************************************************************/

    s = MMpull(m) ;
    if ( s == NIL ) 
    {
        MMpush(m,NIL) ;
        return 0 ;
    }

    O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
    M = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;  
    GetBitmapPalette ( M , &Pal ) ;

    for ( s = 0 ; s < 256 ; s ++ )
    {
        if ( MMpush ( m , ( GET_RED   (&Pal,s))<<1)) return MERRMEM ;
        if ( MMpush ( m , ( GET_GREEN (&Pal,s))<<1)) return MERRMEM ;
        if ( MMpush ( m , ( GET_BLUE  (&Pal,s))<<1)) return MERRMEM ;
    }

    if ( MMpush(m,768<<1)) return MERRMEM ;

    /************************* DEBUG 2D ********************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _get_bitmap_palette done\n" ) ;
    #endif
    /*******************************************************************************/
    
    return MBdeftab ( m ) ;
}





/**************************************************************************************/
/*                                                                                    */
/*  int GRSetBitmapPalette ( mmachine m ) ;                                           */
/*                                                                                    */
/* correspond a la focntion magma ObjBitmap8 _set_bitmap_palette ( ObjBitmap8 , tab[i]*/
/* laquelle affecte une nouvelle palette a un objet bitmap palettee                   */
/*                                                                                    */
/**************************************************************************************/

int GRSetBitmapPalette ( mmachine m )
{

    int s , j  , i ;
    PtrObjVoid O ;
    PtrObjBitmap M ;
    struct Palette Pal ;
    unsigned int * Tablo ;

    /*********************** DEBUG 2D ****************************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _set_bitmap_palette\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**************************************************************************/

    s = MMpull(m) ;
    if ( s == NIL ) return 0 ;
    if ( MMsize(m,s>>1) != 768 )
    {
        MMechostr ( 1 , "The Palette Array haven't the good size\n" ) ;
        return 0 ;
    }
    Tablo = ( unsigned int * ) MMstart(m,(s>>1) );

    s = MMget(m,0) ;
    if ( s == NIL ) 
    {
        MMset(m,0,0) ;
        return 0 ;
    }

    O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
    M = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;  

    i = 0 ;
    for ( j = 0 ; j < 256 ; j ++ )
    {
        GET_RED   (&Pal,j) = Tablo [ i ++ ] >> 1;
        GET_GREEN (&Pal,j) = Tablo [ i ++ ] >> 1;
        GET_BLUE  (&Pal,j) = Tablo [ i ++ ] >> 1;
    }

    SetBitmapPalette ( M , &Pal ) ;
   

    /************************* DEBUG 2D ********************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _get_bitmap_palette done\n" ) ;
    #endif
    /*******************************************************************************/
    
    return 0 ;
}






/*****************************************************************************/
/*                                                                           */
/*  int GRCopyBitmap ( mmachine m ) ;                                        */
/*                                                                           */
/* correspond a la fonction magma Obj _copy_bitmap ( ObjBitmap8 Destination, */
/* I PosX , I PosY , ObjBitmap8 Source , I SPosX,  I SPosY , I TaillW ,      */
/* I TailleH , I Couleur) ;                                                  */
/*                                                                           */
/*****************************************************************************/

int GRCopyBitmap ( mmachine m )
{
        int s , s2 , res , couleur , taillew , tailleh , sposx , sposy , dposx , dposy ;
        PtrObjVoid OS , OD ;       
        PtrObjBitmap BD,BS ;        
        int i , dcx , dcy , scx , scy ;
        register unsigned char * BufS , * BufD ;       
        register j ;
        struct Palette Pal ;       

        /************************** DEBUG 2D ***************************************/
        #ifdef TRACE2D
            MMechostr ( 1 , "DBG _copy bitmap\n" ) ;
            FDebug2D ( m ) ;
        #endif
        /****************************************************************************/

        couleur = MMpull(m) >> 1 ;        
        tailleh = MMpull(m) >> 1 ;
        taillew = MMpull(m)>> 1 ;
        sposy = MMpull(m) >> 1 ;
        sposx = MMpull(m) >> 1 ;
        s2 = MMpull(m) ;
        dposy = MMpull(m) >> 1;
        dposx = MMpull(m) >> 1 ;
        s = MMpull(m) ;
        if ( s2 == NIL )
        {
            MMechostr ( 1 , "PaintBuffer : Buffer object is NIL\n" ) ;
            return MMpush(m,s) ;
        }
        OS = ( PtrObjVoid ) MMstart(m, (s2>>1) ) ;        
        if ( s == NIL )
        {
            MMechostr ( 1 , "PaintBuffer : Window object is NIL\n" ) ;
            return MMpush(m,s) ;
        }
        OD = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        
        BS = ( PtrObjBitmap ) MMstart(m, (OS->Buffer>>1) ) ;
        BD = ( PtrObjBitmap ) MMstart(m, (OD->Buffer>>1) ) ;

		if (BS->bits == NULL)
		{
		    if (OS->Tab==NIL) MMechostr(MSKDEBUG,"bitmap is _CRbitmap\n");
	    	  else  MMechostr(MSKDEBUG,"bitmap is _LDbitmap : %s\n",MMstartstr(m,OS->Tab>>1));

			MMechostr(MSKTRACE,"COPYbitmap : Source Bitmap already destroyed %x\n",BS->handler);
			return MMpush(m,NIL);
		}

		if (BD->bits==NULL)
		{
			MMechostr(MSKTRACE,"COPYbitmap : Dest Bitmap already destroyed %x\n",BD->handler);
			return MMpush(m,NIL);
		}

		
        /* calcule le rectangle final = proc de clipping */
      
        
        if ( ClipBlit ( BD->TailleW , BD->TailleH , BS->TailleW , BS->TailleH ,
                &dposx , &dposy , &sposx , &sposy , &taillew , &tailleh ) )
        {      
         
            if (( BD->BPP == 8 ) && ( BS->BPP == 8 ))
            {
                BufS = ( unsigned char * ) BS->bits;
                BufD = ( unsigned char * ) BD->bits;

                dcy = dposy * BD->BPL + dposx ;
                scy = sposy * BS->BPL + sposx ;
                if ( couleur == NIL )  for ( j = 0 ; j < tailleh ; j ++ )
                {                     
                   
                    CopyMemory(BufD + dcy , BufS + scy , taillew ) ;
                    dcy += BD->BPL ; scy += BS->BPL ;
                } else
                for ( j = 0 ; j < tailleh ; j ++ )
                {                     
                    dcx = dcy  ;
                    scx = scy ;
                    for ( i = 0 ; i < taillew ; i ++ )
                    {             
                        if ( BufS [scx ] != couleur ) BufD [ dcx ] = BufS [ scx ] ;
                        dcx ++ ;
                        scx ++ ;
                    }
                    dcy += BD->BPL ;
                    scy += BS->BPL ;
                }
            } else if (( BD->BPP == 16 ) && ( BS->BPP == 8 ))
            {
				GetBitmapPalette ( BS , & Pal ) ;
                BufS = ( unsigned char * ) BS->bits;
                dcy = dposy * ( BD->BPL >> 1 ) + dposx ;
                scy = sposy * BS->BPL + sposx ;
                                
                for ( j = 0 ; j < tailleh ; j ++ )
                {
                    dcx = dcy ;
                    scx = scy ;
                    for ( i = 0 ; i < taillew ; i ++)
                    {
                        if ( BufS [ scx ] != couleur )
                        BD->bits [ dcx ] =  ( GET_BLUE(&Pal,BufS [ scx ])>>3)  + 
                            ( (GET_GREEN ( &Pal,BufS[ scx ]) >>3)<< 5 )    + 
                            ( (GET_RED ( &Pal , BufS [ scx ]) >>3) << 10 ) ;
                        dcx ++ ;
                        scx ++ ;
                    }
                    dcy += BD->BPL>>1 ;
                    scy += BS->BPL ;
                }
            } else if (( BD->BPP == 16 ) && ( BS->BPP == 16 ))
            {
                dcy = dposy * ( BD->BPL >> 1 ) + dposx ;
                scy = sposy * ( BS->BPL >> 1 ) + sposx ;                           
                if ( couleur == NIL ) 
                {            
                    BufS = (( unsigned char * ) BS->bits ) + ( scy << 1 ) ;
                    BufD = (( unsigned char * ) BD->bits ) + ( dcy << 1 ) ;
                    taillew <<= 1 ;
                    for ( j = 0 ; j < tailleh ; j ++ )
                    {                 
                        memcpy( BufD,BufS,taillew) ;                   
                        BufD += BD->BPL ;
                        BufS += BS->BPL ;
                    }
                }
                else 
                {
                    couleur = Color24To16(couleur); 
                    for ( j = 0 ; j < tailleh ; j ++ )
                    {
                        dcx = dcy ;
                        scx = scy ;
                        for ( i = 0 ; i < taillew ; i ++ , dcx ++ , scx ++ )
                        if ( BS->bits [ scx ] != couleur ) BD->bits [ dcx ] = BS->bits [ scx ] ;                               
                        dcy += BD->BPL >> 1 ;
                        scy += BS->BPL >> 1 ;
                    }
                }
            }
        }
                                   
        res = MMpush(m,s) ;

        /*********************** DEBUG 2D *****************************/
        #ifdef TRACE2D
            FDebug2D ( m ) ;
            MMechostr ( 1 , "DBG _copy_bitmap Done\n" ) ;
        #endif
        /****************************************************************/

        return res ;
}


/**************************************************************************************/
/*                                                                                    */
/*  int GRSetBitmapPalette ( mmachine m ) ;                                           */
/*                                                                                    */
/* correspond a la focntion magma ObjBitmap8 _set_bitmap_palette ( ObjBitmap8 , tab[i]*/
/* laquelle affecte une nouvelle palette a un objet bitmap palettee                   */
/*                                                                                    */
/**************************************************************************************/

int GRAffBitmapPalette ( mmachine m )
{

    int s , j  , i ;
    PtrObjVoid O ;
    PtrObjBitmap M ;
    struct Palette PalSource , PalDest ;
    unsigned int * Tablo ;
    int Convert [ 256 ] ;
    unsigned char * Image ;

    /*********************** DEBUG 2D ****************************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _set_bitmap_palette\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**************************************************************************/

    s = MMpull(m) ;
    if ( s == NIL ) return 0 ;
    i = MMsize ( m , s >> 1 ) ;
    if ( i != 768 )
    {
        MMechostr ( 1 , "The Palette Array haven't the good size == %d\n" , i ) ;
        return 0 ;
    }
    Tablo = ( unsigned int * ) MMstart(m,(s>>1) );

    s = MMget(m,0) ;
    if ( s == NIL ) 
    {
        MMset(m,0,0) ;
        return 0 ;
    }

    O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
    M = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;  

	if (M->bits == NULL)
	{
		MMechostr(MSKTRACE,"AffBitmapPalette : Bitmap already destroyed %x\n",M->handler);
        MMset(m,0,0) ;
		return 0;
	}

	//$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!M->DIBhandler) ObjBitmap_CreateDIBSection (M);


    i = 0 ;
    for ( j = 0 ; j < 256 ; j ++ )
    {
        GET_RED   (&PalDest,j) = Tablo [ i ++ ] >> 1;
        GET_GREEN (&PalDest,j) = Tablo [ i ++ ] >> 1;
        GET_BLUE  (&PalDest,j) = Tablo [ i ++ ] >> 1;
    }
    GetBitmapPalette ( M , &PalSource ) ;
    SetBitmapPalette ( M , &PalDest ) ;
    ProjectionPalette ( &PalDest , &PalSource , Convert ) ;

    Image = ( unsigned char * ) M->bits;

    for ( j = 0 ; j < M->TailleH ; j ++ )
    for ( i = 0 ; i < M->TailleW ; i ++ )
    Image [ i + j * M->BPL ] = Convert [ Image [i + j * M->BPL ] ] ;
   

    /************************* DEBUG 2D ********************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _get_bitmap_palette done\n" ) ;
    #endif
    /*******************************************************************************/
    
    return 0 ;
}






/*************************************************************************************/
/*                                                                                   */
/*  int GRGetBitma p( mmachine m ) ;                                                 */
/*                                                                                   */
/* correspond a la fonction S _get_bitmap ( ObjBitmap )                              */
/*                                                                                   */
/*************************************************************************************/

int GRGetBitmap ( mmachine m ) 
{
    int s , res , offset , l ; 
    PtrObjVoid O ;
    PtrObjBitmap B ;
    unsigned char * BS , *BB ;
    
    /***************************** DEBUG 2D ******************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG get_bitmap\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /************************************************************************************/
    
    s = MMpull(m) ;
    if ( s == NIL ) res = MMpush(m,NIL ) ;
    else
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
        B = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;
        if (( B->BPP == 8 ) && ( B->BPL == B->TailleW ))
            l = B->TailleW * B->TailleH ;
        else if ( B->BPP == 16 && B-> BPL == B->TailleW << 1 ) 
            l = B->TailleW * B->TailleH << 1 ;
        else 
        {
            MMechostr ( 1 , "Not accepted alliance of BPL %d BPP %d SizeW %d\n" ,
                B->BPP , B->BPL , B->TailleW ) ;
            return MMpush ( m , NIL ) ;
        }


        BB = ( unsigned char * ) B->bits;

		if (B->bits == NULL)
		{
			MMechostr(MSKTRACE,"GETnitmap : Bitmap already destroyed %x\n",B->handler);
			return MMpush(m,NIL);
		}

        res = MMmalloc (m,((l+4)>>2)+1,TYPEBUF) ;

        if ( res == NIL )
        {
             MMechostr ( 1 , "Not enough memory to store the bitmap of size %d" , l ) ;
             return MERRMEM ;
        }
        else 
        {
             *( int * ) MMstart(m, res ) = l ;
             BS = ( unsigned char * ) MMstart(m,res  + 1) ;
             for ( offset = 0 ; offset < l ; offset ++ )
             BS [offset ] = BB [offset ] ;
             BS [offset] = 0 ;
        }                            
        res = MMpush(m,( res<<1) + 1 ) ;
    }

    /******************************* DEBUG 2D *****************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG get_bitmap done\n" ) ;
    #endif
    /**********************************************************************************/
    return res ;
}





/*********************************************************************************************/
/*                                                                                           */
/*  int GRSetBitmap ( mmachine m )                                                           */
/*                                                                                           */
/* correspond a la fonction ObjBitmap _set_bitmap ( ObjBitmap  , S )                         */
/*                                                                                           */
/*********************************************************************************************/

  int GRSetBitmap ( mmachine m ) 
{
    int s , res , s2 , l , i , l2 ; 
    PtrObjVoid O ;
    PtrObjBitmap B ;
    unsigned char * Buf ;  
    
    /***************************** DEBUG 2D ******************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG get_bitmap\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /************************************************************************************/


    s2 = MMpull(m) ;
    s = MMpull(m) ;
    if (( s == NIL ) ||  ( s2 == NIL ))
    {
        MMechostr( 1 , "Bitmap or Buffer is NIL\n" ) ;
        return MMpush(m,s) ; 
    }

    O = ( PtrObjVoid ) MMstart(m,(s>>1 ) ) ;
    B = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;
	
	if (B->bits == NULL)
	{
		MMechostr(MSKTRACE,"SETbitmap : Bitmap already destroyed %x\n",B->handler);
		return MMpush(m,NIL);
	}

    l2 =  MMsizestr ( m , s2 >>1 ) ;
    if ( s == NIL ) res = MMpush(m,s) ;
    else
    {
        if ( B->BPP == 8 && B->BPL == B->TailleW )
            l = B->TailleW * B->TailleH ;
        else if ( B->BPP == 16 && ( B->BPL >> 1 ) == B->TailleW ) 
            l = B->TailleW * B->TailleH << 1 ;
        else 
        {
            MMechostr ( 1 , "Bad alliance of BPP %d BPL %d SizeW %d\n" ,
                    B->BPP , B->BPL , B->TailleW ) ;
            return MMpush(m,s) ;             
        }
        if ( l!= l2 ) MMechostr ( 1 , "Wrong size for the string. (bmp %d!= str %d)\n",l,l2 ) ;
		else
        {
            O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
            B = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;
            Buf = ( unsigned char * ) MMstart(m,(s2>>1) +  1) ;            
            for ( i = 0 ; i < l ; i ++ ) (( unsigned char * ) B->bits  )[ i ] = Buf [ i ] ;
        }        
    }
    res = MMpush(m,s) ;

    /******************************* DEBUG 2D *****************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG get_bitmap done\n" ) ;
    #endif
    /**************************************************************************************/

    return res ;
}








/***********************************************************************************************/
/*                                                                                             */
/*      int GRStretchBitmap ( mmachine m )                                                     */
/*                                                                                             */
/* correspond aux fonction de copy avec strectch _SCPbitmap                                    */
/* ( ObjBitmap dest , I dposx1 , I dposy1 , I dposx2 , I dposy2 , ObjBitmap src , I sposx1 ,   */
/* I sposY1 , I sposX2 , I sposY2 , I couleur ) ;                                              */
/*                                                                                             */
/***********************************************************************************************/

//#define DEBUG_STRETCH

int GRStretchBitmap ( mmachine m )
{
    int couleur , sposy2 , sposx2 , sposy1 , sposx1 , dposy2 , dposy1 , dposx2 , dposx1 ;
    int i , j ;
    int ss , sd , res ;
    int ddx , ddy , dsx , dsy , dex , dey  ;
    int ody , odx , ocy , ocx ;
    PtrObjVoid OS , OD ;
    PtrObjBitmap BS , BD ;

    /*************** DEBUG 2D ********************************************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _stretch_bitmap\n" ) ;
        FDebug2D ( m ); 
    #endif
    /*********************************************************************************************/

    couleur = MMpull(m) >> 1 ;   
    sposy2 = MMpull(m) >> 1 ;
    sposx2 = MMpull(m) >> 1 ;
    sposy1 = MMpull(m) >> 1 ;
    sposx1 = MMpull(m) >> 1 ;
    ss = MMpull(m) ;
    dposy2 = MMpull(m) >> 1 ;
    dposx2 = MMpull(m) >> 1 ;
    dposy1 = MMpull(m) >> 1 ;
    dposx1 = MMpull(m) >> 1 ;
    sd = MMpull(m) ;
    
    if ( ss == NIL || sd == NIL )
    {
        MMechostr ( 1 , "stretch _bitmap source or destination bitrmap is NIL\n" ) ;
        return MMpush(m,sd ) ;
    }

    OS = ( PtrObjVoid ) MMstart(m, ( ss >> 1 ) ) ;
    BS = ( PtrObjBitmap )  MMstart(m, ( OS->Buffer>>1) ) ;
    OD = ( PtrObjVoid ) MMstart(m, ( sd >> 1 ) ) ;
    BD = ( PtrObjBitmap ) MMstart(m, ( OD->Buffer>>1) ) ;

	if (BS->bits == NULL)
	{
		MMechostr(MSKTRACE,"STRETCHbitmap : Source Bitmap already destroyed %x\n",BS->handler);
		return MMpush(m,NIL);
	}

	if (BD->bits == NULL)
	{
		MMechostr(MSKTRACE,"STRETCHbitmap : Dest Bitmap already destroyed %x\n",BD->handler);
		return MMpush(m,NIL);
	}

    #ifdef DEBUG_STRETCH

        MMechostr ( 1 , "Avant Clip : Taille D %d %d S %d %d SRECT %d %d %d %d SD %d %d %d %d\n" ,
        BD->TailleW , BD->TailleH , BS->TailleW , BS->TailleH , sposx1 , sposy1 , sposx2 ,
        sposy2 , dposx1 , dposy1 , dposx2 , dposy2 ) ;

    #endif

    if ( ClipStretch ( BD->TailleW , BD->TailleH , BS->TailleW , BS->TailleH ,
        &dposx1 , &dposy1 , &dposx2 , &dposy2 , &sposx1 , &sposy1, &sposx2, &sposy2 ))

    {
    #ifdef DEBUG_STRETCH

        MMechostr ( 1 , "Apres Clip : Taille D %d %d S %d %d SRECT %d %d %d %d SD %d %d %d %d\n" ,
        BD->TailleW , BD->TailleH , BS->TailleW , BS->TailleH , sposx1 , sposy1 , sposx2 ,
        sposy2 , dposx1 , dposy1 , dposx2 , dposy2 ) ;

    #endif

    
      
        ddx = dposx2 - dposx1 + 1 ;
        ddy = dposy2 - dposy1 + 1 ;
        dsx = sposx2 - sposx1 + 1 ;
        dsy = sposy2 - sposy1 + 1 ;
        dex = 0 ;
        dey = 0 ;
        
    #ifdef DEBUG_STRETCH
        MMechostr ( 1 , "Deltas values : ddx%d ddy %d dsx %d dsy %d\r\n",ddx,ddy,dsx,dsy);
    #endif

        if (( BD->BPP == 8 ) && ( BS->BPP == 8 )) 
        {
            ody = dposy1 * BD->BPL + dposx1 ;
            ocy = sposy1 * BS->BPL + sposx1 ;

            for ( j = dposy1 ; j <= dposy2 ; j ++ )
            {
                odx = ody ;
                ocx = ocy ;
                dex = 0 ;
                for ( i = dposx1 ; i <= dposx2 ; i ++ )
                {
                    if ( couleur != (( unsigned char * ) BS->bits) [ ocx ] )
                    (( unsigned char * ) BD->bits) [ odx ] = 
                    (( unsigned char * ) BS->bits) [ ocx ] ;
                    odx ++ ;
                    dex += dsx ;
                    while ( dex >= ddx ) 
                    {
                        ocx ++ ;
                        dex -= ddx ;
                    }
                }
                ody += BD->BPL ;
                dey += dsy ;
                while ( dey >= ddy )
                {
                    ocy += BS->BPL ;
                    dey -= ddy ;
                }
            }
        }
        else if (( BS->BPP == 16 ) && ( BD->BPP == 16 ))
        {

            ody = ((( dposy1) * BD->BPL ) >> 1 ) + dposx1 ;
            ocy = ((( sposy1 ) * BS->BPL ) >> 1 )+ sposx1 ;
            if ( couleur != NIL ) couleur = Color24To16 ( couleur ) ;

        
            for ( j = dposy1 ; j <= dposy2 ; j ++ )
            {
                odx = ody ;
                ocx = ocy ;
                dex = 0 ;
                for ( i = dposx1 ; i <= dposx2 ; i ++ )
                {
                    if ( couleur != BS->bits [ocx ] ) 
                    BD->bits [ odx ] = BS->bits [ ocx ] ;
                    odx ++ ;
                    dex += dsx ;
                    while ( dex >= ddx ) 
                    {
                        ocx ++ ;
                        dex -= ddx ;
                    }
                }
                ody += ( BD->BPL >> 1 ) ;
                dey += dsy ;
                while ( dey >= ddy )
                {
                    ocy += ( BS->BPL >> 1 ) ;
                    dey -= ddy ;
                }
            }
        }
    }
    res = MMpush(m,sd ) ;


    /*********************************** DEBUG 2D ********************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _stretch_bitmap done\n" ) ;
    #endif
    /******************************************************************************************/

    return res ;
}







/************************************************************************************/
/*                                                                                  */
/*  int GRMapBitmap ( mmachine m ) ;                                                */
/*   correspond a la fonction Objbitmap _map_bitmap ( ObjBitmap Destination         */
/* I x1 , I y1 , I x2 , I y2 , I x3 , I y3 , I x4 , I y4 , ObjBitmap Source         */
/*  I sx1 , I sy1 , I Taillew , I TailleH , I couleur )                             */
/*                                                                                  */
/************************************************************************************/

//#define DEBUG_2DMAP ;
int GRMapBitmap ( mmachine m ) 
{
    int xd [ 4 ] ;
    int yd [ 4 ] ;
    int xs [ 4 ] ;
    int ys [ 4 ] ;
    int couleur , res , ss , sd ; 
    char lps , lpe , rps , rpe ;
  
    int xdl ,       /* x courant sur le cote LEFT du polygone destination  */
        ydl ,       /* y courant sur le cote LEFT du polygone destination  */
        xdr ,       /* x courant sur le cote RIGHT du polygone destiantion */
        ydr ,       /* y courant sur le cote RIGHT du polygone destination */
        xsl ,       /* x courant cote LEFT de la map source                */
        ysl ,       /* y courant cote LEFT de la map source                */
        xsr ,       /* x courant cote RIGHT map source                     */
        ysr ,       /* y courant cote RIGHT map source                     */
        deltaleft , /* points verticaux a tracer , polygone dest , LEFT    */
        points ,    /* points a tracer avant que le polygone soit complet  */
        dlx ,       /* delta X cote left polygone destination              */
        dly ,       /* delta Y cote ledt polygone destination              */
        drx ,       /* delta X cote right polygone destination             */
        dry ,       /* delta Y cote right polygone destination             */
        deltaright ,/* points verticaux a tracer , polygone dest , RIGHT   */
        erl ,       /* terme d'erreur pour le pas X , polygone dest LEFT   */
        err ,       /* terme d'erreur pour le pas X , polygone dest , RIGHT*/
        deplx ,     /* pas de deplacement X poly dest , LEFT               */
        deprx ,     /* pas de deplacement Y poly dest , RIGHT              */
        emlx ,      /* terme d'erreur MAP LEFT X                           */
        emly ,      /* terme d'erreur MAP LEFT Y                           */
        emrx ,      /* terme d'erreur MAP RIGHT X                          */
        emry ,      /* terme d'erreur MAP RIGHT Y                          */
        movlx ,     /* pas X MAP LEFT                                      */
        movly ,     /* pas Y MAP LEFT                                      */
        movrx ,     /* pas X MAP RIGHT                                     */
        movry ,     /* pas Y MAP RIGHT                                     */
        delslx ,    /* delta X MAP LEFT                                    */
        delsly ,    /* delta Y MAP LEFT                                    */
        delsrx ,    /* delta X MAP RIGHT                                   */
        delsry ,    /* delta Y MAP RIGHT                                   */
        delmapx,    /* delta X MAP MAPPING                                 */
        delmapy,    /* delta Y MAP MAPPING                                 */
        movmapx,    /* pas X MAP MAPPING                                   */
        movmapy,    /* pas Y MAP MAPPING                                   */
        delmap ,    /* taille du segment a mapper                          */
        movmap,     /* sens                                                */
        xdest ,     /* x du MAPPING                                        */
        mapx ,      /* X MAP MAPPING                                       */
        mapy ,      /* Y MAP MAPPING                                       */
        maperrx ,   /* terme d'errreur X du MAPPER                         */
        maperry ,   /* terme d'erreur Y du MAPPER                          */
        offread ,   /* offset de lecture                                   */
        i ;

    PtrObjVoid OS , OD ;
    PtrObjBitmap BS , BD ;

    /*************************** DEBUG 2D *******************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _map_bitmap\n" ) ;
        FDebug2D ( m );
    #endif
    /********************************************************************************/

    /***************** LECTURE DES VALEURS DANS LA PILE ******************************/
    couleur = MMpull ( m ) >> 1 ;
    if ( couleur != NIL ) couleur = Color24To16 ( couleur ) ;
    ys [ 2 ] = MMpull ( m ) >> 1 ;
    xs [ 2 ] = MMpull ( m ) >> 1 ;
    ys [ 0 ] = MMpull ( m ) >> 1 ;
    xs [ 0 ] = MMpull ( m ) >> 1 ;
    ss = MMpull ( m ) ;
    yd [ 3 ] = MMpull ( m ) >> 1 ;
    xd [ 3 ] = MMpull ( m ) >> 1 ;
    yd [ 2 ] = MMpull ( m ) >> 1 ;
    xd [ 2 ] = MMpull ( m ) >> 1 ;
    yd [ 1 ] = MMpull ( m ) >> 1 ;
    xd [ 1 ] = MMpull ( m ) >> 1 ;
    yd [ 0 ] = MMpull ( m ) >> 1 ;
    xd [ 0 ] = MMpull ( m ) >> 1 ;
    sd = MMpull ( m ) ;

    if (( ss == NIL ) || ( sd == NIL ))
            MMechostr ( 1 , "Source or Destination bitmap is NIL\n" ) ;
    else
    {
        OS = ( PtrObjVoid ) MMstart(m,(ss>>1) ) ;
        OD = ( PtrObjVoid ) MMstart(m,(sd>>1) ) ;
        BS = ( PtrObjBitmap ) MMstart(m,(OS->Buffer>>1) ) ;
        BD = ( PtrObjBitmap ) MMstart(m,(OD->Buffer>>1) ) ;

		if (BS->bits == NULL)
		{
			MMechostr(MSKTRACE,"STRETCHbitmap : Source Bitmap already destroyed %x\n",BS->handler);
			return MMpush(m,NIL);
		}

		if (BD->bits == NULL)
		{
			MMechostr(MSKTRACE,"STRETCHbitmap : Dest Bitmap already destroyed %x\n",BD->handler);
			return MMpush(m,NIL);
		}

        /* refaire les points pour le 2eme rectangle */
        xs [ 1 ] = xs [ 0 ] ;
        ys [ 1 ] = ys [ 2 ] ;     
        xs [ 3 ] = xs [ 2 ] ;
        ys [ 3 ] = ys [ 0 ] ;

        /* recherche du point le plus haut */
        lps = 0 ;
        for ( lpe = 1 ; lpe <= 3 ; lpe ++ )
        if ( yd [ lpe ] < yd [ lps ] ) lps = lpe ;

        /************** CALCUL DES POINTS COURANT - DESTINATION ************************/
        rps = lps ;
        lpe = lps - 1 ;
        if ( lpe < 0 ) lpe = 3 ;
        rpe = rps + 1 ;
        if ( rpe >= 4 ) rpe = 0 ;

        xdl = xd [ lps ] ;
        ydl = yd [ lps ] ;
        xdr = xd [ lps ] ;
        ydr = yd [ lps ] ;

        xsl = xs [ lps ] ;
        ysl = ys [ lps ] ;
        xsr = xs [ lps ] ;
        ysr = ys [ lps ] ;   

        /************** CALCUL DES DELTAS - DESTINATION ***********************************/
        dlx = ( xd [ lpe ] - xd [ lps ] ) ;
        dly = ( yd [ lpe ] - yd [ lps ] ) ;
        drx = ( xd [ rpe ] - xd [ rps ] ) ;
        dry = ( yd [ rpe ] - yd [ rps ] ) ;
        if ( dlx < 0 )
        {
            deplx = -1 ; dlx = - dlx ; 
        } else deplx = 1 ;

        if ( drx < 0 )
        {
             drx = - drx ;
             deprx = -1 ;
        } else deprx = 1 ;

        #ifdef DEBUG_2DMAP
            MMechostr ( 1 , "DEST LEFT : go from %d <%d,%d> to %d<%d,%d>\n" ,
                        lps , xd [ lps ] , yd [ lps ] , lpe , xd [ lpe ] , yd [ lpe ]) ;
            MMechostr ( 1 , "DEST : RIGHT : go from %d <%d,%d> to %d<%d,%d>\n" ,
                        rps , xd [ rps ] , yd [ rps ] , rpe , xd [ rpe ] , yd [ rpe ]) ;
        #endif
        if (( dly < 0 ) || ( dry < 0 ))
        {
            res = MMpush(m,sd ) ;
            MMechostr ( 1 , "Polygone concave \n" ) ;
            return res ;
        }

        deltaleft = dly ;        
        deltaright = dry ;

        /************* INIT DES TERMES D'ERREURS ********************************/
        erl = 0 ; 
        err = 0 ;
        emlx = 0 ;
        emly = 0 ;
        emrx = 0 ;
        emry = 0;

        /************* CALCUL DES DELTA - SOURCE ****************************/
        delslx  = xs [ lpe ] - xs [ lps ] ;
        if ( delslx < 0 )
        {
            delslx = - delslx ;
            movlx = -1 ;
        } else movlx = 1 ;

        delsly = ys [ lpe ] - ys [ lps ] ;
        if ( delsly < 0 )
        {
            delsly = - delsly ;
            movly = -1 ;
        } else movly = 1 ;

        delsrx = xs [ rpe ] - xs [ rps ] ;
        if ( delsrx < 0 )
        {
            delsrx = - delsrx ;
            movrx = -1 ;
        } else movrx = 1 ; 

        delsry = ys [ rpe ] - ys [ rps ] ;
        if ( delsry < 0 )
        {
            delsry = - delsry ;
            movry = -1 ;
        } else movry = 1 ;

        points = 3 ;      
        #ifdef DEBUG_2DMAP                
                    MMechostr ( 1 , "MAP LEFT go from <%d,%d> to <%d,%d>\n",
                        xs [ lps ] , ys [ lps ] , xs [ lpe ] , ys [ lpe ] ) ;                    
                    MMechostr ( 1 , "MAP RIGHT : go from <%d,%d> to <%d,%d>\n" ,
                        xs [ rps ] , ys [ rps ] , xs [ rpe ] , ys [ rpe ] ) ;
        #endif        
        /******************** BOUCLE PRINCIPALE *********************************/
        while ( points != 0 )
        {
            /************** MESSAGE DEBUGGAGE **********************************/
            #ifdef DEBUG_2DMAP
                MMechostr ( 1 , "LEFT : del %d lps %d lpe %d xdl %d ydl %d xsl %d ysl %d\n",
                    deltaleft , lps , lpe , xdl , ydl , xsl , ysl ) ;               
                MMechostr ( 1 , "RIGHT:  del %d rps %d rpe %d xdr %d ydr %d xsr %d ysr %d\n" ,
                    deltaright , rps , rpe , xdr , ydr , xsr , ysr ) ;
                MMechostr ( 1 , "MAP : xl %d yl %d xr %d yr %d\n" , xsl , ysl , xsr , ysr ) ;
                MMechostr ( 1 , "MAP : DLX %d DLY %d DRX %d DRY %d ERLX %d ERLY %d ERRX %d ERRY %d\n",
                        delslx , delsly , delsrx , delsry , emlx , emly , emrx , emry ) ;
                MMechostr ( 1 , "MAP : MLX %d MLY %d MRX %d MRY %d\n" ,
                        movlx , movly , movrx , movry ) ;
            #endif
            
            /* effectue le mapping de la ligne xsl,ysl - xsr,ysr */
                /* sur la ligne xdl , ydl , xdr , ydr */
            /* test de synchronisation des cote LEFT et RIGHT */
            if ( ydl != ydr )
            {
                MMechostr ( 1 , "Y not synchro LEFT %d RIGHT %d\n" , ydl , ydr ) ;
                break ;
            }
            /* test de clipping Y */
            if ( ydl > 0 && ydl < BD->TailleH )
            {

                /* regarde le sens */
                delmap = xdr - xdl ;
                if ( delmap < 0 )
                {
                    movmap = -1 ;
                    delmap = - delmap ;                
                } else movmap = 1 ;
                                 
                delmapx = xsr - xsl ;
                delmapy = ysr - ysl ;
                if ( delmapx < 0 )
                {
                    delmapx = - delmapx ;
                    movmapx = -1 ;
                } else movmapx = 1 ;

                if ( delmapy < 0 )
                {
                    delmapy = - delmapy ;
                    movmapy =-1 ;
                } else movmapy = 1 ;

                xdest = xdl ;
                mapx = xsl ;
                mapy = ysl ;
                maperrx = 0 ;
                maperry = 0 ;
                #ifdef DEBUG_2DMAP
                    MMechostr ( 1 , "LDX %d LDY %d LSX %d LSY %d RDX %d RDY %d RSX %d RSY %d\n" ,
                        xdl , ydl , xsl , ysl , xdr , ydr , xsr , ysr ) ;
                #endif
                for ( i = 0 ; i < delmap ; i ++ )
                {
                    if ( xdest > 0 && xdest < BD->TailleW )
                    if ( mapx > 0 && mapx < BS->TailleW )
                    if ( mapy > 0 && mapy < BS->TailleH )
                    {                                            
                        offread = BS->bits [ mapx + mapy * ( BS->BPL >> 1 ) ] ;
                        if ( offread != couleur )
                            BD->bits [ xdest + ydl * ( BD->BPL >> 1 ) ] = offread ;
                    }
                    xdest += movmap ;
                    maperrx += delmapx ;
                    while ( maperrx > delmap )
                    {
                        mapx += movmapx ;
                        maperrx -= delmap ;
                    }
                    maperry += delmapy ;
                    while ( maperry > delmap )
                    {
                        mapy += movmapy ;
                        maperry -= delmap ;
                    }
                }
            } else if ( ydl >= BD->TailleH ) break ;
            
            /* TEST FIN DE PARCOURS SEGMENT GAUCHE ******************/
            while ( !deltaleft )
            {
                points -- ;
                if ( ! points ) break ;
                lps = lpe ;
                lpe -- ;
                if ( lpe < 0 ) lpe = 3 ;
                #ifdef DEBUG_2DMAP
                    MMechostr ( 1 , "DEST LEFT : go from %d <%d,%d> to %d<%d,%d>\n" ,
                        lps , xd [ lps ] , yd [ lps ] , lpe , xd [ lpe ] , yd [ lpe ]) ;
                    MMechostr ( 1 , "MAP LEFT go from <%d,%d> to <%d,%d>\n",
                        xs [ lps ] , ys [ lps ] , xs [ lpe ] , ys [ lpe ] ) ;
                #endif
                xdl = xd [ lps ] ;
                ydl = yd [ lps ] ;
                xsl = xs [ lps ] ;
                ysl = ys [ lps ] ;
                dlx = ( xd [ lpe ] - xd [ lps ] ) ;
                dly = ( yd [ lpe ] - yd [ lps ] ) ;
                if ( dlx < 0 )
                {
                    deplx = -1 ; dlx = - dlx ; 
                } else deplx = 1 ;

                if ( dly < 0 )
                {

                    res = MMpush(m,sd ) ;
                    MMechostr ( 1 , "Polygone concave  ys %d ye %d \n", yd [ lps ] , yd [ lpe ] ) ;
                    return res ;
                }
                deltaleft = dly ;
                erl = 0 ;
                emlx = 0 ;
                emly = 0 ;
                delslx  = xs [ lpe ] - xs [ lps ] ;
                if ( delslx < 0 )
                {
                    delslx = - delslx ;
                    movlx = -1 ;
                } else movlx = 1 ;

                delsly = ys [ lpe ] - ys [ lps ] ;
                if ( delsly < 0 )
                {
                    delsly = - delsly ;
                    movly = -1 ;
                } else movly = 1 ;
            }

            /* AVANCEMENT SEGMENT  GAUCHE ******************************/
            if ( ! points ) break ;
            deltaleft -- ;
            ydl ++ ;
            erl += dlx ;
            while ( erl >= dly ) 
            {
                erl -= dly ; xdl += deplx ;
            }
            if ( delslx )
            {
                emlx += delslx ;
                while ( emlx >= dly )
                {
                    emlx -= dly ; xsl += movlx ;
                }
            }
            if ( delsly )
            {
                emly += delsly ;
                while ( emly >= dly )
                {
                    emly -= dly ; ysl += movly ;
                }
            }
            
            /* TEST FIN AVANCEMENT SEGMENT DROIT **********************/
            while ( !deltaright )
            {
                points -- ;
                if ( ! points ) break ;
                rps = rpe ;
                rpe ++ ;
                if ( rpe >= 4 ) rpe = 0 ;
                #ifdef DEBUG_2DMAP
                    MMechostr ( 1 , "DEST : RIGHT : go from %d <%d,%d> to %d<%d,%d>\n" ,
                        rps , xd [ rps ] , yd [ rps ] , rpe , xd [ rpe ] , yd [ rpe ]) ;
                    MMechostr ( 1 , "MAP RIGHT : go from <%d,%d> to <%d,%d>\n" ,
                        xs [ rps ] , ys [ rps ] , xs [ rpe ] , ys [ rpe ] ) ;
                #endif
                ydr = yd [ rps ] ;
                xdr = xd [ rps ] ;
                xsr = xs [ rps ] ;
                ysr = ys [ rps ] ;
                drx = ( xd [ rpe ] - xd [ rps ] ) ;
                dry = ( yd [ rpe ] - yd [ rps ] ) ;
                if ( drx < 0 )
                {
                     drx = - drx ;
                     deprx = -1 ;
                } else deprx = 1 ;
                if (dry < 0 )
                {
                    res = MMpush(m,sd ) ;
                    MMechostr ( 1 , "Polygone concave yds %d yde %d\n", yd [ rpe ] , yd [ rps ] ) ;
                    return res ;
                }
                deltaright = dry ;
                err = 0 ;
                emrx = 0 ; 
                emry = 0 ;
                delsrx = xs [ rpe ] - xs [ rps ] ;
                if ( delsrx < 0 )
                {
                    delsrx = - delsrx ;
                    movrx = -1 ;
                } else movrx = 1 ; 

                delsry = ys [ rpe ] - ys [ rps ] ;
                if ( delsry < 0 )
                {
                    delsry = - delsry ;
                    movry = -1 ;
                } else movry = 1 ;
            }

            /* AVANCE SUR LE SEGMENT DROIT **********************/
            if ( ! points ) break ;
            deltaright -- ;
            ydr ++ ;
            err += drx ;
            while ( err >= dry )
            {
                err -= dry ; xdr += deprx ;
            }
            if ( delsrx )
            {
                emrx += delsrx ;
                while ( emrx >= dry )
                {
                    emrx -= dry ; xsr += movrx ;
                }   
            }
            if ( delsry )
            {
                emry += delsry ;
                while ( emry >= dry )
                {
                    emry -= dry ; ysr += movry ;
                }
            }
        }
        /**************** FIN BOUCLE PRINCIPALE **************************/

    }
   

    res = MMpush( m , sd ) ;
    /********************** DEBUG 2D **************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _map_bitmap dione\n" ) ;
    #endif
    /***********************************************************************************/

    return res ;



}


/******************************************************************************************/
/*                                                                                        */
/*    int GRFILEbitmap ( mmachine m );                                                    */
/*                                                                                        */
/*  correspond a la fonction magma S _FILEbitmap ( ObjBitmap )                            */
/*  laquelle fabrique un buffer archivable                                                */
/*                                                                                        */
/******************************************************************************************/

int GRFileBitmap ( mmachine m )
{
    int res ;
    int s , l , offset , i , j , l2 ;
    PtrObjVoid O ;
    PtrObjBitmap B ;
    unsigned char * Bu , * Bub ;
    unsigned short int * Bu16 ;
    BITMAPFILEHEADER * Bfh ;
    BITMAPINFOHEADER * Bih ;
    struct Palette Pal ;
    RGBQUAD * Rgb ;
    /***************************** DEBUG 2D ************************************************/
    #ifdef  TRACE2D
        MMechostr ( 1 , "DBG _file_bitmap\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /****************************************************************************************/

    s = MMpull(m ) ;
    if ( s== NIL ) 
    res = MMpush(m,NIL ) ;
    else
    {
        O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
        B = ( PtrObjBitmap ) MMstart(m, ( O->Buffer>>1) ) ;
        if ( B->BPP == 16 ) l = 1 + sizeof ( BITMAPFILEHEADER ) + sizeof ( BITMAPINFOHEADER ) +
                B->BPL * B->TailleH ;
        else if ( B->BPP == 8 ) l = 1 + sizeof ( BITMAPFILEHEADER ) + sizeof ( BITMAPINFOHEADER ) +
            B->BPL * B->TailleH + 1024 ;
        else 
        {
            l = 0 ;
            MMechostr ( 1 , "_file_bitmap ; unsupported BPP %d\n" , B->BPP ) ;
        }
        if ( l > 0 )
        {
            if (l%3==0) l2 = l>>2 ; else l2 = (l+4)>>2 ;
            MMechostr(MSKDEBUG,"Taille de base l = %d, l2 = %d\n",l,l2);
            res = MMmalloc (m,l2+1,TYPEBUF) ;

            if ( res == NIL )
            {
                 MMechostr ( 1 , "Not enough memory to create the bitmap file of size %d" , l ) ;
                return MERRMEM ;
            }
            else 
            {

                    *( int * ) MMstart(m, res ) = l ;
                    /* ecriture du file header */
                    Bu = ( unsigned char * ) MMstart(m,res  + 1) ;
                    Bfh = ( BITMAPFILEHEADER * ) Bu ;
                    Bfh -> bfType = FILE_TYPE_BMP ;
                    Bfh->bfSize = l ;
                    Bfh->bfReserved1 = 0 ;
                    Bfh->bfReserved2 = 0 ;
                    Bfh->bfOffBits  = sizeof ( BITMAPFILEHEADER ) + sizeof ( BITMAPINFOHEADER ) ;
                    if( B->BPP == 8 ) Bfh->bfOffBits += 1024 ;
                    /* ecriture du ifno header */
                    Bih = ( BITMAPINFOHEADER * ) ( Bu + sizeof ( BITMAPFILEHEADER )) ;
                    Bih->biSize = sizeof ( BITMAPINFOHEADER ) ;
                    Bih->biWidth = B->TailleW ;
                    Bih->biHeight = B->TailleH ;
                    Bih->biPlanes = 1 ;
                    Bih->biBitCount = B->BPP ;
                    Bih->biCompression = 0 ;
                    Bih->biSizeImage = B->BPL * B->TailleH ;
                    Bih->biXPelsPerMeter = 0 ;
                    Bih->biYPelsPerMeter = 0 ;
                    if ( B->BPP == 8 ) Bih->biClrUsed = 256 ;
                    else Bih->biClrUsed = 0 ;
                    Bih->biClrImportant = Bih->biClrUsed ;
                    /* ecriture des couleurs */
                    if ( B->BPP == 8 )
                    {
                        GetBitmapPalette ( B , &Pal ) ;
                        for ( i = 0 ; i < 256 ; i ++ )
                        {
                            Rgb =  ( RGBQUAD * ) & (Bu [ 54 + (  i << 2 )]) ;
                            Rgb->rgbBlue = GET_BLUE(&Pal,i) ;
                            Rgb->rgbGreen = GET_GREEN(&Pal,i) ;
                            Rgb->rgbRed = GET_RED(&Pal,i) ;
                            Rgb->rgbReserved = 0 ;
                        }
                    }

                    /* ecriture du bitmap */
                    if ( B->BPP == 8 )
                    {
                        offset = B->BPL * B->TailleH ;
                        Bub = Bu + Bfh -> bfOffBits ;
                        for ( j = 0 ; j < B->TailleH ; j ++ )
                        for ( i = 0 ; i < B->BPL ; i ++ )
                            Bub [ i + j * B->BPL ] = 
                            (( unsigned char * ) B->bits ) [ i + ( offset - (j+1) * B->BPL) ] ;
                    }
                    if ( B->BPP == 16 )
                    {
                        offset = (B->BPL>>1) * B->TailleH ;
                        Bu16 = ( unsigned short int * ) ( Bu + Bfh->bfOffBits ) ;
                        for ( j =0 ; j < B->TailleH ; j ++ )
                        for ( i = 0 ; i < (B->BPL>>1) ; i ++ )
                            Bu16 [ i + j * ( B->BPL>>1) ] = B->bits [ i + ( offset - ( j+1) * ( B->BPL>>1)) ] ;
                    }


                    /* 0 terminal */
                    Bu [ l - 1 ] = 0 ;

                    res = MMpush(m,(res << 1) + 1) ;
            }
        }   
        else res = MMpush(m,NIL ) ;
        

        
    }

    /***************************** DEBUG 2D***************************************************/
    #ifdef  TRACE2D
        FDebug2D ( m ); 
        MMechostr ( 1 , "DBG _file_bitmap done\n"  );
    #endif
    /*****************************************************************************************/
    return res ;

        
}


/**********************************************************************************************/
/*                                                                                            */
/*  int GRCopyWinBitmap ( mmachine m ) ;                                                      */
/*                                                                                            */
/*  correspond a la fonction magma ObjWin _copybitmap_to_win ( ObjWin , I , I , ObjBitmap , I */
/*  I I I ) ;                                                                                 */
/*                                                                                            */
/**********************************************************************************************/

int GRCopyWinBitmap ( mmachine m )
{
    int sx1 , sy1 , sx2 , sy2 , ss , dx , dy , sd , res ;
    PtrObjVoid OS , OD ;
    PtrObjBitmap B ;
    PtrObjWindow W ;
    HDC Dcb , Dcw ;
    HBITMAP OldBitmap ;


    /************************************ DEBUG 2D *********************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _copy_bitmap_to_win\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /********************************************************************************************/

    sy2 = MMpull(m) >> 1 ;
    sx2 = MMpull(m) >> 1 ;
    sy1 = MMpull(m) >> 1 ;
    sx1 = MMpull(m) >> 1 ;
    ss = MMpull(m) ;
    dy = MMpull(m) >> 1 ;
    dx = MMpull(m) >> 1 ;
    sd = MMpull(m) ;
   
    if ( ss == NIL || sd == NIL )
    {
        MMechostr ( 1 , "_copy_bitmap_to_win : The window Obj or the bitmap is NIL\n" ) ;
         return MMpush(m,sd) ;
    }

 
    OS = ( PtrObjVoid ) MMstart(m,(ss>>1) ) ;
    OD = ( PtrObjVoid ) MMstart(m,(sd>>1) ) ;
    B = ( PtrObjBitmap ) MMstart(m,(OS->Buffer>>1) ) ;
    W = ( PtrObjWindow ) MMstart(m,(OD->Buffer>>1) );

					
	//$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);


    Dcw = GetDC ( W->WHandler ) ;       
    Dcb = CreateCompatibleDC ( Dcw ) ;
    OldBitmap = SelectObject ( Dcb , B->DIBhandler) ;
    BitBlt ( Dcw , dx , dy , sx2 , sy2 , Dcb , sx1 , sy1 , SRCCOPY ) ;
 
    SelectObject ( Dcb , OldBitmap ) ;
    DeleteDC ( Dcb ) ;    

    res = MMpush(m,sd) ;


    /********************************* DEBUG 2D ***************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG copy_bitmap_to_win done\n" ) ;
    #endif
    /**********************************************************************************************/

    return res ;
}





/****************************************************************************************************/
/*                                                                                                  */
/*  int GRGetPixel ( mmachine m ) ;                                                                 */
/*                                                                                                  */
/* donne la couleur d'un pixel d'un bitmap au format 24 bits                                        */
/* correspond a la fonction magma I _get_pixel ( Obj X Y  )                                         */
/*                                                                                                  */
/****************************************************************************************************/

int GRGetPixel ( mmachine m ) 
{
    int s , x , y ;
    PtrObjVoid O ;
    PtrObjBitmap B ;
    struct Palette MaPal ;

    /*************************** DEBUG 2D **********************************************************/
    #ifdef TRACE2D  
        MMechostr ( 1 , "DBG GETpixel\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /************************************************************************************************/

    y = MMpull ( m ) >> 1 ;
    x = MMpull ( m ) >> 1 ;
    s = MMpull ( m ) ;
    if ( s == NIL )
    {
        MMechostr ( 1 , "Bitmap is NIL in _GETPixel\n" ) ;
        return MMpush ( m , NIL ) ;
    } ;

    O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
    B = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;

	if (B->bits == NULL)
	{
		MMechostr(MSKTRACE,"GETpixel : Bitmap already destroyed %x\n",B->handler);
		return MMpush(m,NIL);
	}

    if ( x < 0 || x >= B->TailleW || y < 0 ||y >= B->TailleH )
    {
        MMechostr ( 1 , "Coordonates outside bitmap in _GETpixel\n" ) ;
        return MMpush(m,NIL);    
    }
   
    if ( B->BPP == 16 )
    {       
       s = B->bits [ x + y * ( B->BPL >> 1 ) ] ;          
       s = Color16To24(s);
       s = MMpush(m,s<<1);

    }
    else if ( B->BPP == 8 )
    {
        
        s = (( unsigned char * ) B->bits  ) [ x + y * B->BPL ] ;
        GetBitmapPalette ( B , & MaPal ) ;
       
        s = GET_RED(&MaPal,s) + ( GET_GREEN(&MaPal,s) << 8 ) + ( GET_BLUE(&MaPal,s) << 16 ) ;
      
        s = MMpush(m,s<<1) ;
    } else MMpush(m,NIL ) ;

    /************************** DEBUG 2D *************************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG GETpixel done\n" ) ;
    #endif
    /**************************************************************************************************/

    return s ;

}





/*******************************************************************************************/
/*                                                                                         */
/*  int GRDrawPoly ( mmachine m ) ;                                                        */
/*                                                                                         */
/*      Cette fonction correspond a la fonction magma _draw_poly ,                         */
/*  (ObjBitmap , tab [ I I ] , I mode contour , I taille contour , I couleur contour       */
/*  I mode fill,  I couleur fill ) laquelle dessine un polygone plein dans un bitmap       */
/*                                                                                         */
/*******************************************************************************************/

int GRDrawPoly ( mmachine m ) 
{
    int couleur_fill, mode_fill , couleur_contour, taille_contour , mode_contour , tablo , s ,
        np , i , ti , np2 ;
    PtrObjVoid O ;
    PtrObjBitmap B ;
    POINT p [ 1024 ] ;
    HPEN Pinceau , OldPen ;
    HDC Dc ;
    HWND Item , OldItem ;
    LOGBRUSH Brush ;
    HBRUSH HBrush , OldBrush ;


    /***************************** DEBUG 2D *********************************************/
    #ifdef DEBUG2D
        MMechostr ( 1 , "DBG DrawPoly\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /***************************************************************************************/

    couleur_fill = MMpull ( m ) >> 1 ;
    mode_fill = MMpull(m) >> 1 ;
    couleur_contour = MMpull(m) >> 1 ;
    taille_contour = MMpull(m) >> 1 ;
    mode_contour = MMpull(m) >> 1 ;

    tablo = MMpull(m) ;
    np2 = MMpull(m) >> 1 ;
    s = MMpull(m) ;

    if ( tablo == NIL || s == NIL )
    {
        MMechostr ( 1 , "(ERR) _DRAWpoly: Array of points empty or object bitmap nil\n" ) ;
        return MMpush(m,NIL);
    }

    O = (PtrObjVoid)  MMstart(m,(s>>1) ) ;
    B = ( PtrObjBitmap )MMstart(m,(O->Buffer>>1) ) ;   

    //$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

   
    np = MMsize ( m , tablo >> 1 ) ;
    if ( np2 > 1024 || np2 <= 0 || np2 > np)
    {
        MMechostr ( 1 , "(ERR) _Drawpoly : polygon size is wrong. ARRAY SIZE = %d , POINTS = %d \n",np,np2 ) ;
        return MMpush(m,s);
    }
    for ( i = 0 ; i < np2 ; i ++ )
    {
        ti = *MMstart(m, ( tablo >> 1 ) + i ) ;
        p [ i ] . x = ( *MMstart(m, ( ti >> 1 ) ) ) >> 1 ;
        p [ i ] . y = ( *MMstart(m, ( ti >> 1 ) + 1) ) >> 1 ;
    }

    Dc = CreateCompatibleDC ( NULL ) ;

    Item = (HWND)B->DIBhandler;


    if ( taille_contour <= 0 ) { taille_contour = 0 ; mode_contour = DRAW_INVISIBLE; }

    if ( mode_contour == DRAW_INVISIBLE )
        Pinceau = CreatePen ( PS_NULL  , taille_contour , couleur_contour ) ;
    else Pinceau = CreatePen ( PS_SOLID , taille_contour , couleur_contour ) ;
    OldPen = SelectObject ( Dc , Pinceau ) ;
    OldItem = SelectObject ( Dc , Item ) ;
    if ( mode_fill == DRAW_INVISIBLE ) Brush.lbStyle = BS_NULL ;
    else Brush.lbStyle = BS_SOLID ;
    Brush.lbColor = couleur_fill ;
      
    HBrush = CreateBrushIndirect ( & Brush ) ;
    OldBrush = SelectObject ( Dc , HBrush ) ;

    Polygon ( Dc , p , np2 ) ;
    SetPolyFillMode ( Dc , ALTERNATE ) ;

    SelectObject ( Dc , OldItem ) ;
    OldPen = SelectObject ( Dc , OldPen ) ;
    OldBrush = SelectObject ( Dc , OldBrush ) ;
    DeleteDC ( Dc ) ;
    DeleteObject ( OldPen ) ;
    DeleteObject ( OldBrush ) ;
    s = MMpush(m,s) ;

    /************************** DEBUG 2D *****************************************************/
    #ifdef DEBUG2d
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG draw_poly DONE\n" ) ;
    #endif
    /******************************************************************************************/
    
    return s ;

}





/****************************************************************************************/
/*                                                                                      */
/*  int GRPutPixel ( mmachine m ) ;                                                     */
/*                                                                                      */
/*  Correspond a la fonction magma ObjBitmap _PUTpixel ObjBitmap I I I                  */
/*  laquelle definit la couleur d'un pixel dans un objet bitmap                         */
/*                                                                                      */
/****************************************************************************************/

int GRPutPixel ( mmachine m )
{
    int s , x , y , c ;
    PtrObjVoid O ;
    PtrObjBitmap B ;

    /********************* DEBUG 2D *****************************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _PUTpixel\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /************************************************************************************/

    c = MMpull(m) >> 1 ;
    y = MMpull(m) >> 1 ;
    x = MMpull(m) >> 1 ;
    s = MMpull(m) ;

    if ( s != NIL )
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
        B = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) ) ;

		if (B->bits == NULL)
		{
			MMechostr(MSKTRACE,"PUTpixel : Bitmap already destroyed %x\n",B->handler);
			return MMpush(m,NIL);
		}

        if ( x < 0 || x >= B->TailleW || y < 0 ||y >= B->TailleH )
        {
            MMechostr ( 1 , "Coordonates outside bitmap in _PUTpixel\n" ) ;
            return MMpush(m,s);    
        }
   
       
        if ( B->BPP == 16 )
            B->bits [ x + y * ( B->BPL >> 1 ) ] = Color24To16 (c) ;          
        else if ( B->BPP == 8 )
            (( unsigned char * ) B->bits ) [ x + y * B->TailleW ] = ( unsigned char ) c ;
       
    }

    s = MMpush(m,s);

    /************************** DEBUG 2D **************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _PUTpixel done\n" ) ;
    #endif
    /*************************************************************************************/
        
    return s ;
}






/********************************************************************************************/
/*                                                                                          */
/*  int GRAndBitmap ( mmachine m ) ;                                                        */
/*                                                                                          */
/*  correspond a la fonction _ANDbitmap ( Dest,Xd,Yd,W,H,Source,Sx,Sy);                     */
/*  qui effectue une operation AND entre 2 bitmap                                           */
/*                                                                                          */
/********************************************************************************************/

#define BITMAP_AND      0
#define BITMAP_OR       1
#define BITMAP_XOR      2
#define BITMAP_PLUS     3
#define BITMAP_MINUS    4
#define BITMAP_PLUS2    5
#define BITMAP_MINUS2   6

int GenericOpBitmap ( mmachine m , int Op )
{
    int sx,sy,h,w,dy,dx,ss,sd;
    int i , j ,dcx,dcy,scx,scy;

    int ColorD, rd,gd,bd, ColorS,rs,gs,bs ;

    PtrObjVoid OS ;
    PtrObjVoid OD ;
    PtrObjBitmap FS ;
    PtrObjBitmap FD ;
   
    sy = MMpull(m)>>1;
    sx = MMpull(m)>>1;
    ss = MMpull(m);
    h=MMpull(m)>>1;
    w=MMpull(m)>>1;
    dy=MMpull(m)>>1;
    dx=MMpull(m)>>1;
    sd=MMpull(m);

    if (sd == NIL || ss == NIL )
    {
        MMechostr (1,"Source or Destination is NIL in _ANDbitmap\n");
        return MMpush(m,sd);
    }

    OS = ( PtrObjVoid ) MMstart(m,(ss>>1)) ;
    OD = ( PtrObjVoid ) MMstart(m,(sd>>1)) ;
    FS = ( PtrObjBitmap ) MMstart(m,(OS->Buffer>>1));
    FD = ( PtrObjBitmap ) MMstart(m,(OD->Buffer>>1));

	if (FS->bits == NULL)
	{
		MMechostr(MSKTRACE,"OPcopy : Source Bitmap already destroyed %x\n",FS->handler);
		return MMpush(m,NIL);
	}

	if (FD->bits == NULL)
	{
		MMechostr(MSKTRACE,"OPcopy : Dest Bitmap already destroyed %x\n",FD->handler);
		return MMpush(m,NIL);
	}

    if ( !ClipBlit ( FD->TailleW , FD->TailleH , FS->TailleW , FS->TailleH ,
                &dx , &dy , &sx , &sy , &w , &h ) )
    {        
        return MMpush(m,sd);
    }   

    dcy = dy * ( FD->BPL >> 1 ) + dx ;
    scy = sy * ( FS->BPL >> 1 ) + sx ;                           
    
    switch ( Op)
    {
        case BITMAP_AND:
        for ( j = 0 ; j < h ; j ++ )
        { dcx = dcy ; scx = scy ;
                for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
                    FD->bits [ dcx ] &= FS->bits [ scx ] ;                               
                dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
        } break;

        case BITMAP_OR:
        for (j=0;j<h;j++)
        { dcx = dcy ; scx = scy ;
               for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
                    FD->bits [ dcx ] |= FS->bits [ scx ] ;                               
                dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
        } break;

        case BITMAP_XOR:
        for (j=0;j<h;j++)
        { dcx = dcy ; scx = scy ;
               for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
                    FD->bits [ dcx ] ^= FS->bits [ scx ] ;                               
                dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
        } break;

        case BITMAP_PLUS2:
        for (j=0;j<h;j++)
        { dcx = dcy ; scx = scy ;
               for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
               {
                    ColorD = FD->bits [ dcx ] ;                                            
                    ColorD = Color16To24 ( ColorD ) ;
                    ColorS = FS->bits [ scx ] ;
                    ColorS = Color16To24 ( ColorS ) ;
                    rd=(ColorD & 255);
                    gd=(ColorD>>8)&255;
                    bd=(ColorD>>16)&255;                       
                    rs=(ColorS & 255);
                    gs=(ColorS>>8 & 255);
                    bs=(ColorS>>16 & 255);
                    rd+=rs ;
                    if (rd>255) rd = 255 ;
                    gd+=gs;
                    if(gd>255) gd=255;
                    bd+=bs;
                    if(bd>255) bd=255;
                    rd>>=3; gd>>=3; bd>>=3;
                    FD->bits [ dcx ] = bd + ( gd << 5 ) + ( rd << 10 ) ;                                        
               }
               dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
               
        } break;

        case BITMAP_MINUS2:
        for (j=0;j<h;j++)
        { dcx = dcy ; scx = scy ;
               for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
                  {
                    ColorD = FD->bits [ dcx ] ;                                            
                    ColorD = Color16To24 ( ColorD ) ;
                    ColorS = FS->bits [ scx ] ;
                    ColorS = Color16To24 ( ColorS ) ;
                    rd=(ColorD & 255);
                    gd=(ColorD>>8)&255;
                    bd=(ColorD>>16)&255;                       
                    rs=(ColorS & 255);
                    gs=(ColorS>>8 & 255);
                    bs=(ColorS>>16 & 255);
                    rd-=rs ;
                    if (rd<0) rd =0 ;
                    gd-=gs;
                    if(gd<0) gd=0;
                    bd-=bs;
                    if(bd<0) bd=0;
                    rd>>=3; gd>>=3; bd>>=3;
                    FD->bits [ dcx ] = bd + ( gd << 5 ) + ( rd << 10 ) ;                                        
               }                                  
                dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
        } break;

        case BITMAP_PLUS:
        for (j=0;j<h;j++)
        { dcx = dcy ; scx = scy ;
               for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
                 {
                    ColorD = FD->bits [ dcx ] ;                                            
                    ColorD = Color16To24 ( ColorD ) ;
                    ColorS = FS->bits [ scx ] ;
                    ColorS = Color16To24 ( ColorS ) ;
                    rd=(ColorD & 255);
                    gd=(ColorD>>8)&255;
                    bd=(ColorD>>16)&255;                       
                    rs=(ColorS & 255);
                    gs=(ColorS>>8 & 255);
                    bs=(ColorS>>16 & 255);
                    rd+=rs ;
                    if (rd>255) rd -= 255 ;
                    gd+=gs;
                    if(gd>255) gd-=255;
                    bd+=rs;
                    if(bd>255) bd-=255;
                    rd>>=3; gd>>=3; bd>>=3;
                    FD->bits [ dcx ] = bd + ( gd << 5 ) + ( rd << 10 ) ;                                        
               }                             
                dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
        } break;

        case BITMAP_MINUS:
        for (j=0;j<h;j++)
        { dcx = dcy ; scx = scy ;
               for ( i = 0 ; i < w ; i ++ , dcx ++ , scx ++ )
                  {
                    ColorD = FD->bits [ dcx ] ;                                            
                    ColorD = Color16To24 ( ColorD ) ;
                    ColorS = FS->bits [ scx ] ;
                    ColorS = Color16To24 ( ColorS ) ;
                    rd=(ColorD & 255);
                    gd=(ColorD>>8)&255;
                    bd=(ColorD>>16)&255;                       
                    rs=(ColorS & 255);
                    gs=(ColorS>>8 & 255);
                    bs=(ColorS>>16 & 255);
                    rd-=rs ;
                    if (rd<0) rd += 255 ;
                    gd-=gs;
                    if(gd<0) gd+=255;
                    bd-=bs;
                    if(bd<0) bd+=255;
                    rd>>=3; gd>>=3; bd>>=3;
                    FD->bits [ dcx ] = bd + ( gd << 5 ) + ( rd << 10 ) ;                                        
               }                              
                dcy += FD->BPL >> 1 ; scy += FS->BPL >> 1 ;
        } break;
          
     }

    return MMpush(m,sd);
}

int GRAndBitmap ( mmachine m )
{return GenericOpBitmap ( m , BITMAP_AND );}

int GROrBitmap ( mmachine m )
{return GenericOpBitmap ( m,BITMAP_OR);}

int GRXorBitmap (mmachine m)
{return GenericOpBitmap(m,BITMAP_XOR);}

int GRPlusBitmap(mmachine m)
{return GenericOpBitmap(m,BITMAP_PLUS);}

int GRMinusBitmap(mmachine m)
{return GenericOpBitmap(m,BITMAP_MINUS);}

int GRSPlusBitmap(mmachine m)
{return GenericOpBitmap(m,BITMAP_PLUS2);}

int GRSMinusBitmap(mmachine m)
{return GenericOpBitmap(m,BITMAP_MINUS2);}

int GRNotBitmap(mmachine m)
{
    int s,x,y,w,h,i,j,dcy,dcx;
    PtrObjVoid O ;
    PtrObjBitmap B ;

    h=MMpull(m)>>1;
    w=MMpull(m)>>1;
    y=MMpull(m)>>1;
    x=MMpull(m)>>1;
    s=MMpull(m);
    
    if (s!=NIL)
    {    
        O=(PtrObjVoid) MMstart(m,(s>>1));
        B=(PtrObjBitmap) MMstart(m,(O->Buffer>>1));

		if (B->bits == NULL)
		{
			MMechostr(MSKTRACE,"NOTbitmap : Bitmap already destroyed %x\n",B->handler);
			return MMpush(m,NIL);
		}

        if (ClipSimple(B->TailleW,B->TailleH,&x,&y,&w,&h))
        {
            dcy = y * ( B->BPL >> 1 ) + x ;
            for ( j = 0 ; j < h ; j ++ )
            { 
                dcx = dcy ;
                for ( i = 0 ; i < w ; i ++ , dcx ++ )
                    B->bits [ dcx ] = 0x7FFF - B->bits [ dcx ] ;                               

                dcy += B->BPL >> 1 ; 
            }
        }
     }
    return MMpush(m,s);
}






/**********************************************************************************/
/*                                                                                */
/*      int GRMulBitmap ( mmachine m ) ;                                          */
/*                                                                                */
/*  correspond ala fonction magma ObjBitmap _MULbitmap ( ObjBitmap , I I I I F F F*/
/*  laquelle multiplie dans une section d'un bitmap, les composantes couleurs par */
/* des flottants                                                                  */
/*                                                                                */
/**********************************************************************************/

int GRMulBitmap ( mmachine m )
{
    float fr , fg , fb ;
    int x , y , w , h , r , g , b , col ;
    int s , dcx , dcy , i , j , Color , s1 , s2 , s3;
    PtrObjVoid O ;
    PtrObjBitmap B ;

    /****************************** DEBUG 2D **************************************/
    #ifdef TRACE_2D
        MMechostr ( 1 , "DBG _MULbitmap\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /******************************************************************************/

    col = MMpull(m)>>1;

    s1 = MMpull(m);
    if(s1==NIL) fb = 1 ; else fb = *((float *) MMstart(m,(s1>>1)));

    s2=MMpull(m) ;

    if(s2==NIL) fg = 1 ; else fg = *(( float *) MMstart(m,(s2>>1))) ;

    s3=MMpull(m) ;
    if(s3==NIL) fr = 1 ; else fr = *(( float * ) MMstart(m,(s3>>1))) ;

    h = MMpull(m)>>1;
    w = MMpull(m)>>1;
    y = MMpull(m)>>1;
    x = MMpull(m)>>1;
    s=MMpull(m);
    
    if (s!=NIL)
    {        
        O=(PtrObjVoid) MMstart(m,(s>>1));
        B=(PtrObjBitmap) MMstart(m,(O->Buffer>>1));

		if (B->bits == NULL)
		{
			MMechostr(MSKTRACE,"MULbitmap : Bitmap already destroyed %x\n",B->handler);
			return MMpush(m,NIL);
		}

        if (ClipSimple(B->TailleW,B->TailleH,&x,&y,&w,&h))
        {
            dcy = y * ( B->BPL >> 1 ) + x ;
            for ( j = 0 ; j < h ; j ++ )
            { 
                dcx = dcy ;
                for ( i = 0 ; i < w ; i ++ , dcx ++ )
                {
                        Color = B->bits [ dcx ] ;                                            
                        Color = Color16To24 ( Color ) ;
                         if ( col == Color ) continue ;
                       
                        r=(Color & 255);
                        g=(Color>>8)&255;
                        b=(Color>>16)&255;                       
                        b = ( int ) ((( float ) b ) * fb );
                        g = ( int ) ((( float ) g ) * fg );
                        r = ( int ) ((( float ) r ) * fr );
                        if(b>255) b = 255;
                        if(r>255) r = 255;
                        if(g>255) g = 255;
                        r>>=3; g>>=3; b>>=3;
                        B->bits [ dcx ] = b + ( g << 5 ) + ( r << 10 ) ;
                }                                          
                dcy += B->BPL >> 1 ; 
            }
        }
    }

    s =  MMpush(m,s) ;

    /******************* DEBUG 2D ************************************************/
    #ifdef TRACE_2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _MULbitmap one\n" ) ;
    #endif
    /*******************************************************************************/

    return s ;
}

/********************************************************************************************************/
/*                                                                                                      */
/*  int GRDrawRectangleText ( mmachine m ) ;                                                            */
/*  correspond a la fonction ObjBitmap _DRAWrectangleText (ObjBitmap,Font,I,I,I,I,I,I,S);               */
/*  laquelle dessine un rectangle transparent de texte                                                  */
/*                                                                                                      */
/********************************************************************************************************/

int GRDrawRectangleText ( mmachine m )
{

    RECT r ;
    int s,sf,sb,flag,color,oldbk,TFlag ;
    char * texte ;
    PtrObjVoid OB , OF ;
    PtrObjFont F ;
    PtrObjBitmap B ;
    HBITMAP OldBuffer ;
    HDC Dc ;
    HWND Hw ; 
    HFONT OldFont ; 

    /********************************* DEBUG 2D **********************************************/
    #ifdef TRACE2D
        MMechostr(MSKDEBUG,"(DBG) _DRAWrectangleText\n");
        FDebug2D(m);
    #endif
    /******************************************************************************************/

    s = MMpull(m);
    flag = MMpull(m)>>1;
    color = MMpull(m)>>1;
    r.bottom = MMpull(m)>>1;
    r.right = MMpull(m)>>1;
    r.top = MMpull(m)>>1;
    r.left = MMpull(m)>>1;

    sf = MMpull(m) ;
    sb = MMpull(m) ;

    if (sf != NIL && sb != NIL && s != NIL)
    {
        texte = ( char * ) MMstart(m,(s>>1)+1) ; 
        OB= ( PtrObjVoid ) MMstart(m,(sb>>1));
        OF = ( PtrObjVoid ) MMstart(m,(sf>>1));
        B = ( PtrObjBitmap ) MMstart(m,(OB->Buffer>>1));
        F = ( PtrObjFont ) MMstart(m,(OF->Buffer>>1)) ;

        switch ( OB->Type >> 1 )
        {
            case OBJ_TYPE_BITMAP :
                   B = ( PtrObjBitmap ) MMstart(m, ( OB->Buffer >> 1 ) ) ;

				   //$ LB (13/06/2002) : create DIBSection from bitmap buffer
				   if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);

                   Dc = CreateCompatibleDC ( NULL ) ;                 
                   OldBuffer = SelectObject ( Dc , B->DIBhandler) ;        
                   break ;

           case OBJ_TYPE_WINDOW :
                   Hw = (( PtrObjWindow ) MMstart(m,( OB->Buffer>>1) ) ) -> WHandler ;
                   Dc = GetDC ( Hw ) ;
                   break ;
                
            default :
    
            MMechostr ( 1 , "TextOut : Parameter Win/Bitmap is type %d\n" , OB->Type >> 1) ;
            return MMpush(m,s) ;
        }
   
    

        TFlag = DT_EDITCONTROL|DT_WORDBREAK ;
    
        if ( flag & TD_BOTTOM ) TFlag |= DT_BOTTOM ;
        if ( flag & TD_TOP ) TFlag |= DT_TOP ;
        if ( flag & TD_CENTER ) TFlag |= DT_CENTER ;
        if ( flag & TD_LEFT ) TFlag |= DT_LEFT ;
        if ( flag & TD_RIGHT ) TFlag |= DT_RIGHT ;
    
    
        OldFont = SelectObject ( Dc , F->WHandler ) ;
   
        SetTextColor ( Dc , color ) ;
        oldbk = SetBkMode ( Dc , TRANSPARENT ) ;       

        /* etend le rectangle car c'est x1 y1 w h qu'on a passe en parametre */
        r.bottom += r.top ;
        r.right += r.left ;
        DrawText (Dc , texte , strlen ( texte ) , &r , TFlag ) ;   

        SetBkMode ( Dc , oldbk ) ;
        SelectObject ( Dc , OldFont ) ;
  
        if ( OB->Type == ( OBJ_TYPE_BITMAP << 1 )  )
        {
              OldBuffer = SelectObject ( Dc , OldBuffer ) ;         
            DeleteDC ( Dc ) ;
        }
    }

    s = MMpush(m,sb);

    /***************************** DEBUG 2D ***************************************************/
    #ifdef TRACE2D
        FDebug2D(m);
        MMechostr(MSKDEBUG,"(DBG) _DRAWrectangleText done\n");
    #endif
    /*******************************************************************************************/

    return s ;
}



/* fonction indiquant si une surface est dote d'un z-buffer,
en mode soft, elle retourne toujours 1 */
int GRtstzbuf(mmachine m)
{
	if (MMget(m,0)==NIL) return 0;
	MMset(m,0,1<<1);
	return 0;
}





int GRCopyWindow ( mmachine m )
{
    int sx1 , sy1 , sx2 , sy2 , ss , dx , dy , sd , res ;
    PtrObjVoid OS , OD ;
    PtrObjBitmap B ;
    PtrObjWindow W ;
    HDC Dcb , Dcw ;
    HBITMAP OldBitmap ;


    sy2 = MMpull(m) >> 1 ;
    sx2 = MMpull(m) >> 1 ;
    dy = MMpull(m) >> 1 ;
    dx = MMpull(m) >> 1 ;
    sd = MMpull(m) ;
    sy1 = MMpull(m) >> 1 ;
    sx1 = MMpull(m) >> 1 ;
    ss = MMpull(m) ;
   
    if ( ss == NIL || sd == NIL )
    {
        MMechostr ( 1 , "_copy_win_to_bitmap : The window Obj or the bitmap is NIL\n" ) ;
         return MMpush(m,ss) ;
    }

 
    OS = ( PtrObjVoid ) MMstart(m,(ss>>1) ) ;
    OD = ( PtrObjVoid ) MMstart(m,(sd>>1) ) ;
    B = ( PtrObjBitmap ) MMstart(m,(OS->Buffer>>1) ) ;
    W = ( PtrObjWindow ) MMstart(m,(OD->Buffer>>1) );

	
    //$ LB (13/06/2002) : create DIBSection from bitmap buffer
	if (!B->DIBhandler) ObjBitmap_CreateDIBSection (B);


    Dcw = GetDC ( W->WHandler ) ;       
    Dcb = CreateCompatibleDC ( Dcw ) ;
    OldBitmap = SelectObject ( Dcb , B->DIBhandler) ;
    BitBlt ( Dcb , sx1 , sy1 , sx2 , sy2 , Dcw , dx , dy , SRCCOPY ) ;
 
    SelectObject ( Dcb , OldBitmap ) ;
    DeleteDC ( Dcb ) ;    

    res = MMpush(m,ss) ;


    return res ;
}

