/****************************************************************************/
/*                                                                          */
/*  font.c                                                                  */
/*                                                                          */
/*  Implementation des fonctions du package font.pkg                        */
/*                                                                          */
/****************************************************************************/


#include "x/Version.h"
#include "x/scolplugin.h"

#include <stdio.h>
#include <string.h>

#include "objstr.h"
#include "objects/cursor.h"


//#define TRACE2D    

int TESTCURSORMOVE = 0 ;

HFONT DefaultFont = NULL ;




/****************************************************************************************/
/*                                                                                      */
/* HFONT NewObjFont ( PtrObjFont , char * Name ) ;                                      */
/*                                                                                      */
/* Cree une nouvelle font                                                               */
/*                                                                                      */
/****************************************************************************************/

HFONT NewObjFont ( PtrObjFont F , char * Name )
{

        int FF , Weight , Italic , Underline , Strike ;

        FF = F->Flags ;
        if ( FF & FF_WEIGHT ) Weight = 700 ; else Weight = 0 ;
        if ( FF & FF_ITALIC ) Italic = 1 ; else Italic = 0 ;
        if ( FF & FF_UNDERLINE ) Underline = 1 ; else Underline = 0 ;
        if ( FF & FF_STRIKED ) Strike = 1 ; else Strike = 0 ;
        if ( FF & FF_PIXEL ) F->Height=-MulDiv(F->Height , GetDeviceCaps(GetDC(NULL),LOGPIXELSY), 72);
     
        
      return CreateFont ( F->Height , 0 , 0 ,
        F->Direction , Weight , Italic , Underline , Strike ,
        DEFAULT_CHARSET , 0 , 0 , 0 , 0 , Name ) ;
}



/*****************************************************************************/
/*                                                                           */
/*  int GRSetDefaultFont ( mmachine m )                                      */
/*                                                                           */
/*   correspond a la fonction magma ObjFont _SETdefaultFont ( ObjFont ) ;    */
/*   laquelle definit la police par default                                  */
/*                                                                           */
/*****************************************************************************/

int GRSetDefaultFont ( mmachine m ) 
{
    int s ;
    PtrObjVoid O ;
    PtrObjFont F ;

    s = MMpull(m);
    if ( s != NIL )
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
        F = ( PtrObjFont ) MMstart(m,(O->Buffer>>1) ) ;
        DefaultFont = F->WHandler ; 

    } else DefaultFont = NULL ;

    return MMpush(m,s) ;
}
 
/*****************************************************************************/
/*                                                                           */
/*  int GRGetDefaultFont ( mmachine m ) ;                                    */
/*                                                                           */
/*  correspond a la fonction magma ObjFont _GETdefaultFont ( ) ;             */
/*  laquelle retourne la font par default                                    */
/*                                                                           */
/*****************************************************************************/

int GRGetDefaultFont ( mmachine m ) 
{
    int p ;

    if (DefaultFont == NULL) return MMpush(m,NIL);
    else
    {
        p = OBJfindTH(m,OBJTYPTEXT,(int)DefaultFont);
        if ( p != NIL ) p = MMfetch(m,p,OFFOBJMAG);
        return MMpush(m,p);
    }
}

/*****************************************************************************/
/*                                                                           */
/* int GRCreateFont ( mmachine m )                                           */
/*                                                                           */
/* correspond a la fonction magma Obj _create_font ( I I I S )               */
/* laquelle cree un objet Font                                               */
/*                                                                           */
/*****************************************************************************/

int GRCreateFont ( mmachine m )
{
    PtrObjVoid O ;
    PtrObjFont F ;
    int s , l , s2 , res ;
    HFONT HW ;
    char Name [ 256 ] ;

    /********************* DEBUG 2D *****************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _create_font\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /*************************************************************************/

    /* teste le channel */
    if (MMget(m,4)==NIL)
    {
        MMechostr(MSKDEBUG,"_CRfont : Channel is NIL\n") ;
        m->pp+=4;
        return 0;
    }


    s = MMpull(m) ;
    if (( s == NIL )||(MMsizestr(m,s>>1)>128)) strcpy ( Name , "" ) ;
    else strcpy ( Name , ( char * ) MMstartstr(m, (s>>1)) ) ;
    Mpushstrbloc ( m , Name ) ;


    l = ( sizeof ( struct ObjVoid ) + 3 ) >> 2 ;
    s = MMmallocCLR (m,l,TYPETAB) ;
    if( s == NIL ) return MERRMEM ;
    if ( MMpush(m,(s<<1)+1)) return MERRMEM ;
    l = ( sizeof ( struct ObjFont ) + 3 ) >> 2 ;
    s2 = MMmalloc ( m,l,TYPEBUF ) ;
    if ( s2 == NIL ) return MERRMEM ;
    s = MMpull(m) ;

    O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
    F = ( PtrObjFont ) MMstart(m, s2 ) ;
    O->Type = OBJ_TYPE_FONT << 1 ;
    O->Tab = NIL; MMpull(m) ;  //   SH000111
    O->Buffer = ( s2 << 1 ) + 1 ;  
    F->Flags = MMpull(m) >> 1 ;
    F->Direction = MMpull(m) >> 1 ;
    F->Height = MMpull(m) >> 1 ;   
    O->Father = NIL ;

    HW = NewObjFont ( F , Name ) ;    
    if ( ! HW )
    {
        MMechostr(MSKDEBUG,"ERROR : _create_font : WIN95 can't create the font : Check the values\n" ) ;
        MMset(m,0,NIL);
        return 0 ;
    }
    F->WHandler = HW ;
    MMpush(m,s) ;
    res = OBJcreate (m,OBJTYPFONT,(int)HW,0,(int)NULL) ;
	//MMechostr(1,"----> _CRfont id:%d\n",(int)F->WHandler);
    /************************** DEBUG 2D ********************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _create_font done\n" ) ;
    #endif
    /*************************************************************************/

    return res ;
}

/*****************************************************************************/
/*                                                                           */
/* int GRAffectGFont ( mmachine m ) ;                                        */
/*                                                                           */
/* correspond a la fonction magma Obj _affect_font [ Obj Obj ]               */
/* laquelle affecte une police de caractere a un objet contenant du texte    */
/*                                                                           */
/*****************************************************************************/

int GRAffectFont ( mmachine m )
{

    PtrObjVoid OF , OC ;
    PtrObjFont FF ;    
    int s , s2 , res ;
    HWND H ;

    /******************************* DEBUG 2D **********************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _affect_font\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /***************************************************************************/

    s2 = MMpull(m) ;
    s = MMpull(m) ;
    if ( s2 == NIL )
    {
        MMechostr ( 1 , "AffectFont : Font object is NIL\n" ) ;
        return MMpush(m,s) ;
    }
    OF = ( PtrObjVoid ) MMstart(m,(s2>>1) ) ;    
    
    if ( s == NIL )
    {
        MMechostr ( 1 , "AffectFont : Affetect object is NIL\n" ) ;
        return MMpush(m,s) ;
    }
    OC = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
    FF = ( PtrObjFont ) MMstart(m,(OF->Buffer>>1) ) ;

    switch ( OC->Type >> 1 )
    {
        case OBJ_TYPE_TEXT :
        case OBJ_TYPE_EDIT_LINE :
        case OBJ_TYPE_EDIT_TEXT :

            H = (( PtrObjText ) MMstart(m,(OC->Buffer>>1) )) ->WHandler ;
            break ;

        case OBJ_TYPE_PUSHBUTTON :
        case OBJ_TYPE_CHECK_BOX :
            H = ((PtrObjButton ) MMstart(m,(OC->Buffer>>1) )) ->WHandler ;
            break ;


        case OBJ_TYPE_COMBO_BOX :
        case OBJ_TYPE_LIST_BOX :
            H = (( PtrObjCombo ) MMstart(m,(OC->Buffer>>1) )) ->WHandler ;
            break ;

        case OBJ_TYPE_WINDOW :
            H = ((PtrObjWindow ) MMstart(m,(OC->Buffer>>1)))->WHandler ;
            break ;
       
        default :
            MMechostr ( 1 , "You can't affect a font to this type of object\n" ) ;
            H = NULL ;
        
        break ;
    }
    
    if ( H )  SendMessage ( H , WM_SETFONT , ( WPARAM ) FF->WHandler , ( LPARAM ) TRUE ) ;      

    res = MMpush(m,s) ;

    /******************************* DEBUG 2D ********************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 ,  "DBG _affect_font done\n" ) ;
    #endif
    /*************************************************************************/

    return res ;
}
      

/**********************************************************************/
/*                                                                    */
/*  int GRDestroyObjFont ( mmachine m )                               */
/*                                                                    */
/* correspond a la fonction magma I _destroy_obj_font ( ObjFont )     */
/* laquelle detruit une police de caractere                           */
/*                                                                    */
/**********************************************************************/

int GRDestroyObjFont ( mmachine m )
{
    int s ;
    PtrObjVoid O ;
    HFONT h ;

    /************************** DEBUG 2D ******************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _destroy_obj_font\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /*********************************************************/

    s = MMget(m,0) ;
    if ( s != NIL )
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
        h = (( PtrObjFont ) ( MMstart(m, (O->Buffer>>1 )) ))->WHandler ;
        MMechostr(1,"----> _DSfont id:%d\n",(int)h);
		OBJdelTH(m,OBJTYPFONT,(int)h) ;
    }
    
    /********************** DEBUG 2D *************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG _destroy_obj_win Done\n" ) ;
    #endif
    /***********************************************************/

    MMset(m,0,0) ;
    return 0 ;
}


/**************************************************************************************/
/*                                                                                    */
/*      int GRGetStringSize ( mmachine m ) ;                                          */
/*                                                                                    */
/*  correspond a la fonction magma [I I ] _GETstringSize ( font , strinf ) ;          */
/*                                                                                    */
/**************************************************************************************/

int GRGetStringSize ( mmachine m ) 
{
    int s , w , h ;
    PtrObjVoid  O ;
    PtrObjFont F ;
    char * text ;
    SIZE R ;
    HDC H ;
    HFONT oldfont;

    /********************************* DEBUG 2D ****************************************/
    #ifdef DEBUG2D
        MMechostr ( 1 , "DBG _get_string_size\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**************************************************************************************/

    s = MMpull ( m ) ;
    if ( s == NIL )
    {
       MMpull(m) ;
       w = 0 ; 
       h = 0 ; 
    } else
    {
        text = ( char * ) MMstart(m,(s>>1)+1) ;
        s = MMpull ( m ) ;
        if ( s == NIL )
        {
            w = 0 ; 
            h = 0 ;
        } else 
        {
            O = ( PtrObjVoid ) MMstart(m,(s>>1));
            F = ( PtrObjFont ) MMstart(m,(O->Buffer>>1)) ;
            H = CreateCompatibleDC ( NULL ) ;
            oldfont = SelectObject (H,F->WHandler);
            GetTextExtentPoint32 ( H , text , strlen ( text ) , &R ) ;
            SelectObject(H,oldfont);
            DeleteDC ( H ) ;
            w = R.cx ;
            h = R.cy ;
        }
    }

    if ( MMpush(m,w<<1)) return MERRMEM ;
    if ( MMpush(m,h<<1)) return MERRMEM ;
    if ( MMpush(m,4)) return MERRMEM ;
    s = MBdeftab ( m ) ;

    /****************************** DEBUG 2D ************************************************/
    #ifdef DEBUG2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _get_string_size done\n" ) ;
    #endif
    /******************************************************************************************/
		return s;
      
}


/**************************************************************************************/
/*                                                                                    */
/*  int GRCreateCursor ( mmachine m ) ;                                               */
/*                                                                                    */
/*  cette fonction correspond a la fonction magma ObjCursor _create_cursor ( )        */
/*  ( ObjBitmap I Color1 I color2 I X I Y );                                          */
/*  laquelle cree un curseur de souris                                                */
/*                                                                                    */
/**************************************************************************************/

int GRCreateCursor ( mmachine m )
{
    int x , y , color1 , color2 , s , s2 , sb , l , res , i , j ;
    HCURSOR Hc ;
    PtrObjVoid O ;
    PtrObjBitmap B ;
    PtrObjCursor C ;
    int SourisW , SourisH , tand , tor , tailleW ;
    unsigned short int * Bitmap ;
    unsigned char * tbitand, * tbitor ;

    /**************************** DEBUG 2D***********************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _create_cursor\n") ;
        FDebug2D ( m ) ;
    #endif
    /*************************************************************************************/

    /* test le channel */
    if (MMget(m,5)==NIL)
    {
        MMechostr(MSKDEBUG,"CRchannel : channel NIL\n");
        m->pp+=5;
        return 0;
    }

   color2 = MMpull ( m ) >> 1 ;
    color1 = MMpull ( m ) >> 1 ;
    y = MMpull ( m ) >> 1 ;
    x = MMpull ( m ) >> 1 ;
    sb = MMpull( m ) ; 
    if ( sb == NIL )
    {
        MMechostr (1,"Bitmap is NIL in _CRcursor\n");
        MMpull(m); /* depile le channel */
        return MMpush(m,NIL);
    } else MMpush(m,sb);

    l = ( sizeof ( struct ObjVoid ) + 3 ) >> 2 ;
    s = MMmallocCLR (m,l,TYPETAB) ;
    if( s == NIL ) return MERRMEM ;
    if ( MMpush(m,(s<<1)+1)) return MERRMEM ;
  
    l = ( sizeof ( struct ObjCursor ) + 3 ) >> 2 ;
    s2 = MMmalloc ( m,l,TYPEBUF ) ;
    if ( s2 == NIL ) return MERRMEM ;
    s = MMpull(m) ;

    O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
    C = ( PtrObjCursor ) MMstart(m, s2 ) ;
    O->Type = OBJ_TYPE_CURSOR << 1 ; 
    O->Buffer = ( s2 << 1 ) + 1 ;       
    O->Father = NIL ;
    O->Tab= MMget(m,0) ;//SH
    sb = MMpull( m ) ; 
    O = ( PtrObjVoid ) MMstart(m,(sb>>1) ) ;
    B = ( PtrObjBitmap ) MMstart(m,(O->Buffer>>1) )  ;
    SourisW = GetSystemMetrics ( SM_CXCURSOR ) ;
    SourisH = GetSystemMetrics ( SM_CYCURSOR ) ;

    if ( B->TailleW != SourisW || B->TailleH != SourisH )
    {
        MMechostr ( 1 , "_create_cursor : The Bitmap haven't the good size. Cursor must be %dx%d and bitmap is %dx%d\n",
            SourisW , SourisH , B->TailleW , B->TailleH ) ;
		MMpull(m); //SH
        res = MMpush(m,NIL ) ;
    }
    else
    {

            Bitmap = B->bits ;
			tailleW=B->TailleW;
            l = SourisW * SourisH  ;            
            if ( l & 7 ) l= l + 8 ;
            l >>= 3 ;

            if ( MMpush(m,s)) return MERRMEM ;
            tand = MMmallocCLR(m,( l + 3 ) >> 2,TYPEBUF ) ;
            if ( MMpush(m,(tand<<1)+1)) return MERRMEM ;
            tor = MMmallocCLR(m , ( l + 3 ) >> 2 , TYPEBUF ) ;
            tand = MMpull ( m ) >> 1 ;

            tbitand = ( unsigned char * ) MMstart(m, tand ) ;
            tbitor = ( unsigned char * ) MMstart(m, tor ) ;
          
            for ( i = 0 ; i < l ; i ++ )
            {        
                tbitor [ i ] = 0 ;
                tbitand [ i ] = 255 ;
            }
            

            for ( j = 0 ; j < SourisH ; j ++ )
            for ( i = 0 ; i < SourisW ; i ++ ) 
            {   
                
                l = i + j * tailleW ;                             
                if ( Bitmap [ l ] == color2 )
                {
                    tbitor [ l/8 ] ^= 01 << ( 7 -( i & 7 )) ;   
                    tbitand [l/8 ] ^= 01 << ( 7 - (i & 7 )) ;                   
                } else                
                if ( Bitmap [ l ] == color1 )
                {
                    
                    tbitand [ l /8 ] ^= 01 << ( 7 - (i & 7 )) ;                                      
                } 
            }

            s = MMpull( m ) ;
            O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
            C = ( PtrObjCursor ) MMstart(m,(O->Buffer>>1)) ;
            C->X = x ;
            C->Y = y ;                    
            C->HCursor = CreateCursor ( (HINSTANCE)SCgetExtra("this_inst") , x , y , SourisW , SourisH , tbitand , tbitor ) ;
            Hc = C->HCursor ;
            if ( C->HCursor == NULL ) 
            {
                MMechostr ( 1 , "ERROR : _create_cursor failed x=%d y=%d w =%d h =%d\n",x,y,SourisW,SourisH);            
				MMpull(m);//SH
                res = MMpush ( m , NIL ) ;
            }
            else
            {
                res=MMpush(m,s) ;                
                res = OBJcreate (m,OBJTYPCURSOR,(int)Hc,-1,(int)NULL) ;
                
            }       
                                 
    }
    

    /************************************** DEBUG 2D *******************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _create_cursor done\n" ) ;
    #endif
    /*********************************************************************************************/

    return res ;
}


/**************************************************************************************************/
/*                                                                                                */
/*  int GRGetCursorSize ( mmachine m ) ;                                                          */
/*                                                                                                */
/*  correspond a la fonction [i i ] _get_cursor_size    () ;                                      */
/*                                                                                                */
/**************************************************************************************************/

int GRGetCursorSize ( mmachine m )
{
    int x , y ;

    /************************* DEBUG 2D *********************************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _get_cusor_size\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /*********************************************************************************************/

    x = GetSystemMetrics ( SM_CXCURSOR ) ;
    y = GetSystemMetrics ( SM_CYCURSOR ) ;

    if ( MMpush ( m , x << 1 )) return MERRMEM ;
    if ( MMpush ( m , y << 1 )) return MERRMEM ;
    if ( MMpush ( m , 4 )) return MERRMEM ;

    /**************************** DEBUG 2D *******************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ); 
        MMechostr ( 1 , "DBG _get_cursor_size done\n" ) ;
    #endif
    /********************************************************************************************/

    return MBdeftab ( m ) ; 
}


/************************************************************************************************/
/*                                                                                              */
/*  int GRSetCursor ( mmachine m ) ;                                                            */
/*                                                                                              */
/*  correspond a la fonction magma _set_cursor ( ObjCursor ) -> ObjCursor                       */
/*                                                                                              */
/************************************************************************************************/


int GRSetCursor ( mmachine m )
{
    int s , res , sw ;
    PtrObjVoid O , OW ;
    PtrObjCursor C ;
    HWND H ;
    HCURSOR Hc ;
    POINT P ;

    /********************************** DEBUG 2D **************************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _set_cursor\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**********************************************************************************************/


    s = MMpull( m ) ;
    sw = MMpull ( m ) ;
    if ( sw != NIL )
    {
        if ( s == NIL ) Hc = arrowCursor; // SH : Hc = NULL
        else
        {
            O = ( PtrObjVoid ) MMstart(m,(s>>1) );
            C = ( PtrObjCursor ) MMstart(m,(O->Buffer >> 1 ) ) ;
            Hc = C->HCursor ; 
        }
        OW = ( PtrObjVoid ) MMstart(m,(sw>>1) ) ;
        switch ( OW->Type >> 1 )
        {
            case OBJ_TYPE_WINDOW :
                H =  (( PtrObjWindow ) MMstart(m,(OW->Buffer>>1) ) ) ->WHandler ;
                (( PtrObjWindow ) MMstart(m,(OW->Buffer>>1) ) ) ->Cursor =Hc;
                break ;
            case OBJ_TYPE_CHECK_BOX :
            case OBJ_TYPE_PUSHBUTTON :
                H = (( PtrObjButton ) MMstart(m,(OW->Buffer>>1) )) -> WHandler ;
                break ;
            case OBJ_TYPE_PUSH_BUTTON_BITMAP :
                H = (( PtrObjPushButtonBitmap ) MMstart(m,(OW->Buffer>>1) )) ->WHandler ;
                break ;
            default : H = NULL ;
                break ;
        }
        if ( Hc && H )
        {
            if ( Hc != ( HCURSOR ) GetClassLong ( H , GCL_HCURSOR ))
            {

                SetClassLong ( H , GCL_HCURSOR , (long) Hc ) ;
                if (GetCapture ( ) == H ) SetCursor ( Hc ) ;

                GetCursorPos (&P) ;
                TESTCURSORMOVE = 1 ;
                SetCursorPos(P.x , P.y);           
            }
        }
        
    }
    else MMechostr ( 1 , "ERROR : SETcursor : Cursor or Window object is NIL\n" ) ;
    res = MMpush ( m , sw ) ;


    /******************************** DEBUG 2D ******************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ; 
        MMechostr ( 1 , "DBG _set_cursor done\n" ) ;
    #endif
    /*************************************************************************************************/

    return res ;
}


/***********************************************************************************************/  
/*                                                                                             */
/*  int GRDestroyCursor ( mmachine m ) ;                                                       */
/*                                                                                             */
/*  correspond a la fonction magma I _destroy_cursor ( ObjCursor )                             */
/* laquelle detruit un curseur souris                                                          */
/*                                                                                             */
/***********************************************************************************************/

int GRDestroyCursor ( mmachine m )
{
    int s ;
    PtrObjVoid O ;
    HCURSOR h ;


    /************************ DEBUG 2D *********************************************/
    #ifdef TRACE2D
        MMechostr ( 1 , "DBG _destroy_cursor \n" ) ;
        FDebug2D ( m ) ;
    #endif
    /********************************************************************************/



    s = MMget(m,0) ;
    if ( s != NIL )
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1) ) ;
        h = (( PtrObjCursor ) ( MMstart(m, (O->Buffer>>1 )) ))->HCursor ;
        OBJdelTH(m,OBJTYPCURSOR,(int)h) ;
    }
    
    /********************** DEBUG 2D *************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr( 1 , "DBG _destroy_cursor Done\n" ) ;
    #endif
    /***********************************************************/

    MMset(m,0,0) ;
    return 0 ;
}

/*************************************************************************************/
/*                                                                                   */
/*  int GRGetCursor ( mmachine m ) ;                                                 */
/*                                                                                   */
/*  correspond a la fonction ObjCursor _get_cursor ( ObjWin )                        */
/*  laquelle retourne le curseur d'une fenetre / bouton                              */
/*                                                                                   */
/*************************************************************************************/

int GRGetCursor ( mmachine m )
{
    int s , l , s2 ;
    PtrObjVoid O ;
    PtrObjCursor C ;
    HWND H ;


    /******************** DEBUG 2D ****************************************************/
    #ifdef TRACE2D
        MMechostr( 1 , "DBG _get_cursor\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /**********************************************************************************/

    s = MMpull( m ) ;
    if ( s == NIL ) return MMpush ( m , NIL ) ;
    O = ( PtrObjVoid ) MMstart(m,(s>>1) )  ;

    switch ( O->Type >> 1 )
    {
        case OBJ_TYPE_WINDOW :
            H = (( PtrObjWindow ) MMstart(m,(O->Buffer>>1) ))->WHandler ;
            break ;
        case OBJ_TYPE_PUSHBUTTON :
            H = (( PtrObjButton ) MMstart(m,(O->Buffer>>1) ))->WHandler ;
            break ;
    }

    l = ( sizeof ( struct ObjVoid ) + 3 ) >> 2 ;
    s = MMmallocCLR (m,l,TYPETAB) ;
    if( s == NIL ) return MERRMEM ;
    if ( MMpush(m,(s<<1)+1)) return MERRMEM ;
  
    l = ( sizeof ( struct ObjCursor ) + 3 ) >> 2 ;
    s2 = MMmalloc ( m,l,TYPEBUF ) ;
    if ( s2 == NIL ) return MERRMEM ;
    s = MMpull(m) ;

    O = ( PtrObjVoid ) MMstart(m, (s>>1) ) ;
    C = ( PtrObjCursor ) MMstart(m, s2 ) ;
    O->Type = OBJ_TYPE_CURSOR << 1 ; 
    O->Buffer = ( s2 << 1 ) + 1 ;       
    O->Father = NIL ;
    O->Tab= NIL ;
    C->HCursor = ( HCURSOR ) GetClassLong ( H , GCL_HCURSOR ) ;    
    s2 = MMpush( m , s ) ;

    /******************************** DEBUG 2D *************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1 , "DBG _get_cursor done\n" ) ;
    #endif
    /********************************************************************************/

    return s2 ;
}


/*************************************************************************************/
/*                                                                                   */
/*  int GRSetCursorPos ( mmachine m ) ;                                              */
/*                                                                                   */
/*  correspond a la fonction magma ObjWin _SETcursorPos ( ObjWin , I , I )           */
/*  laquelle positionne la souris dans une fenetre                                   */
/*                                                                                   */
/*************************************************************************************/

int GRSetCursorPos ( mmachine m )
{
    int s , x , y ;
    PtrObjVoid O;
    PtrObjWindow OW ;
    HWND W ;
    POINT P ;


    /************************** DEBUG 2D *********************************************/
    #ifdef TRACE2D
        MMechostr (1,"DBG _SETcursorPos\n" ) ;
        FDebug2D ( m ) ;
    #endif
    /*********************************************************************************/

    y = MMpull(m) >> 1 ;
    x = MMpull(m) >> 1 ;
    s = MMpull(m) ;

    if ( s != NIL )
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1));
        OW = ( PtrObjWindow) MMstart(m,(O->Buffer>>1)) ;
        W = OW->WHandler ;
        
        P.x = 0 ;
        P.y = 0 ;
        ClientToScreen ( W , &P ) ;        
       
        SetCursorPos (x + P.x , y + P.y ) ;
               
    }

    s = MMpush(m,s) ;


    /******************************* DEBUG 2D *****************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr (1,"DBG _SETcursorPOs done\n" ) ;
    #endif
    /***********************************************************************************/

    return s;
}


/**************************************************************************************/
/*                                                                                    */
/*  int GRGetCursorPos ( mmachine m ) ;                                               */
/*                                                                                    */
/*  Correspond a la fonction magma [ I I ] _GETcursorPos ( ObjWin )                   */
/*  laquelle retourne les coordonnees de la souris par rapport a une fenetre          */
/*                                                                                    */
/**************************************************************************************/

int GRGetCursorPos ( mmachine m )
{

    int s ;
    PtrObjWindow OW ;
    PtrObjVoid O ;
    HWND W ;
    POINT P , P2 ;

    /**************** DEBUG 2D ****************************************************/
    #ifdef TRACE2D
        MMechostr (1,"DBG _GETcursorPos\n") ;
        FDebug2D(m);
    #endif
    /********************************************************************************/

    s = MMpull(m);
    if (s == NIL ) s = MMpush(m,NIL);
    else 
    {
        O = ( PtrObjVoid ) MMstart(m,(s>>1)) ;
        OW = ( PtrObjWindow ) MMstart(m,(O->Buffer>>1)) ;
        W = OW->WHandler ;
        GetCursorPos (&P ) ;
        P2.x = 0 ; P2.y = 0 ;
        ClientToScreen ( W , &P2 ) ;
        P.x -= P2.x ;
        P.y -= P2.y ;
        if (MMpush(m,(P.x)<<1)) return MERRMEM ;
        if (MMpush(m,(P.y)<<1)) return MERRMEM ;
        if (MMpush(m,4)) return MERRMEM;
        s = MBdeftab (m);
    }

    /******************* DEBUG 2D ***************************************************/
    #ifdef TRACE2D
        FDebug2D ( m ) ;
        MMechostr ( 1,"DBG _GETcursorPos\n");
    #endif
    /**********************************************************************************/
    return s;             
}

/****************************************************************************************/
/*                                                                                      */
/*  int GRShowCursor ( mmachine m ) ;                                                   */
/*                                                                                      */
/*  correspond a la fonction magma I _SHOWcursor ( I )                                  */
/*  laquelle active ou desactive la souris                                              */
/*                                                                                      */
/****************************************************************************************/

int GRShowCursor ( mmachine m )
{
    int s , sf ;
    PtrObjVoid O ;
    PtrObjWindow W ;

    s = MMpull(m)>>1 ; 
    sf = MMpull(m);
    if ( sf != NIL )
    {
          O = ( PtrObjVoid ) MMstart(m,(sf>>1));
          W = ( PtrObjWindow ) MMstart(m,(O->Buffer>>1));
          SetFocus (W->WHandler);
          
          if ( s == 0 )
          {
              while ( ShowCursor ( FALSE ) >= 0 ) ;
              W->Flags |= WN_NOCURSOR;
          }
          else if ( s == 1 )
          {
              while ( ShowCursor ( TRUE ) < 0 ) ;
              W->Flags &= (0xFFFFFFFF - WN_NOCURSOR);
          }
    }
    return MMpush(m,sf) ;
}

