/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////*****************************************************************************************************////////////////
///////////////** 																									**///////////////
//////////////**								~ Zoo Engine (Moteur 3D pour SCOL) ~								 **//////////////
/////////////**											  ~ version 1.0 ~											  **/////////////
/////////////**																										  **/////////////
//////////////**										par Pacme DANHIEZ		  									 **//////////////
///////////////** 																									**///////////////
////////////////*****************************************************************************************************////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///																																  ///
///		FICHIER :	ZooSoft.cpp																								  ///
///																																  ///
///		NATURE	:	Dfinition des principales classes qui dfinissent le mode 3D												  ///
///																																  ///
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////






#include	"..\Basic\ZooSoft.h"



#include	"..\Basic\ZooScene.h"
#include	"..\SCOL\winuser.h"
#include	"..\SCOL\ext.h"
#include	"..\SCOL\objstr.h"
#include	"..\SCOL\macro.h"







int		tabtrame[4]={0x0000,0x8000,0xc000,0x4000};


CELL	*Zbuffer;

short	palette16[TABCOL*64*16];



int fillRenderBuffer(mmachine m, RenderBuffer *buffer, int surf)
{
	PtrObjVoid		OB;
	PtrObjBitmap	B;

	if(surf==NIL)			return -1;

	OB	= (PtrObjVoid)	 MMstart(m,MTOP(surf));
	B	= (PtrObjBitmap) MMstart(m,MTOP(OB->Buffer));

	if(B->bits==NULL)		return -1;

	buffer->Image	= (char*) B->bits;
	buffer->BPL		= B->BPL;
	buffer->width	= B->TailleW;
	buffer->height	= B->TailleH;

	return 0;
}





int Color24To15(int Color)
{
	int		r, g, b;

    r=(Color & 255);
    g=(Color>>8)&255;
    b=(Color>>16)&255;
     
    r>>=3;
    g>>=3;
    b>>=3;
    
	return b + (g<<5) + (r<<10); 
}



int Color24To16(int Color)
{
    int r , g , b ;

     r=(Color & 255);
     g=(Color>>8)&255;
     b=(Color>>16)&255;
     
     r>>=3;
     g>>=3;
     b>>=3;
     return b + ( g << 5 ) + ( r << 10 ) ; 
}




int RGBto15(int r,int g,int b)
{
	return ((((r&255))>>3)<<10)+((((g&255))>>3)<<5)+(((b&255))>>3);
}
























///////////////////////////////////////////////////////////////////////////////////////
///		Render_FLAT																	///
///////////////////////////////////////////////////////////////////////////////////////

void Render_FLAT(short *buffer, CELL *Zbuf, int tailleX, ZFaceSOFT *face, int xa, int xb, float za, float zb, int yscan)
{
	int		i, li, x, dx, dxa, l, dl, dy;
	int		r, g, b;
	float	fl, fdl;
	
	CELL	Zcur;
	float	Zpente;

	
	if(xa!=xb)		Zpente = (zb-za)/(xb-xa); 
	else			Zpente = 0;

	Zcur.f	 = za;
	Zbuf += xa+tailleX;
	buffer	+= xa;

	if (face->color16 > 65535)		return;


	// Remplissage flat
	if(!face->table)
	{

		for(i=xa; i<xb; i++)
		{
			if(Zcur.d > (*Zbuf).d)
			{
				*(buffer)	= face->color16;
				(*Zbuf).f	= Zcur.f;
			}

			Zcur.f += Zpente;
			Zbuf++;
			buffer++;
		}
		return;
	}

	x   = (int) face->xg; 
	dx  = (int) face->xd; 
	dx -= x; 
	dxa = xa - x;

	if (dx==0) return;

	r = (face->color16    ) & 31;
	g = (face->color16>>5 ) & 31;
	b = (face->color16>>10) & 31;

	fdl = (face->lgtd - face->lgtg) * 65536 / dx;
	fl  = face->lgtg*65536+fdl*dxa;
	l   = (int) fl;
	dl  = (int) fdl;
	dy  = (yscan&1) << 1;



	for (i=xa; i<xb; i++)
	{
		if (Zcur.d > (*Zbuf).d)
		{
			li			= ( ((l + tabtrame[dy+(i&1)])>>16)&63 ) << LOGTABCOL;
			*(buffer)	= (face->table)[li+r] + (face->table)[32+li+g] + (face->table)[64+li+b];
			(*Zbuf).f	= Zcur.f;
		}

		Zcur.f += Zpente;
		Zbuf++;
		buffer++;
		l += dl;
	}
}



///////////////////////////////////////////////////////////////////////////////////////
///		Render_TRANSP_NOTEX															///
///////////////////////////////////////////////////////////////////////////////////////

void Render_TRANSP_NOTEX(short *buffer, CELL *Zbuf, int tailleX, ZFaceSOFT *face, int xa, int xb, float za, float zb, int yscan)
{
	int		i;
	short	*pal1;
	short	*pal2;
	
	CELL	Zcur;												
	float	Zpente;							
	if(xa!=xb)		Zpente = (zb-za)/(xb-xa); 
	else			Zpente = 0;

	Zcur.f	 = za;												
	Zbuf += xa+tailleX;
	buffer	+= xa;

	pal1 = face->table + 32;
	pal2 = face->table + 64;

	for(i=xa; i<xb; i++)
	{
		if(Zcur.d > (*Zbuf).d)
		{



			*(buffer) = (face->table) [(*buffer)&31] + pal1[((*buffer)>>5)&31] + pal2[((*buffer)>>10)&31];
		}

		Zcur.f += Zpente;
		Zbuf++;
		buffer++;
	}
}




#define MICRO_SCAN		16
#define LOG_MICRO_SCAN	4



///////////////////////////////////////////////////////////////////////////////////////
///		Macro INIT_TRACE															///
///////////////////////////////////////////////////////////////////////////////////////
#define INIT_TRACE												\
	int		i, x, dx, dxa, dy;									\
	short	*pal0, *pal1, *pal2;								\
	float	fu, fv, fnu, fnv, fdnu, fdnv, fz, fdz, k, fl, fdl;  \
	unsigned short *bitm;										\
	int		w, h, du, dv, u, v, u1, v1, li, l, dl;		        \
	short	z, z0;												\
																\
	CELL	Zcur;												\
	float	Zpente;												\
	if (xa==xb) return;											\
	Zcur.f   = za;	 Zpente = (zb-za)/(xb-xa);					\
	Zbuf += xa + tailleX;										\
	buffer  += xa;												\
																\
	x   = face->xg;												\
	dx  = face->xd;												\
	dx -= x;													\
	dxa = xa-x;													\
																\
	if (dx==0) return;											\
	i	 = face->lgtg;											\
	pal0 = face->table;											\
	pal1 = pal0 + 32;											\
	pal2 = pal0 + 64;											\
																\
	fdnu = (face->ud - face->ug)*256;							\
	fnu	 = dx*face->ug*256 + dxa*fdnu;							\
	fdnv = (face->vd - face->vg)*256;							\
	fnv	 = dx*face->vg*256 + dxa*fdnv;							\
	fdz	 = face->zd - face->zg;									\
	fz	 = dx*face->zg + dxa*fdz;								\
	fdl	 = (face->lgtd - face->lgtg)*65536/dx;					\
	fl	 = face->lgtg*65536 + fdl*dxa;							\
	dl	 = fdl;													\
	l	 = fl;													\
	dy	 = (yscan&1)<<1;										\
																\
	fu	 = fnu/fz;												\
	fv	 = fnv/fz;												\
	u	 = fu;													\
	v	 = fv;													\
	bitm = (unsigned short*) face->bitmap;						\
	z0	 = face->color16;											


///////////////////////////////////////////////////////////////////////////////////////
///		Macro EXACT_TRACE															///
///////////////////////////////////////////////////////////////////////////////////////
#define EXACT_TRACE(blit)										\
	while(xb-xa >= MICRO_SCAN)									\
	{															\
	    fnu += MICRO_SCAN*fdnu;                                 \
		fnv += MICRO_SCAN*fdnv;                                 \
		fz	+= MICRO_SCAN*fdz;                                  \
		k	= 1/fz;												\
		fu	= fnu*k;											\
		fv	= fnv*k;											\
																\
		u1	= fu;												\
		v1	= fv;												\
		du	= (u1-u)>>LOG_MICRO_SCAN;							\
		dv	= (v1-v)>>LOG_MICRO_SCAN;							\
		for(i=0; i<MICRO_SCAN; i++)                             \
		{														\
            li = (((l+tabtrame[dy+(i&1)])>>16)&63)<<LOGTABCOL;	\
			blit                                                \
			u += du;											\
			v += dv;											\
			l += dl;											\
		}														\
		xa	+= MICRO_SCAN;                                      \
	}															\
	dx = xb-xa;													\
	if (dx==0) return;											\
																\
	fnu += dx*fdnu;												\
	fnv += dx*fdnv;												\
	fz	+= dx*fdz;												\
	k	= 1/fz;													\
	fu	= fnu*k;												\
	fv	= fnv*k;												\
																\
	u1 = fu;													\
	v1 = fv;													\
	du = (u1-u)/dx;												\
	dv = (v1-v)/dx;												\
	for(i=0; i<dx; i++)											\
	{															\
		li = (((l+tabtrame[dy+(i&1)])>>16)&63)<<LOGTABCOL;		\
		blit													\
		u += du;                                                \
		v += dv;                                                \
		l += dl;												\
	}



///////////////////////////////////////////////////////////////////////////////////////
///		Macro DISPATCH_TRACE														///
///////////////////////////////////////////////////////////////////////////////////////
#define DISPATCH_TRACE(copy_func)																				\
	w = face->width;																							\
	h = face->height;																							\
	if ((w==1024)&&(h==1024)) { EXACT_TRACE( z=bitm[((v<<4)&0xffc00)+((u>>6)&1023)];      copy_func ) return; }	\
	if ((w==1024)&&(h==512)) { EXACT_TRACE( z=bitm[((v<<3)&0x7fc00)+((u>>6)&1023)];      copy_func ) return; }	\
	if ((w==512)&&(h==1024)) { EXACT_TRACE( z=bitm[((v<<3)&0x7fe00)+((u>>7)&511)];      copy_func ) return; }	\
	if ((w==512)&&(h==512)) { EXACT_TRACE( z=bitm[((v<<2)&0x3fe00)+((u>>7)&511)];      copy_func ) return; }	\
	if ((w==512)&&(h==256)) { EXACT_TRACE( z=bitm[((v<<1)&0x1fe00)+((u>>7)&511)];      copy_func ) return; }	\
	if ((w==256)&&(h==512)) { EXACT_TRACE( z=bitm[((v<<1)&0x1ff00)+((u>>8)&255)];      copy_func ) return; }	\
	if ((w==256)&&(h==256)) { EXACT_TRACE( z=bitm[(v&0xff00)+((u>>8)&255)];      copy_func ) return; }			\
	if ((w==256)&&(h==128)) { EXACT_TRACE( z=bitm[((v&0xfe00)>>1)+((u>>8)&255)]; copy_func ) return; }			\
	if ((w==128)&&(h==256)) { EXACT_TRACE( z=bitm[((v&0xff00)>>1)+((u>>9)&127)]; copy_func ) return; }			\
	if ((w==128)&&(h==128)) { EXACT_TRACE( z=bitm[((v&0xfe00)>>2)+((u>>9)&127)]; copy_func ) return; }			\
	if ((w==128)&&(h==64))  { EXACT_TRACE( z=bitm[((v&0xfc00)>>3)+((u>>9)&127)]; copy_func ) return; }			\
	if ((w==64) &&(h==128)) { EXACT_TRACE( z=bitm[((v&0xfe00)>>3)+((u>>10)&63)]; copy_func ) return; }			\
	if ((w==64) &&(h==64))  { EXACT_TRACE( z=bitm[((v&0xfc00)>>4)+((u>>10)&63)]; copy_func ) return; }			\
	if ((w==64) &&(h==32))  { EXACT_TRACE( z=bitm[((v&0xf800)>>5)+((u>>10)&63)]; copy_func ) return; }			\
	if ((w==32) &&(h==64))  { EXACT_TRACE( z=bitm[((v&0xfc00)>>5)+((u>>11)&31)]; copy_func ) return; }			\
	if ((w==32) &&(h==32))  { EXACT_TRACE( z=bitm[((v&0xf800)>>6)+((u>>11)&31)]; copy_func ) return; }



///////////////////////////////////////////////////////////////////////////////////////
///		M3DtraceExactL																///
///////////////////////////////////////////////////////////////////////////////////////

void Render_TEX(short *buffer, CELL *Zbuf, int tailleX, ZFaceSOFT *face, int xa, int xb, float za, float zb, int yscan)
{

	INIT_TRACE
	DISPATCH_TRACE
	(

	if (Zcur.d > (*Zbuf).d)
		{
			*(buffer)		= pal0[li+(z&31)] + pal1[li+((z>>5)&31)] + pal2[li+((z>>10)&31)];
			(*Zbuf).f	= Zcur.f;
		}

		Zcur.f += Zpente;
		Zbuf++;
		buffer++;
	)
	
}



///////////////////////////////////////////////////////////////////////////////////////
///		Render_TRANSP_TEX_1															///
///////////////////////////////////////////////////////////////////////////////////////
void Render_TRANSP_TEX_1(short *buffer, CELL *Zbuf, int tailleX, ZFaceSOFT *face, int xa, int xb, float za, float zb, int yscan)
{

	INIT_TRACE
	DISPATCH_TRACE
	(	
		if (z!=z0) 
		{
			if (Zcur.d > (*Zbuf).d)
			{
				*(buffer)		= pal0[li+(z&31)] + pal1[li+((z>>5)&31)] + pal2[li+((z>>10)&31)];
				(*Zbuf).f	= Zcur.f;
			}

			Zcur.f += Zpente;
			Zbuf++;
			buffer++;
		}
		else
		{
			Zcur.f += Zpente;
			Zbuf++;
			buffer++;
		}
	)
}



///////////////////////////////////////////////////////////////////////////////////////
///		M3DtraceExactTranspLA														///
///////////////////////////////////////////////////////////////////////////////////////
void Render_TRANSP_TEX_2(short *buffer, CELL *Zbuf, int tailleX, ZFaceSOFT *face, int xa, int xb, float za, float zb, int yscan)
{

	INIT_TRACE
  
	DISPATCH_TRACE 
	(	
		if (z!=z0) 
		{
			if (Zcur.d > (*Zbuf).d)
			{
				*(buffer)		= (((*buffer)>>1)&0x3def) + (((pal0[li+(z&31)] + pal1[li+((z>>5)&31)] + pal2[li+((z>>10)&31)])>>1)&0x3def);
			}

			Zcur.f += Zpente;
			Zbuf++;
			buffer++;
		}
		else
		{
			Zcur.f += Zpente;
			Zbuf++;
			buffer++;
		}
	)
}



///////////////////////////////////////////////////////////////////////////////////////
///		M3Dline																		///
///////////////////////////////////////////////////////////////////////////////////////
/*
int M3Dline(renderbuffer rb,float x1,float y1,float x2,float y2,int col)
{
	float xclip,yclip;
	int ax,ay,bx,by,dx,dy,kx,ky,k,i;
	short *p;

	xclip=-rb->xorigin+1;
	if (x1<xclip)
	{
		if (x2<xclip) return 0;
		y1+=(xclip-x1)*(y2-y1)/(x2-x1);
		x1=xclip;
	}
	if (x2<xclip)
	{
		if (x1<xclip) return 0;
		y2+=(xclip-x2)*(y1-y2)/(x1-x2);
		x2=xclip;
	}
	xclip=rb->nbcolumn-rb->xorigin-1;
	if (x1>xclip)
	{
		if (x2>xclip) return 0;
		y1+=(xclip-x1)*(y2-y1)/(x2-x1);
		x1=xclip;
	}
	if (x2>xclip)
	{
		if (x1>xclip) return 0;
		y2+=(xclip-x2)*(y1-y2)/(x1-x2);
		x2=xclip;
	}
	yclip=rb->yorigin-rb->nblines+1;
	if (y1<yclip)
	{
		if (y2<yclip) return 0;
		x1+=(yclip-y1)*(x2-x1)/(y2-y1);
		y1=yclip;
	}
	if (y2<yclip)
	{
		if (y1<yclip) return 0;
		x2+=(yclip-y2)*(x1-x2)/(y1-y2);
		y2=yclip;
	}

	yclip=rb->yorigin-1;
	if (y1>yclip)
	{
		if (y2>yclip) return 0;
		x1+=(yclip-y1)*(x2-x1)/(y2-y1);
		y1=yclip;
	}
	if (y2>yclip)
	{
		if (y1>yclip) return 0;
		x2+=(yclip-y2)*(x1-x2)/(y1-y2);
		y2=yclip;
	}
	
	dx=abs(x2-x1); dy=abs(y2-y1);
	if (dx>=dy)
	{
		if (x2>x1) { ax=x1; ay=y1; bx=x2; by=y2;}
		else { ax=x2; ay=y2; bx=x1; by=y1;}
		kx=bx-ax; ky=by-ay;
		dx=1;dy=rb->nextline>>1;
		if (ky>0) dy=-dy; else ky=-ky;
		p=(short*)rb->start;
		p+=(rb->xorigin+ax)+((rb->yorigin-ay)*(rb->nextline>>1));
		k=kx>>1;
		for(i=ax;i<=bx;i++)
		{
			*p=col;
			k+=ky;
			if (k>kx)
			{
				k-=kx;
				p+=dy;
			}
			p+=dx;
		}
	}
	else
	{
		if (y2>y1) { ax=x1; ay=y1; bx=x2; by=y2;}
		else { ax=x2; ay=y2; bx=x1; by=y1;}
		kx=bx-ax; ky=by-ay;
		dx=1;dy=-rb->nextline>>1;
		if (kx<0) { dx=-dx; kx=-kx;}
		p=(short*)rb->start;
		p+=(rb->xorigin+ax)+((rb->yorigin-ay)*(rb->nextline>>1));
		k=ky>>1;
		for(i=ay;i<=by;i++)
		{
			*p=col;
			k+=kx;
			if (k>ky)
			{
				k-=ky;
				p+=dx;
			}
			p+=dy;
		}
	}
	return 0;
}
*/

int M3Dline(RenderBuffer *rendbuf, float x1, float y1, float x2, float y2, int col)
{
	float	xclip, yclip;
	int		ax, ay, bx, by, dx, dy, kx, ky, k, i;
	short	*buffer;




	x1 += rendbuf->width>>1;
	x2 += rendbuf->width>>1;

	// Clipping 2D //
	xclip = (float) (1);
	if (x1 < xclip)
	{
		if (x2 < xclip) return 0;
		y1 += (xclip-x1) * (y2-y1)/(x2-x1);
		x1  = xclip;
	}
	if (x2 < xclip)
	{
		if (x1 < xclip) return 0;
		y2 += (xclip-x2) * (y1-y2)/(x1-x2);
		x2  = xclip;
	}

	xclip = (float) (rendbuf->width - 1);
	if (x1 > xclip)
	{
		if (x2 > xclip) return 0;
		y1 += (xclip-x1) * (y2-y1)/(x2-x1);
		x1  = xclip;
	}
	if (x2 > xclip)
	{
		if (x1 > xclip) return 0;
		y2 += (xclip-x2) * (y1-y2)/(x1-x2);
		x2  = xclip;
	}

//	yclip = (float) (- rendbuf->height + 1);
	yclip = (float) (1);
	if (y1 < yclip)
	{
		if (y2 < yclip) return 0;
		x1 += (yclip-y1) * (x2-x1)/(y2-y1);
		y1  = yclip;
	}
	if (y2 < yclip)
	{
		if (y1 < yclip) return 0;
		x2 += (yclip-y2) * (x1-x2)/(y1-y2);
		y2  = yclip;
	}

//	yclip = (float) (- 1);
	yclip = (float) (rendbuf->height - 1);
	if (y1 > yclip)
	{
		if (y2 > yclip) return 0;
		x1 += (yclip-y1) * (x2-x1)/(y2-y1);
		y1  = yclip;
	}
	if (y2 > yclip)
	{
		if (y1 > yclip) return 0;
		x2 += (yclip-y2) * (x1-x2)/(y1-y2);
		y2  = yclip;
	}

	// Trac de la ligne //
	dx = abs((int) (x2-x1));
	dy = abs((int) (y2-y1));
	if (dx >= dy)
	{
		if (x2>x1)
		{
			ax = (int) x1; 
			ay = (int) y1; 
			bx = (int) x2; 
			by = (int) y2;
		}
		else
		{
			ax = (int) x2; 
			ay = (int) y2; 
			bx = (int) x1; 
			by = (int) y1;
		}

		kx = bx - ax; //  +
		ky = by - ay; // +/-
		dx = 1;
		dy = rendbuf->BPL >> 1;

		if (ky>0)	dy = -dy; 
		else		ky = -ky;
		buffer  = (short*) rendbuf->Image;
//		buffer += ax + ((-ay)*(rendbuf->BPL>>1));
		buffer += ax + ((rendbuf->height-ay)*(rendbuf->BPL>>1));
		k		= kx>>1;

		for(i=ax; i<=bx; i++)
		{
			*buffer = col;
			k	   += ky;
			if (k > kx)
			{
				k	   -= kx;
				buffer += dy;
			}
			buffer += dx;
		}
	}
	else
	{
		if (y2 > y1) 
		{ 
			ax = (int) x1; 
			ay = (int) y1; 
			bx = (int) x2; 
			by = (int) y2;
		}
		else 
		{ 
			ax = (int) x2; 
			ay = (int) y2; 
			bx = (int) x1; 
			by = (int) y1;
		}

		kx = bx - ax; 
		ky = by - ay;
		dx = 1;
		dy = - rendbuf->BPL>>1;

		if (kx<0) 
		{ 
			dx = -dx; 
			kx = -kx;
		}

		buffer  = (short*) rendbuf->Image;
//		buffer += ax + ((-ay)*(rendbuf->BPL>>1));
		buffer += ax + ((rendbuf->height-ay)*(rendbuf->BPL>>1));
		k		= ky>>1;
		for(i=ay; i<=by; i++)
		{
			*buffer = col;
			k += kx;
			if (k > ky)
			{
				k -= ky;
				buffer += dx;
			}
			buffer += dy;
		}
	}
	return 0;
}







int lastFogColor = -1;


///////////////////////////////////////////////////////////////////////////////////////
///		InitFogPalette																///
///////////////////////////////////////////////////////////////////////////////////////
///
///		Initialise la palette de FOG courant -- 15 bits par couleur
///
int InitFogPalette(int color)
{
	int		l, i, x, r, g, b, k, lp, i0, i1;

	int		b_mask  = 0x1f;
	int		g_mask  = 0x3e0;
	int		r_mask  = 0x7c00;
	int		b_shift = 4;
	int		g_shift = 4;
	int		r_shift = 4;

	if(color == lastFogColor)		return 0;

	r = color & r_mask;
	g = color & g_mask;
	b = color & b_mask;


	for(l=1; l<16; l++)
	{
		lp = 16-l;
		i0 = l*TABCOL*64;

		for(i=0; i<64; i++)
		{
			i1 = i<<LOGTABCOL;
			for(k=0; k<32; k++)
			{
				x = palette16[i1+k];
				x = ((x*lp+b*l) >> b_shift) & b_mask;
				palette16[i0+i1+k]	  = x;

				x = palette16[i1+32+k];
				x = ((x*lp+g*l) >> g_shift) & g_mask;
				palette16[i0+i1+32+k] = x;

				x = palette16[i1+64+k];
				x = ((x*lp+r*l) >> r_shift) & r_mask;
				palette16[i0+i1+64+k] = x;

			}
		}
	}

	lastFogColor = color;

	return 0;
}



///////////////////////////////////////////////////////////////////////////////////////
///		InitPaletteSOFT																///
///////////////////////////////////////////////////////////////////////////////////////
///
///		Initialise la palette utilise pour le FOG -- 15 bits par couleur
///
int	InitPaletteSOFT()
{
	int l, i, j;
	int g_shift = 5;
	int r_shift = 10;


	for(l=0; l<64; l++)
	{

		for(i=0; i<32; i++)
		{
			if (i)		j = i+ (((l-31)*128)>>8);
			else		j = 0;
			
			if (j>31)	j = 31;
			if (j<0)	j = 0;


		palette16[(l<<LOGTABCOL)+i]		= j;
			palette16[(l<<LOGTABCOL)+32+i]	= j << g_shift;
			palette16[(l<<LOGTABCOL)+64+i]	= j << r_shift;
		}
	}

	InitFogPalette(0xffff);
	
	return 0;
}
