/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////*****************************************************************************************************////////////////
///////////////** 																									**///////////////
//////////////**								~ Zoo Engine (Moteur 3D pour SCOL) ~								 **//////////////
/////////////**											  ~ version 1.0 ~											  **/////////////
/////////////**																										  **/////////////
//////////////**										par Pacme DANHIEZ		  									 **//////////////
///////////////** 																									**///////////////
////////////////*****************************************************************************************************////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///																																  ///
///		FICHIER :	ZooWorld.h																									  ///
///																																  ///
///		NATURE	:	Dfinition des principales classes qui dfinissent le mode 3D												  ///
///																																  ///
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////








#ifndef __ZOOWORLD_H__
#define __ZOOWORLD_H__



#include	"..\Basic\ZooStd.h"

#include	"..\Scene Graph\ZooObject.h"
#include	"..\Scene Graph\ZooLight.h"
#include	"..\Scene Graph\ZooCamera.h"
#include	"..\Scene Graph\ZooShell.h"
#include	"..\Scene Graph\ZooColl.h"
#include	"..\Scene Graph\ZooSprite.h"
#include	"..\Scene Graph\ZooAnim.h"
#include	"..\Scene Graph\ZooParticle.h"
#include	"..\Scene Graph\ZooSound.h"

#include	"..\Basic\ZooScene.h"



float GetTimePassed(DWORD *lastTime);




///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// ZFaceProj Class
///
///		- Classe des faces transparentes
///		- stockage en liste chaine pour le tri en Z
///		- champs ncessaire pour l'affichage : donnes sur la face, et sur les vertices correspondants
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct ZFaceProj
{
	ZFace		*face;							// face correspondant  ce polygone

	ZVector3	vert3D[3];						// coordonnes des 3 vertices, dj transformes (tires de VertTransf dans ZObject)
	float		Zproj[3];						// coordonnes Z des trois vertices projets  l'cran
	
	ZVector3	*colorVerts[3];					// couleurs des vertices

	ZVector3	normales[3];					// normales aux vertices
	ZVector3	normFace;

	float		Zmin;

	int			type;							// 0 = objet	;	1 = particule	;	2 = sprite

	ZVector4	pclColor;

	ZSprite		*sprite;
};




struct ZVertSOFT
{
	float		posX;
	int			posY;
	float		posZ;

	float		u, v;

	float		light;
};



struct ZFaceSOFT
{
	ZFaceSOFT	*next;
	ZFaceSOFT	*prev;

	ZVertSOFT	vert[4];
	
	ZFace		*face;							// face correspondant  ce polygone

	int			Npoints;

	int			ymin, ymax;

	short		pointG, pointD;


	float		xg, zg, dxg, dzg;
	float		xd, zd, dxd, dzd;

	float		ug, vg, dug, dvg;
	float		ud, vd, dud, dvd;

	float		lgtg, dlgtg;
	float		lgtd, dlgtd;


	bool		active;


	int			color16;
	bool		transp;
	short		*table;

	char		*bitmap;
	int			width;
	int			height;

	void		*render;
};






#include	"ZooSoft.h"





///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// ZWorld Class
///
///		- Classe du World
///		- listes globales et linaires des objets, lights et cameras (parcours rapide)
///		- liste des polygones transparents du monde
///		- methodes de creation de liste et de gestion globale du monde
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class ZWorld : public ZNode
{

protected:

	///////////////////////////////////////////////////////////////////////////
	/// createNodeListsRecurs()												///
	///////////////////////////////////////////////////////////////////////////
	void createNodeListsRecurs(ZNode *node);


public:

	ZArray<ZObject*>	objList;
	ZArray<ZLight*>		lghList;
	ZArray<ZCamera*>	camList;
	ZArray<ZShell*>		shlList;
	ZArray<ZEmitter*>	pclList;
	ZArray<ZSound*>		sndList;
	ZArray<ZSprite*>	sprList;


	vector<int>			tempQuickSort;

	vector<ZFaceProj>	transpList;
	vector<ZFaceProj*>	transpListSorted;

	vector<ZFaceSOFT>	SOFTList;
	vector<ZFaceSOFT*>	SOFTListSorted;
	ZFaceSOFT			*base;

	ZVector3 			defaultLightDir;		// spcifique au mode SOFT

	ZScene				*scene3D;

	///////////////////////////////////////////////////////////////////////////
	/// Constructor	& Destructor											///
	///////////////////////////////////////////////////////////////////////////
	ZWorld(ZScene *scene);
	~ZWorld();


	///////////////////////////////////////////////////////////////////////////
	/// Free the entire world												///
	///////////////////////////////////////////////////////////////////////////
	void FreeWorld();


	///////////////////////////////////////////////////////////////////////////
	/// createNodeLists()													///
	///////////////////////////////////////////////////////////////////////////
	void createNodeLists(ZNode* baseNode);


	///////////////////////////////////////////////////////////////////////////
	/// PutLightSettings()													///
	///////////////////////////////////////////////////////////////////////////
	void PutLightSettings(const ZMatrix &matCam);



	///////////////////////////////////////////////////////////////////////////
	/// UpdateSounds()														///
	///////////////////////////////////////////////////////////////////////////
	void UpdateSounds(ZSound *listener);



	void ViewFrustrumCull(ZCamera *curCam, float tailleX, float tailleY, bool ortho);


	///////////////////////////////////////////////////////////////////////////
	/// RenderObjectPass1 & RenderObjectPass2								///
	///////////////////////////////////////////////////////////////////////////
	void RenderObjectPass1();
	void RenderObjectPass2();
	void RenderObjectPass3();
	void RenderObjectPass4();
	
	void RenderObjectPassTransp(ZCamera *curCam);

	void RenderParticles(const ZVector3 &RealAngleCam, const ZMatrix &RealMatCam, const ZMatrix &matCam);
	void RenderAdditiveParticles(const ZVector3 &RealAngleCam, const ZMatrix &matCam);
	void RenderSpritesPASS1(float camRotationZ);
	void RenderSpritesPASS2(float camRotationZ);
	void RenderSpritesPASS3(float camRotationZ);


	///////////////////////////////////////////////////////////////////////////
	/// RenderObjectWire													///
	///////////////////////////////////////////////////////////////////////////
	void RenderObjectWired(float lineWidth);


	///////////////////////////////////////////////////////////////////////////
	/// ComputeMatrices()													///
	///////////////////////////////////////////////////////////////////////////
	void RecursComputeMatrices(ZNodeGraph* activeCam, ZNodeGraph*  baseNode, const ZMatrix &refMat, float refScale, bool rangeOK);
	void ComputeMatrices(ZNodeGraph* activeCam, ZNodeGraph*  baseNode, const ZMatrix &refMat, float refScale, bool rangeOK);


	///////////////////////////////////////////////////////////////////////////
	/// TransformVertices()													///
	///////////////////////////////////////////////////////////////////////////
	void TransformVertices();
	void ComputeNormals(bool accel);
	
	


	///////////////////////////////////////////////////////////////////////////
	/// SelectionTest()														///
	///////////////////////////////////////////////////////////////////////////
	void SelectionTest(ZMatrix &camMat, const ZVector3 &orig, const ZVector3 &ray, float* dist, float distMin, float* u, float* v, float* z, ZObject**	result, int* faceNb);


	void SelectFaces(ZCamera *curCam, bool wired, bool ortho);
	
	void RenderSOFT(ZCamera *cam, RenderBuffer *buffer, short fond16, bool clear);
	void RenderSOFT_Wired(ZCamera *cam, RenderBuffer *buffer, short fond16, bool clear);

	void RenderlineSOFT(RenderBuffer *rendBuf, ZCamera *cam, int yscan, int traceHeight);
	void ComputeLightTarget(const ZMatrix &matCam);
	float Calclight(ZVector3 *pos, ZVector3 *norm);

	void UpdateFace(ZFaceSOFT *face, int ytab);

	
//	void DetectCollision();


};



///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderObjectPass1																	///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderObjectPass1()
{
	ZObject	*curObj;

	for(int i=0; i<objList.Size(); i++)
	{
		curObj = objList[i];

		if(curObj->visible)
		{
			curObj->RenderPass1();							// and finally render the object
		}
	}
}



///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderObjectPass2																	///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderObjectPass2()
{
	ZObject	*curObj;
	
	for(int i=0; i<objList.Size(); i++)
	{
		curObj = objList[i];
		
		if(curObj->visible)
		{
			curObj->RenderPass2();							// and finally render the object
		}
	}
}



///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderObjectPass3																	///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderObjectPass3()
{
	ZObject	*curObj;
	
	for(int i=0; i<objList.Size(); i++)
	{
		curObj = objList[i];
		
		if(curObj->visible)
		{
			curObj->RenderPass3();							// and finally render the object
		}
	}
}



///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderObjectPass3																	///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderObjectPass4()
{
	ZObject	*curObj;
	
	for(int i=0; i<objList.Size(); i++)
	{
		curObj = objList[i];
		
		if(curObj->visible)
		{
			curObj->RenderPass4();							// and finally render the object
		}
	}
}



///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderParticles																		///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderParticles(const ZVector3 &RealAngleCam, const ZMatrix &RealMatCam, const ZMatrix &matCam)
{
	ZEmitter	*curPcl;


	for(int i=0; i<pclList.Size(); i++)
	{
		curPcl	= pclList[i];
		curPcl->NextStep();

		if(curPcl->rangeOK)
			curPcl->Render(RealAngleCam, matCam);
	}
}


///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderAdditiveParticles																///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderAdditiveParticles(const ZVector3 &RealAngleCam, const ZMatrix &matCam)
{
	ZEmitter	*curPcl;


	for(int i=0; i<pclList.Size(); i++)
	{
		curPcl	= pclList[i];

		if( curPcl->rangeOK && (curPcl->mask&PCL_ADDITIVE) )
			curPcl->RenderAdditive(RealAngleCam, matCam);
	}
}



///////////////////////////////////////////////////////////////////////////////////////////////
///		RenderObjectWired																	///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::RenderObjectWired(float lineWidth)
{
	ZObject	*curObj;
	
	glLoadIdentity();

	for(int i=0; i<objList.Size(); i++)
	{
		curObj = objList[i];
		
		if(curObj->visible)
		{
			curObj->RenderWired(lineWidth);					// and finally render the object
		}
	}
}







///////////////////////////////////////////////////////////////////////////////////////////////
///		ComputeLightTarget																	///
///////////////////////////////////////////////////////////////////////////////////////////////
inline void ZWorld::ComputeLightTarget(const ZMatrix &matCam)
{
	if(lghList.Size()==0)
	{
		defaultLightDir.SetCoord(matCam._13, matCam._23, matCam._33);
		defaultLightDir.Normalize();
	}
	else
	{
		for(int i=0; i<lghList.Size(); i++)
		{
			lghList[i]->target.SetCoord(lghList[i]->worldMat._13, lghList[i]->worldMat._23, lghList[i]->worldMat._33);
			lghList[i]->target.Normalize();
		}
	}
}





///////////////////////////////////////////////////////////////////////////
///		UpdateSounds													///
///////////////////////////////////////////////////////////////////////////
inline void ZWorld::UpdateSounds(ZSound *pZSound)
{
	if( !pZSound ) return ;

	ZSound	*curSound;

	for(int i=0; i<sndList.Size(); i++)
	{
		curSound = sndList[i];
		curSound->update( pZSound );
	}
}










bool TestObject(ZMatrix &camMat, ZObject* curObj, const ZVector3 &orig, const ZVector3 &ray, float* dist, float distMin, float* u, float* v, float* z, int* faceNb);




#endif











