/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////*****************************************************************************************************////////////////
///////////////** 																									**///////////////
//////////////**								~ Zoo Engine (Moteur 3D pour SCOL) ~								 **//////////////
/////////////**											  ~ version 1.0 ~											  **/////////////
/////////////**																										  **/////////////
//////////////**										par Pacme DANHIEZ		  									 **//////////////
///////////////** 																									**///////////////
////////////////*****************************************************************************************************////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///																																  ///
///		FICHIER :	ZooWorld.h																									  ///
///																																  ///
///		NATURE	:	Dfinition des principales classes qui dfinissent le mode 3D												  ///
///																																  ///
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////








#ifndef __ZOOCAMERA_H__
#define __ZOOCAMERA_H__



#include	"..\Basic\ZooStd.h"
#include	"..\Scene Graph\ZooNodeGraph.h"









///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// ZCamera Class
///
///		- Classe des camras
///		- target = axe de vise (normal), ou angles de vise (VizThta, VizPhi)
///		- Field of vision en "degrs" !!!
///		- Znear, Zfar
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class ZCamera : public ZNodeGraph
{

public:
	
	float		fov;							// field of view
	float		Znear, Zfar;					// Znear and Zfar for clipping
	float		Zfog;

	ZMatrix		cameraMatx;						// camera matrix : rotation, translation
	ZMatrix		visionMatx;						// vision matrix : perspective

	int			width, height;
	int			dx, dy;

	ZVector3	RealWorldAngles;
	ZVector3	RealWorldPos;
	ZMatrix		RealWorldMat;

	
	///////////////////////////////////////////////////////////////////////////
	/// Constructor	& Destructor		      								///
	///////////////////////////////////////////////////////////////////////////
	ZCamera(int s3d);
	~ZCamera();


	///////////////////////////////////////////////////////////////////////////
	/// Set the components of the camera      								///
	///////////////////////////////////////////////////////////////////////////
	void SetCamera(float nfov, float nZnear, float nZfar);


	///////////////////////////////////////////////////////////////////////////
	/// Compute the matrix of camera space    								///
	///////////////////////////////////////////////////////////////////////////
	void ComputeMatrix();
	void ComputeMatrixOrtho();
};




///////////////////////////////////////////////////////////////////////////
/// Set the components of the camera      								///
///////////////////////////////////////////////////////////////////////////
inline void ZCamera::SetCamera(float nfov, float nZnear, float nZfar)
{
	fov			= nfov;
	Znear		= nZnear;
	Zfar		= nZfar;
}






#endif